@@ -284,12 +284,16 @@ class basic_ast_visitor {
 
   template <typename N>
   static void end_visit(N& node, Args... args) {
-    // TODO(afuller): Replace with c++17 folding syntax when available.
-    auto cb = [&](auto& arg) {
-      return make_scope_guard([&] { ast_detail::end_visit(node, arg); });
+    auto make_func = [&](auto& arg) {
+      return std::function<void()>([&] { ast_detail::end_visit(node, arg); });
     };
-    using _ = int[];
-    void(_{0, (cb(args), 0)...});
+    (void)make_func; // Suppress the dead store warning for zero args.
+    constexpr size_t num_args = sizeof...(args);
+    std::function<void()> funcs[num_args > 0 ? num_args : 1] = {
+        make_func(args)...};
+    for (size_t i = num_args; i > 0; --i) {
+      funcs[i - 1]();
+    }
   }
 
   template <typename C>
@@ -313,6 +317,44 @@ class basic_ast_visitor {
   }
 };
 
+enum class context_type {
+  // States of removal.
+  legacy = -2,
+  deprecated = -1,
+
+  // States of release.
+  released = 0, // The default context_type.
+  beta = 1,
+  experimental = 2,
+
+  // Other ~orthogonal context types.
+  testing,
+  no_legacy,
+  no_deprecated,
+};
+
+constexpr inline const char* getContextTypeUri(context_type type) {
+  switch (type) {
+    case context_type::legacy:
+      return "facebook.com/thrift/annotation/Legacy";
+    case context_type::deprecated:
+      return "facebook.com/thrift/annotation/Deprecated";
+    case context_type::beta:
+      return "facebook.com/thrift/annotation/Beta";
+    case context_type::experimental:
+      return "facebook.com/thrift/annotation/Experimental";
+    case context_type::testing:
+      return "facebook.com/thrift/annotation/Testing";
+    case context_type::no_legacy:
+      return "facebook.com/thrift/annotation/NoLegacy";
+    case context_type::no_deprecated:
+      return "facebook.com/thrift/annotation/NoDeprecated";
+    case context_type::released:
+      break;
+  }
+  return "";
+}
+
 template <bool is_const, typename N = t_node>
 class basic_visitor_context {
   using node_type = ast_detail::node_type<is_const, N>;
@@ -321,6 +363,21 @@ class basic_visitor_context {
  public:
   bool visiting() const noexcept { return !context_.empty(); }
 
+  bool has(context_type type) const {
+    assert(type != context_type::released);
+    // Get the uri for the associated annotation.
+    const char* uri = getContextTypeUri(type);
+    // Search for the context annotation.
+    for (auto itr = context_.rbegin(); itr != context_.rend(); ++itr) {
+      if (auto* named = dynamic_cast<const t_named*>(*itr)) {
+        if (named->find_structured_annotation_or_null(uri)) {
+          return true;
+        }
+      }
+    }
+    return false;
+  }
+
   // The first node visited.
   node_type* root() const noexcept {
     assert(!context_.empty());
