@@ -16,20 +16,20 @@
 
 #pragma once
 
-#include <functional>
-#include <string>
+#include <map>
 #include <type_traits>
 #include <typeindex>
 #include <utility>
 
 #include <thrift/compiler/ast/ast_visitor.h>
-#include <thrift/compiler/ast/t_node.h>
 #include <thrift/compiler/diagnostic.h>
 
 namespace apache {
 namespace thrift {
 namespace compiler {
 
+class t_node;
+
 // A cache for metadata associated with an AST node.
 //
 // Useful for avoiding re-computation during a traversal of an AST.
@@ -93,17 +93,12 @@ class node_metadata_cache {
 // A context aware reporter for diagnostic results.
 class diagnostic_context : public diagnostics_engine,
                            public const_visitor_context {
- private:
-  template <typename With>
-  using if_with =
-      decltype(std::declval<With&>()(std::declval<std::ostream&>()));
-
  public:
   using diagnostics_engine::diagnostics_engine;
 
   static diagnostic_context ignore_all(source_manager& sm) {
     return diagnostic_context(
-        sm, [](const diagnostic&) {}, diagnostic_params::only_failures());
+        sm, [](const diagnostic&) {}, diagnostic_params::only_errors());
   }
 
   // A cache for traversal-specific metadata.
@@ -115,6 +110,12 @@ class diagnostic_context : public diagnostics_engine,
     warning(*current(), msg, std::forward<T>(args)...);
   }
 
+  using diagnostics_engine::error;
+  template <typename... T>
+  void error(fmt::format_string<T...> msg, T&&... args) {
+    error(*current(), msg, std::forward<T>(args)...);
+  }
+
   using diagnostics_engine::check;
   template <typename... T>
   bool check(bool condition, fmt::format_string<T...> msg, T&&... args) {
@@ -123,91 +124,6 @@ class diagnostic_context : public diagnostics_engine,
 
   using diagnostics_engine::report;
 
-  // TODO(afuller): Remove all deprecated overloads below.
-  void report(
-      diagnostic_level level,
-      int lineno,
-      std::string text,
-      std::string name = "") {
-    report({level, std::move(text), program().path(), lineno, std::move(name)});
-  }
-
-  template <typename With, typename = if_with<With>>
-  void report(
-      diagnostic_level level, int lineno, std::string name, With&& with) {
-    std::ostringstream o;
-    with(static_cast<std::ostream&>(o));
-    report({level, o.str(), program().path(), lineno, std::move(name)});
-  }
-
-  template <typename With, typename = if_with<With>>
-  void report(diagnostic_level level, int lineno, With&& with) {
-    report(level, lineno, {}, std::forward<With>(with));
-  }
-
-  void report(diagnostic_level level, const t_node& node, std::string msg) {
-    report(level, node.lineno(), std::move(msg));
-  }
-
-  template <typename With, typename = if_with<With>>
-  void report(diagnostic_level level, const t_node& node, With&& with) {
-    report(level, node.lineno(), std::forward<With>(with));
-  }
-
-  void report(
-      diagnostic_level level,
-      std::string name,
-      const t_node& node,
-      std::string msg) {
-    report(level, node.lineno(), std::move(msg), std::move(name));
-  }
-
-  template <typename With>
-  void report(
-      diagnostic_level level,
-      std::string name,
-      const t_node& node,
-      std::string msg,
-      With&& with) {
-    report(
-        level,
-        node.lineno(),
-        {},
-        std::move(msg),
-        std::move(name),
-        std::forward<With>(with));
-  }
-
-  // TODO(ytj): use path in t_node and delete this function overload
-  template <typename With>
-  void report(
-      diagnostic_level level,
-      const t_node& node,
-      std::string path,
-      With&& with) {
-    std::ostringstream o;
-    with(static_cast<std::ostream&>(o));
-    report({level, o.str(), std::move(path), node.lineno(), {}});
-  }
-
-  template <typename With>
-  void report(diagnostic_level level, With&& with) {
-    assert(current() != nullptr);
-    report(level, current()->lineno(), std::forward<With>(with));
-  }
-
-  template <typename... Args>
-  void failure(Args&&... args) {
-    report(diagnostic_level::failure, std::forward<Args>(args)...);
-  }
-  template <typename... Args>
-  bool failure_if(bool cond, Args&&... args) {
-    if (cond) {
-      report(diagnostic_level::failure, std::forward<Args>(args)...);
-    }
-    return cond;
-  }
-
  private:
   node_metadata_cache cache_;
 };
