@@ -1,11 +1,11 @@
 /*
- * Copyright 2018-present Facebook, Inc.
+ * Copyright (c) Meta Platforms, Inc. and affiliates.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
  * You may obtain a copy of the License at
  *
- *   http://www.apache.org/licenses/LICENSE-2.0
+ *     http://www.apache.org/licenses/LICENSE-2.0
  *
  * Unless required by applicable law or agreed to in writing, software
  * distributed under the License is distributed on an "AS IS" BASIS,
@@ -13,8 +13,11 @@
  * See the License for the specific language governing permissions and
  * limitations under the License.
  */
+
 #pragma once
 
+#include <thrift/compiler/ast/t_node.h>
+
 namespace apache {
 namespace thrift {
 namespace compiler {
@@ -24,13 +27,11 @@ class t_program;
 /**
  * This node models an include statement in the Thrift program.
  */
-class t_include : public t_doc {
+class t_include : public t_node {
  public:
   explicit t_include(t_program* program) : program_(program) {}
 
-  t_program* get_program() const {
-    return program_;
-  }
+  t_program* get_program() const { return program_; }
 
  private:
   t_program* program_;
