@@ -77,13 +77,33 @@ class t_program : public t_named {
       : t_program(std::move(path), std::make_shared<t_scope>()) {}
 
   void set_package(t_package package) { package_ = std::move(package); }
-  const t_package package() const { return package_; }
+  const t_package& package() const { return package_; }
 
   // Defintions, in the order they were added.
   node_list_view<t_named> definitions() { return definitions_; }
   node_list_view<const t_named> definitions() const { return definitions_; }
   void add_definition(std::unique_ptr<t_named> definition);
 
+  // A convience function that:
+  //  - optionally sets the uri (overriding any set value or
+  // inheritted default),
+  //  - adds the definition to the program, and
+  //  - returns a mutable reference to the stored value, so it can be
+  //  additionally configured.
+  template <typename T>
+  T& add_def(std::unique_ptr<T> definition, fmt::string_view uri = {}) {
+    auto* ptr = definition.get();
+    if (uri.data() != nullptr) {
+      definition->set_uri(std::string(uri.data(), uri.size()));
+    }
+    add_definition(std::move(definition));
+    return *ptr;
+  }
+  template <typename T, typename... Args>
+  T& create_def(Args&&... args) {
+    return add_def(std::make_unique<T>(std::forward<Args>(args)...));
+  }
+
   // Concrete instantiation of a templated type.
   node_list_view<t_templated_type> type_instantiations() { return type_insts_; }
   node_list_view<const t_templated_type> type_instantiations() const {
@@ -157,7 +177,7 @@ class t_program : public t_named {
    */
   std::vector<t_program*> get_included_programs() const {
     std::vector<t_program*> included_programs;
-    for (auto const& include : includes_) {
+    for (const auto& include : includes_) {
       included_programs.push_back(include->get_program());
     }
     return included_programs;
@@ -181,9 +201,14 @@ class t_program : public t_named {
    */
   const std::string& get_namespace(const std::string& language) const;
 
+  struct namespace_config {
+    bool no_top_level_domain = false;
+    bool no_domain = false;
+    bool no_filename = false;
+  };
+
   std::vector<std::string> gen_namespace_or_default(
-      const std::string& language,
-      std::function<std::vector<std::string>()> gen) const;
+      const std::string& language, namespace_config config) const;
 
   /**
    * This creates a new program for every thrift file in an
