@@ -1,5 +1,5 @@
 /*
- * Copyright (c) Facebook, Inc. and its affiliates.
+ * Copyright (c) Meta Platforms, Inc. and affiliates.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -41,8 +41,8 @@ class t_program;
  */
 class t_struct : public t_structured {
  public:
-  t_struct(t_program* program, const std::string& name)
-      : t_structured(program, name) {}
+  t_struct(t_program* program, std::string name)
+      : t_structured(program, std::move(name)) {}
 
   std::string get_full_name() const override {
     return make_full_name("struct");
