@@ -1,5 +1,5 @@
 /*
- * Copyright (c) Facebook, Inc. and its affiliates.
+ * Copyright (c) Meta Platforms, Inc. and affiliates.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -88,7 +88,30 @@ const t_type* t_type::get_true_type() const {
       this, [](const t_type* type) { return !type->is_typedef(); });
 }
 
-const t_type& t_type_ref::deref() const {
+bool t_type_ref::resolved() const noexcept {
+  return !empty() &&
+      (unresolved_type_ == nullptr || unresolved_type_->type().resolved());
+}
+
+bool t_type_ref::resolve() {
+  if (unresolved_type_ != nullptr) { // Try to resolve.
+    if (!unresolved_type_->resolve()) {
+      return false;
+    }
+    unresolved_type_ = nullptr;
+  }
+  return true;
+}
+
+const t_type& t_type_ref::deref() {
+  if (!resolve()) {
+    throw std::runtime_error(
+        "Could not resolve type: " + unresolved_type_->get_full_name());
+  }
+  return deref_or_throw();
+}
+
+const t_type& t_type_ref::deref_or_throw() const {
   if (type_ == nullptr) {
     throw std::runtime_error("t_type_ref has no type.");
   }
@@ -98,6 +121,15 @@ const t_type& t_type_ref::deref() const {
   return *type_;
 }
 
+t_type_ref t_type_ref::for_placeholder(t_placeholder_typedef& unresolved_type) {
+  return t_type_ref{unresolved_type, unresolved_type};
+}
+
+const t_type_ref& t_type_ref::none() {
+  static const t_type_ref empty;
+  return empty;
+}
+
 } // namespace compiler
 } // namespace thrift
 } // namespace apache
