@@ -1,5 +1,5 @@
 /*
- * Copyright (c) Facebook, Inc. and its affiliates.
+ * Copyright (c) Meta Platforms, Inc. and affiliates.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -21,6 +21,7 @@
 #include <cstring>
 #include <map>
 #include <sstream>
+#include <stdexcept>
 #include <string>
 
 #include <boost/optional.hpp>
@@ -32,6 +33,7 @@ namespace thrift {
 namespace compiler {
 
 class t_program;
+class t_placeholder_typedef;
 
 /**
  * Generic representation of a thrift type.
@@ -214,37 +216,68 @@ class t_templated_type : public t_type {
 class t_type_ref final {
  public:
   t_type_ref() = default;
-  /* implicit */ t_type_ref(const t_type& type) : type_(&type) {}
+  /* implicit */ t_type_ref(const t_type& type) : t_type_ref(&type) {}
   /* implicit */ t_type_ref(t_type&&) = delete;
 
+  // Returns the type being referenced, resolving it if need be.
+  //
+  // Throws a std::runtime_error if the type has not been set could not be
+  // resolved.
+  const t_type& deref();
+
   // Returns the resolved type being referenced.
   //
   // Throws a std::runtime_error if the type has not been set, or an unresolved
   // t_placeholder_typedef is encountered.
-  const t_type& deref() const;
+  const t_type& deref() const { return deref_or_throw(); }
+
+  // Returns true the type reference has not been initalized.
+  bool empty() const noexcept { return type_ == nullptr; }
+  explicit operator bool() const { return !empty(); }
+  // Returns true if the type has been resolved.
+  bool resolved() const noexcept;
+  bool resolve();
+
+  // Helpers for constructing from pointers.
+  static t_type_ref from_ptr(const t_type* type) { return t_type_ref(type); }
+  static t_type_ref from_req_ptr(const t_type* type) {
+    if (type == nullptr) {
+      throw std::runtime_error("type required");
+    }
+    return from_ptr(type);
+  }
+
+  // Smart pointer-like operators.
+  const t_type* operator->() { return &deref(); }
+  const t_type& operator*() { return deref(); }
   const t_type* operator->() const { return &deref(); }
   const t_type& operator*() const { return deref(); }
 
- private:
-  friend class t_placeholder_typedef;
-  const t_type* type_ = nullptr;
+  static const t_type_ref& none();
 
-  void set_type(const t_type* type) { type_ = type; }
+ private:
+  const t_type* type_ = nullptr;
+  // The placeholder we have write access to, if we need to resolve the type
+  // before derefing.
+  // Note: It is not thread safe to access this value if 'this' is const.
+  // TODO(afuller): Move all lazy type resolution logic here, and delete
+  // t_placeholder_typedef.
+  t_placeholder_typedef* unresolved_type_ = nullptr;
+  explicit t_type_ref(
+      const t_type& type, t_placeholder_typedef& unresolved_type)
+      : type_(&type), unresolved_type_(&unresolved_type) {}
+
+  // Note: Use from_ptr or from_req_ptr for public access.
+  explicit t_type_ref(const t_type* type) : type_(type) {}
+
+  const t_type& deref_or_throw() const;
 
   // TODO(afuller): Remove everything below this comment. It is only provided
   // for backwards compatibility.
  public:
   const t_type* get_type() const { return type_; }
-
-  static boost::optional<t_type_ref> from_ptr(const t_type* type) {
-    if (type == nullptr) {
-      return boost::none;
-    }
-    return t_type_ref(*type);
-  }
-  static t_type_ref from_req_ptr(const t_type* type) {
-    return from_ptr(type).value();
-  }
+  static t_type_ref for_placeholder(t_placeholder_typedef& unresolved_type);
+  t_placeholder_typedef* get_unresolved_type() { return unresolved_type_; }
 };
 
 } // namespace compiler
