@@ -41,11 +41,11 @@
 
 #include <thrift/compiler/ast/diagnostic_context.h>
 #include <thrift/compiler/ast/t_program_bundle.h>
+#include <thrift/compiler/detail/system.h>
 #include <thrift/compiler/diagnostic.h>
 #include <thrift/compiler/generate/t_generator.h>
 #include <thrift/compiler/mutator/mutator.h>
 #include <thrift/compiler/parse/parsing_driver.h>
-#include <thrift/compiler/platform.h>
 #include <thrift/compiler/sema/standard_mutator.h>
 #include <thrift/compiler/sema/standard_validator.h>
 #include <thrift/compiler/validator/validator.h>
@@ -117,16 +117,13 @@ void usage() {
   fprintf(stderr, "\n");
   fprintf(stderr, "Available generators (and options):\n");
 
-  t_generator_registry::gen_map_t gen_map =
-      t_generator_registry::get_generator_map();
-  t_generator_registry::gen_map_t::iterator iter;
-  for (iter = gen_map.begin(); iter != gen_map.end(); ++iter) {
-    fprintf(
+  for (const auto& gen : generator_registry::get_generators()) {
+    fmt::print(
         stderr,
-        "  %s (%s):\n",
-        iter->second->get_short_name().c_str(),
-        iter->second->get_long_name().c_str());
-    fprintf(stderr, "%s", iter->second->get_documentation().c_str());
+        "  {} ({}):\n{}",
+        gen.second->name(),
+        gen.second->long_name(),
+        gen.second->documentation());
   }
 }
 
@@ -144,7 +141,7 @@ bool isComma(const char& c) {
 
 // Returns the input file name if successful, otherwise returns an empty
 // string.
-std::string parseArgs(
+std::string parse_args(
     const std::vector<std::string>& arguments,
     parsing_params& pparams,
     gen_params& gparams,
@@ -247,7 +244,7 @@ std::string parseArgs(
       bool out_path_is_absolute = (flag == "out");
 
       // Strip out trailing \ on a Windows path
-      if (platform_is_windows()) {
+      if (detail::platform_is_windows()) {
         int last = out_path.length() - 1;
         if (out_path[last] == '\\') {
           out_path.erase(last);
@@ -303,6 +300,30 @@ std::string parseArgs(
   return arguments[arg_i];
 }
 
+enum class parse_control : bool { stop, more };
+
+// `callback(key, value)` will be called for each key=value generator's option.
+// If there is no value, `value` will be empty string.
+void parse_generator_options(
+    const std::string& option_string,
+    std::function<parse_control(std::string, std::string)> callback) {
+  std::vector<std::string> parts;
+  bool inside_braces = false;
+  boost::algorithm::split(parts, option_string, [&inside_braces](char c) {
+    if (c == '{' || c == '}') {
+      inside_braces = (c == '{');
+    }
+    return c == ',' && !inside_braces;
+  });
+  for (const auto& part : parts) {
+    auto key = part.substr(0, part.find('='));
+    auto value = part.substr(std::min(key.size() + 1, part.size()));
+    if (callback(std::move(key), std::move(value)) == parse_control::stop) {
+      break;
+    }
+  }
+}
+
 // Generate code.
 bool generate(
     const gen_params& params,
@@ -323,39 +344,39 @@ bool generate(
 
   // Generate code!
   try {
-    diags.report(
-        source_location(),
-        diagnostic_level::info,
-        "program: {}",
-        program.path());
-
     bool success = true;
     std::ofstream genfile;
     if (!params.genfile.empty()) {
       genfile.open(params.genfile);
     }
     for (auto target : params.targets) {
-      auto pos = target.find(':');
-      std::string lang = target.substr(0, pos);
-      auto generator =
-          std::unique_ptr<t_generator>{t_generator_registry::get_generator(
-              &program, params.context, target)};
-      if (generator == nullptr) {
+      auto colon_pos = target.find(':');
+      auto generator_name = target.substr(0, colon_pos);
+      auto options = std::map<std::string, std::string>();
+      parse_generator_options(
+          target.substr(colon_pos + 1), [&](std::string k, std::string v) {
+            options[std::move(k)] = std::move(v);
+            return parse_control::more;
+          });
+      auto generator = generator_registry::make_generator(
+          generator_name, program, params.context, options);
+      if (!generator) {
         continue;
       }
 
-      validator::diagnostics_t diagnostics;
-      validator_list validators(diagnostics);
+      auto validator_diags = diagnostics_engine(
+          diags.source_mgr(),
+          [&](diagnostic d) {
+            if (d.level() == diagnostic_level::error) {
+              success = false;
+            }
+            diags.report(std::move(d));
+          },
+          diags.params());
+      validator_list validators(validator_diags);
       generator->fill_validator_list(validators);
       validators.traverse(&program);
-
-      bool has_failure = false;
-      for (const auto& d : diagnostics) {
-        has_failure = has_failure || (d.level() == diagnostic_level::failure);
-        std::cerr << d << std::endl;
-      }
-      if (has_failure) {
-        success = false;
+      if (!success) {
         continue;
       }
 
@@ -393,22 +414,22 @@ std::string get_include_path(
     const std::string& input_filename) {
   std::string include_prefix;
   for (const auto& target : generator_targets) {
-    auto const colon_pos = target.find(':');
+    const auto colon_pos = target.find(':');
     if (colon_pos == std::string::npos) {
       continue;
     }
-    auto const lang_name = target.substr(0, colon_pos);
+    const auto lang_name = target.substr(0, colon_pos);
     if (lang_name != "cpp2" && lang_name != "mstch_cpp2") {
       continue;
     }
 
-    auto const lang_args = target.substr(colon_pos + 1);
+    const auto lang_args = target.substr(colon_pos + 1);
     parse_generator_options(lang_args, [&](std::string k, std::string v) {
       if (k.find("include_prefix") != std::string::npos) {
         include_prefix = std::move(v);
-        return CallbackLoopControl::Break;
+        return parse_control::stop;
       }
-      return CallbackLoopControl::Continue;
+      return parse_control::more;
     });
   }
 
@@ -474,7 +495,7 @@ std::unique_ptr<t_program_bundle> parse_and_get_program(
   parsing_params pparams;
   gen_params gparams;
   diagnostic_params dparams;
-  std::string filename = parseArgs(arguments, pparams, gparams, dparams);
+  std::string filename = parse_args(arguments, pparams, gparams, dparams);
 
   if (filename.empty()) {
     return {};
@@ -490,7 +511,7 @@ compile_result compile(const std::vector<std::string>& arguments) {
   parsing_params pparams;
   gen_params gparams;
   diagnostic_params dparams;
-  std::string input_filename = parseArgs(arguments, pparams, gparams, dparams);
+  std::string input_filename = parse_args(arguments, pparams, gparams, dparams);
   if (input_filename.empty()) {
     return result;
   }
@@ -517,11 +538,9 @@ compile_result compile(const std::vector<std::string>& arguments) {
       get_include_path(gparams.targets, input_filename));
 
   // Validate it!
-  for (auto&& diag : validator::validate(program->root_program())) {
-    ctx.report(std::forward<decltype(diag)>(diag));
-  }
+  validator::validate(program->root_program(), ctx);
   standard_validator()(ctx, *program->root_program());
-  if (result.detail.has_failure()) {
+  if (result.detail.has_error()) {
     return result;
   }
 
@@ -532,7 +551,7 @@ compile_result compile(const std::vector<std::string>& arguments) {
       result.retcode = compile_retcode::success;
     }
   } catch (const std::exception& e) {
-    ctx.failure(*program->root_program(), [&](auto& o) { o << e.what(); });
+    ctx.error(*program->root_program(), "{}", e.what());
   }
   ctx.end_visit(*program->root_program());
   return result;
