@@ -26,10 +26,8 @@ namespace compiler {
 namespace {
 const char* level_to_string(diagnostic_level level) {
   switch (level) {
-    case diagnostic_level::parse_error:
+    case diagnostic_level::error:
       return "ERROR";
-    case diagnostic_level::failure:
-      return "FAILURE";
     case diagnostic_level::warning:
       return "WARNING";
     case diagnostic_level::info:
