@@ -42,20 +42,20 @@ class TypeResolverTest : public ::testing::Test {
     return resolver_.can_resolve_to_scalar(node);
   }
 
-  const std::string& get_type_name(const t_type& node) {
-    return resolver_.get_type_name(node);
+  const std::string& get_native_type(const t_type& node) {
+    return resolver_.get_native_type(node);
   }
 
-  const std::string& get_type_name(const t_field& node) {
-    return resolver_.get_type_name(node, struct_);
+  const std::string& get_native_type(const t_field& node) {
+    return resolver_.get_native_type(node, struct_);
   }
 
-  const std::string& get_standard_type_name(const t_type& node) {
-    return resolver_.get_standard_type_name(node);
+  const std::string& get_standard_type(const t_type& node) {
+    return resolver_.get_standard_type(node);
   }
 
-  const std::string& get_storage_type_name(const t_field& node) {
-    return resolver_.get_storage_type_name(node, struct_);
+  const std::string& get_storage_type(const t_field& node) {
+    return resolver_.get_storage_type(node, struct_);
   }
 
   std::string get_type_tag(const t_type& type) {
@@ -70,16 +70,16 @@ class TypeResolverTest : public ::testing::Test {
 };
 
 TEST_F(TypeResolverTest, BaseTypes) {
-  EXPECT_EQ(get_type_name(t_base_type::t_void()), "void");
-  EXPECT_EQ(get_type_name(t_base_type::t_bool()), "bool");
-  EXPECT_EQ(get_type_name(t_base_type::t_byte()), "::std::int8_t");
-  EXPECT_EQ(get_type_name(t_base_type::t_i16()), "::std::int16_t");
-  EXPECT_EQ(get_type_name(t_base_type::t_i32()), "::std::int32_t");
-  EXPECT_EQ(get_type_name(t_base_type::t_i64()), "::std::int64_t");
-  EXPECT_EQ(get_type_name(t_base_type::t_float()), "float");
-  EXPECT_EQ(get_type_name(t_base_type::t_double()), "double");
-  EXPECT_EQ(get_type_name(t_base_type::t_string()), "::std::string");
-  EXPECT_EQ(get_type_name(t_base_type::t_binary()), "::std::string");
+  EXPECT_EQ(get_native_type(t_base_type::t_void()), "void");
+  EXPECT_EQ(get_native_type(t_base_type::t_bool()), "bool");
+  EXPECT_EQ(get_native_type(t_base_type::t_byte()), "::std::int8_t");
+  EXPECT_EQ(get_native_type(t_base_type::t_i16()), "::std::int16_t");
+  EXPECT_EQ(get_native_type(t_base_type::t_i32()), "::std::int32_t");
+  EXPECT_EQ(get_native_type(t_base_type::t_i64()), "::std::int64_t");
+  EXPECT_EQ(get_native_type(t_base_type::t_float()), "float");
+  EXPECT_EQ(get_native_type(t_base_type::t_double()), "double");
+  EXPECT_EQ(get_native_type(t_base_type::t_string()), "::std::string");
+  EXPECT_EQ(get_native_type(t_base_type::t_binary()), "::std::string");
 
   EXPECT_FALSE(can_resolve_to_scalar(t_base_type::t_void()));
   EXPECT_TRUE(can_resolve_to_scalar(t_base_type::t_bool()));
@@ -115,50 +115,44 @@ TEST_F(TypeResolverTest, BaseTypes) {
       "::apache::thrift::type::double_t");
 }
 
-TEST_F(TypeResolverTest, BaseTypes_Adapter) {
-  t_base_type dbl(t_base_type::t_double());
-  dbl.set_annotation("cpp.adapter", "DblAdapter");
-  // The standard type is the default, double.
-  EXPECT_EQ(get_standard_type_name(dbl), "double");
+TEST_F(TypeResolverTest, Struct_Adapter) {
+  t_struct strct(&program_, "Foo");
+  strct.add_structured_annotation(
+      adapter_builder(program_, "cpp").make("FooAdapter"));
+  // The standard type is the default.
+  EXPECT_EQ(get_standard_type(strct), "::path::to::detail::Foo");
   // The c++ type is adapted.
-  EXPECT_EQ(
-      get_type_name(dbl),
-      "::apache::thrift::adapt_detail::adapted_t<DblAdapter, double>");
-  EXPECT_TRUE(can_resolve_to_scalar(dbl));
+  EXPECT_EQ(get_native_type(strct), "::path::to::Foo");
 
-  // cpp.type overrides the 'default' standard type.
-  t_base_type ui64(t_base_type::t_i64());
-  ui64.set_annotation("cpp.type", "uint64_t");
-  ui64.set_annotation("cpp.adapter", "HashAdapter");
-  EXPECT_EQ(get_standard_type_name(ui64), "uint64_t");
-  EXPECT_EQ(
-      get_type_name(ui64),
-      "::apache::thrift::adapt_detail::adapted_t<HashAdapter, uint64_t>");
-  EXPECT_TRUE(can_resolve_to_scalar(ui64));
+  // cpp.name overrides the 'default' standard type.
+  t_struct strct2(&program_, "Foo");
+  strct2.set_annotation("cpp.name", "Bar");
+  strct2.add_structured_annotation(
+      adapter_builder(program_, "cpp").make("FooAdapter"));
+  EXPECT_EQ(get_standard_type(strct2), "::path::to::detail::Bar");
+  EXPECT_EQ(get_native_type(strct2), "::path::to::Bar");
 
   // cpp.adapter could resolve to a scalar.
-  t_base_type str(t_base_type::t_string());
-  str.set_annotation("cpp.adapter", "HashAdapter");
-  EXPECT_TRUE(can_resolve_to_scalar(str));
+  EXPECT_TRUE(can_resolve_to_scalar(strct));
 }
 
 TEST_F(TypeResolverTest, CppName) {
   t_enum tenum(&program_, "MyEnum");
   tenum.set_annotation("cpp.name", "YourEnum");
-  EXPECT_EQ(get_type_name(tenum), "::path::to::YourEnum");
+  EXPECT_EQ(get_native_type(tenum), "::path::to::YourEnum");
 }
 
 TEST_F(TypeResolverTest, Containers) {
   // A container could not resolve to a scalar.
   t_map tmap(t_base_type::t_string(), t_base_type::t_i32());
-  EXPECT_EQ(get_type_name(tmap), "::std::map<::std::string, ::std::int32_t>");
+  EXPECT_EQ(get_native_type(tmap), "::std::map<::std::string, ::std::int32_t>");
   EXPECT_FALSE(can_resolve_to_scalar(tmap));
   t_list tlist(t_base_type::t_double());
-  EXPECT_EQ(get_type_name(tlist), "::std::vector<double>");
+  EXPECT_EQ(get_native_type(tlist), "::std::vector<double>");
   EXPECT_FALSE(can_resolve_to_scalar(tlist));
   t_set tset(tmap);
   EXPECT_EQ(
-      get_type_name(tset),
+      get_native_type(tset),
       "::std::set<::std::map<::std::string, ::std::int32_t>>");
   EXPECT_FALSE(can_resolve_to_scalar(tset));
 }
@@ -169,82 +163,70 @@ TEST_F(TypeResolverTest, Containers_CustomTemplate) {
   t_map tmap(t_base_type::t_string(), t_base_type::t_i32());
   tmap.set_annotation("cpp.template", "std::unordered_map");
   EXPECT_EQ(
-      get_type_name(tmap), "std::unordered_map<::std::string, ::std::int32_t>");
+      get_native_type(tmap),
+      "std::unordered_map<::std::string, ::std::int32_t>");
   EXPECT_FALSE(can_resolve_to_scalar(tmap));
   t_list tlist(t_base_type::t_double());
   tlist.set_annotation("cpp2.template", "std::list");
-  EXPECT_EQ(get_type_name(tlist), "std::list<double>");
+  EXPECT_EQ(get_native_type(tlist), "std::list<double>");
   EXPECT_FALSE(can_resolve_to_scalar(tlist));
   t_set tset(t_base_type::t_binary());
   tset.set_annotation("cpp2.template", "::std::unordered_set");
-  EXPECT_EQ(get_type_name(tset), "::std::unordered_set<::std::string>");
+  EXPECT_EQ(get_native_type(tset), "::std::unordered_set<::std::string>");
   EXPECT_FALSE(can_resolve_to_scalar(tset));
 }
 
 TEST_F(TypeResolverTest, Containers_Adapter) {
   // cpp.adapter could resolve to a scalar.
-  t_base_type ui64(t_base_type::t_i64());
-  ui64.set_annotation("cpp.type", "uint64_t");
-  ui64.set_annotation("cpp.adapter", "HashAdapter");
-  EXPECT_TRUE(can_resolve_to_scalar(ui64));
+  t_struct strct(&program_, "Foo");
+  strct.add_structured_annotation(
+      adapter_builder(program_, "cpp").make("HashAdapter"));
+  EXPECT_TRUE(can_resolve_to_scalar(strct));
 
   // Adapters work on container type arguments.
-  t_map tmap(t_base_type::t_i16(), ui64);
-  tmap.set_annotation("cpp.adapter", "MapAdapter");
+  t_map tmap(t_base_type::t_i16(), strct);
   EXPECT_EQ(
-      get_standard_type_name(tmap), "::std::map<::std::int16_t, uint64_t>");
+      get_standard_type(tmap),
+      "::std::map<::std::int16_t, ::path::to::detail::Foo>");
   EXPECT_EQ(
-      get_type_name(tmap),
-      "::apache::thrift::adapt_detail::adapted_t<"
-      "MapAdapter, "
-      "::std::map<::std::int16_t, ::apache::thrift::adapt_detail::adapted_t<HashAdapter, uint64_t>>>");
-  EXPECT_TRUE(can_resolve_to_scalar(tmap));
+      get_native_type(tmap), "::std::map<::std::int16_t, ::path::to::Foo>");
 
   // The container can also be addapted.
-  t_set tset(ui64);
-  tset.set_annotation("cpp.adapter", "SetAdapter");
+  t_set tset(strct);
   tset.set_annotation("cpp.template", "std::unordered_set");
   // The template argument is respected for both standard and adapted types.
-  EXPECT_EQ(get_standard_type_name(tset), "std::unordered_set<uint64_t>");
   EXPECT_EQ(
-      get_type_name(tset),
-      "::apache::thrift::adapt_detail::adapted_t<"
-      "SetAdapter, "
-      "std::unordered_set<::apache::thrift::adapt_detail::adapted_t<HashAdapter, uint64_t>>>");
-  EXPECT_TRUE(can_resolve_to_scalar(tset));
+      get_standard_type(tset), "std::unordered_set<::path::to::detail::Foo>");
+  EXPECT_EQ(get_native_type(tset), "std::unordered_set<::path::to::Foo>");
 
   // cpp.type on the container overrides the 'default' standard type.
-  t_list tlist(ui64);
-  tlist.set_annotation("cpp.adapter", "ListAdapter");
+  t_list tlist(strct);
   tlist.set_annotation("cpp.type", "MyList");
-  EXPECT_EQ(get_standard_type_name(tlist), "MyList");
-  EXPECT_EQ(
-      get_type_name(tlist),
-      "::apache::thrift::adapt_detail::adapted_t<ListAdapter, MyList>");
-  EXPECT_TRUE(can_resolve_to_scalar(tlist));
+  EXPECT_EQ(get_standard_type(tlist), "MyList");
+  EXPECT_EQ(get_native_type(tlist), "MyList");
 }
 
 TEST_F(TypeResolverTest, Structs) {
   t_struct tstruct(&program_, "Foo");
-  EXPECT_EQ(get_type_name(tstruct), "::path::to::Foo");
+  EXPECT_EQ(get_native_type(tstruct), "::path::to::Foo");
   EXPECT_FALSE(can_resolve_to_scalar(tstruct));
   t_union tunion(&program_, "Bar");
-  EXPECT_EQ(get_type_name(tunion), "::path::to::Bar");
+  EXPECT_EQ(get_native_type(tunion), "::path::to::Bar");
   EXPECT_FALSE(can_resolve_to_scalar(tunion));
   t_exception texcept(&program_, "Baz");
-  EXPECT_EQ(get_type_name(texcept), "::path::to::Baz");
+  EXPECT_EQ(get_native_type(texcept), "::path::to::Baz");
   EXPECT_FALSE(can_resolve_to_scalar(texcept));
 }
 
 TEST_F(TypeResolverTest, TypeDefs) {
   // Scalar
   t_typedef ttypedef1(&program_, "Foo", t_base_type::t_bool());
-  EXPECT_EQ(get_type_name(ttypedef1), "::path::to::Foo");
+  EXPECT_EQ(get_native_type(ttypedef1), "::path::to::Foo");
   EXPECT_TRUE(can_resolve_to_scalar(ttypedef1));
 
   // Non-scalar
   t_typedef ttypedef2(&program_, "Foo", t_base_type::t_string());
-  EXPECT_EQ(get_type_name(ttypedef2), "::path::to::Foo");
+  EXPECT_EQ(get_native_type(ttypedef2), "::path::to::Foo");
   EXPECT_FALSE(can_resolve_to_scalar(ttypedef2));
 }
 
@@ -252,43 +234,40 @@ TEST_F(TypeResolverTest, TypeDefs_Nested) {
   // Scalar
   t_typedef ttypedef1(&program_, "Foo", t_base_type::t_bool());
   t_typedef ttypedef2(&program_, "Bar", ttypedef1);
-  EXPECT_EQ(get_type_name(ttypedef2), "::path::to::Bar");
+  EXPECT_EQ(get_native_type(ttypedef2), "::path::to::Bar");
   EXPECT_TRUE(can_resolve_to_scalar(ttypedef1));
   EXPECT_TRUE(can_resolve_to_scalar(ttypedef2));
 
   // Non-scalar
   t_typedef ttypedef3(&program_, "Foo", t_base_type::t_string());
   t_typedef ttypedef4(&program_, "Bar", ttypedef3);
-  EXPECT_EQ(get_type_name(ttypedef4), "::path::to::Bar");
+  EXPECT_EQ(get_native_type(ttypedef4), "::path::to::Bar");
   EXPECT_FALSE(can_resolve_to_scalar(ttypedef3));
   EXPECT_FALSE(can_resolve_to_scalar(ttypedef4));
 }
 
 TEST_F(TypeResolverTest, TypeDefs_Adapter) {
-  t_base_type ui64(t_base_type::t_i64());
-  ui64.set_annotation("cpp.type", "uint64_t");
-  ui64.set_annotation("cpp.adapter", "HashAdapter");
+  t_struct strct(&program_, "Foo");
+  strct.add_structured_annotation(
+      adapter_builder(program_, "cpp").make("HashAdapter"));
 
   // Type defs can refer to adatped types.
-  t_typedef ttypedef1(&program_, "MyHash", ui64);
+  t_typedef ttypedef1(&program_, "MyHash", strct);
   // It does not affect the type name.
-  EXPECT_EQ(get_standard_type_name(ttypedef1), "uint64_t");
-  EXPECT_EQ(get_type_name(ttypedef1), "::path::to::MyHash");
+  EXPECT_EQ(get_standard_type(ttypedef1), "::path::to::detail::Foo");
+  EXPECT_EQ(get_native_type(ttypedef1), "::path::to::MyHash");
   // It is the refered to type that has the adapter.
-  EXPECT_EQ(
-      get_type_name(*ttypedef1.type()),
-      "::apache::thrift::adapt_detail::adapted_t<HashAdapter, uint64_t>");
+  EXPECT_EQ(get_native_type(*ttypedef1.type()), "::path::to::Foo");
   EXPECT_TRUE(can_resolve_to_scalar(ttypedef1));
   ASSERT_TRUE(resolver_.find_first_adapter(ttypedef1));
   EXPECT_EQ(*resolver_.find_first_adapter(ttypedef1), "HashAdapter");
 
   // Type defs can also be adapted.
   t_typedef ttypedef2(ttypedef1);
-  ttypedef2.set_annotation("cpp.adapter", "TypeDefAdapter");
-  EXPECT_EQ(get_standard_type_name(ttypedef2), "uint64_t");
-  EXPECT_EQ(
-      get_type_name(ttypedef2),
-      "::apache::thrift::adapt_detail::adapted_t<TypeDefAdapter, ::path::to::MyHash>");
+  ttypedef2.add_structured_annotation(
+      adapter_builder(program_, "cpp").make("TypeDefAdapter"));
+  EXPECT_EQ(get_standard_type(ttypedef2), "::path::to::detail::Foo");
+  EXPECT_EQ(get_native_type(ttypedef2), "::path::to::MyHash");
   EXPECT_TRUE(can_resolve_to_scalar(ttypedef2));
   ASSERT_TRUE(resolver_.find_first_adapter(ttypedef2));
   EXPECT_EQ(*resolver_.find_first_adapter(ttypedef2), "TypeDefAdapter");
@@ -307,174 +286,166 @@ TEST_F(TypeResolverTest, CustomType) {
   t_base_type tui64(t_base_type::t_i64());
   tui64.set_name("ui64");
   tui64.set_annotation("cpp2.type", "::std::uint64_t");
-  EXPECT_EQ(get_type_name(tui64), "::std::uint64_t");
+  EXPECT_EQ(get_native_type(tui64), "::std::uint64_t");
   EXPECT_TRUE(can_resolve_to_scalar(tui64));
 
   t_union tunion(&program_, "Bar");
   tunion.set_annotation("cpp2.type", "Other");
-  EXPECT_EQ(get_type_name(tunion), "Other");
+  EXPECT_EQ(get_native_type(tunion), "Other");
   EXPECT_TRUE(can_resolve_to_scalar(tunion));
 
   t_typedef ttypedef1(&program_, "Foo", t_base_type::t_bool());
   ttypedef1.set_annotation("cpp2.type", "Other");
-  EXPECT_EQ(get_type_name(ttypedef1), "Other");
+  EXPECT_EQ(get_native_type(ttypedef1), "Other");
   EXPECT_TRUE(can_resolve_to_scalar(ttypedef1));
 
   t_typedef ttypedef2(&program_, "FooBar", t_base_type::t_string());
   ttypedef2.set_annotation("cpp2.type", "Other");
-  EXPECT_EQ(get_type_name(ttypedef2), "Other");
+  EXPECT_EQ(get_native_type(ttypedef2), "Other");
   EXPECT_TRUE(can_resolve_to_scalar(ttypedef2));
 
   t_map tmap1(t_base_type::t_string(), tui64);
-  EXPECT_EQ(get_type_name(tmap1), "::std::map<::std::string, ::std::uint64_t>");
+  EXPECT_EQ(
+      get_native_type(tmap1), "::std::map<::std::string, ::std::uint64_t>");
   EXPECT_FALSE(can_resolve_to_scalar(tmap1));
 
   // Can be combined with template.
   t_map tmap2(tmap1);
   tmap2.set_annotation("cpp.template", "std::unordered_map");
   EXPECT_EQ(
-      get_type_name(tmap2),
+      get_native_type(tmap2),
       "std::unordered_map<::std::string, ::std::uint64_t>");
   EXPECT_FALSE(can_resolve_to_scalar(tmap2));
 
   // Custom type overrides template.
   t_map tmap3(tmap2);
   tmap3.set_annotation("cpp.type", "MyMap");
-  EXPECT_EQ(get_type_name(tmap3), "MyMap");
+  EXPECT_EQ(get_native_type(tmap3), "MyMap");
   EXPECT_TRUE(can_resolve_to_scalar(tmap3));
 }
 
 TEST_F(TypeResolverTest, StreamingRes) {
   t_base_type ui64(t_base_type::t_i64());
   ui64.set_annotation("cpp.type", "uint64_t");
-  ui64.set_annotation("cpp.adapter", "HashAdapter");
 
   t_stream_response res1(ui64);
   EXPECT_EQ(
-      get_standard_type_name(res1), "::apache::thrift::ServerStream<uint64_t>");
-  EXPECT_EQ(
-      get_type_name(res1),
-      "::apache::thrift::ServerStream<::apache::thrift::adapt_detail::adapted_t<HashAdapter, uint64_t>>");
+      get_standard_type(res1), "::apache::thrift::ServerStream<uint64_t>");
+  EXPECT_EQ(get_native_type(res1), "::apache::thrift::ServerStream<uint64_t>");
 
   t_stream_response res2(ui64);
   res2.set_first_response_type(t_type_ref(ui64));
   EXPECT_EQ(
-      get_standard_type_name(res2),
+      get_standard_type(res2),
       "::apache::thrift::ResponseAndServerStream<uint64_t, uint64_t>");
   EXPECT_EQ(
-      get_type_name(res2),
-      "::apache::thrift::ResponseAndServerStream<"
-      "::apache::thrift::adapt_detail::adapted_t<HashAdapter, uint64_t>, "
-      "::apache::thrift::adapt_detail::adapted_t<HashAdapter, uint64_t>>");
+      get_native_type(res2),
+      "::apache::thrift::ResponseAndServerStream<uint64_t, uint64_t>");
 }
 
 TEST_F(TypeResolverTest, StreamingSink) {
-  t_base_type ui64(t_base_type::t_i64());
-  ui64.set_annotation("cpp.type", "uint64_t");
-  ui64.set_annotation("cpp.adapter", "HashAdapter");
+  t_struct tstruct(&program_, "Foo");
+  tstruct.add_structured_annotation(
+      adapter_builder(program_, "cpp").make("HashAdapter"));
 
-  t_sink req1(ui64, ui64);
+  t_sink req1(tstruct, tstruct);
   EXPECT_EQ(
-      get_standard_type_name(req1),
-      "::apache::thrift::SinkConsumer<uint64_t, uint64_t>");
+      get_standard_type(req1),
+      "::apache::thrift::SinkConsumer<::path::to::detail::Foo, ::path::to::detail::Foo>");
   EXPECT_EQ(
-      get_type_name(req1),
+      get_native_type(req1),
       "::apache::thrift::SinkConsumer<"
-      "::apache::thrift::adapt_detail::adapted_t<HashAdapter, uint64_t>, "
-      "::apache::thrift::adapt_detail::adapted_t<HashAdapter, uint64_t>>");
+      "::path::to::Foo, "
+      "::path::to::Foo>");
 
-  t_sink req2(ui64, ui64);
-  req2.set_first_response_type(t_type_ref(ui64));
+  t_sink req2(tstruct, tstruct);
+  req2.set_first_response_type(t_type_ref(tstruct));
   EXPECT_EQ(
-      get_standard_type_name(req2),
-      "::apache::thrift::ResponseAndSinkConsumer<uint64_t, uint64_t, uint64_t>");
-  EXPECT_EQ(
-      get_type_name(req2),
+      get_standard_type(req2),
       "::apache::thrift::ResponseAndSinkConsumer<"
-      "::apache::thrift::adapt_detail::adapted_t<HashAdapter, uint64_t>, "
-      "::apache::thrift::adapt_detail::adapted_t<HashAdapter, uint64_t>, "
-      "::apache::thrift::adapt_detail::adapted_t<HashAdapter, uint64_t>>");
+      "::path::to::detail::Foo, "
+      "::path::to::detail::Foo, "
+      "::path::to::detail::Foo>");
+  EXPECT_EQ(
+      get_native_type(req2),
+      "::apache::thrift::ResponseAndSinkConsumer<"
+      "::path::to::Foo, "
+      "::path::to::Foo, "
+      "::path::to::Foo>");
 }
 
 TEST_F(TypeResolverTest, StorageType) {
-  t_base_type ui64(t_base_type::t_i64());
-  ui64.set_annotation("cpp.type", "uint64_t");
-  ui64.set_annotation("cpp.adapter", "HashAdapter");
+  t_struct strct(&program_, "Foo");
+  strct.add_structured_annotation(
+      adapter_builder(program_, "cpp").make("HashAdapter"));
   {
-    t_field ui64_field(ui64, "hash", 1);
-    EXPECT_EQ(
-        get_storage_type_name(ui64_field),
-        "::apache::thrift::adapt_detail::adapted_t<HashAdapter, uint64_t>");
+    t_field strct_field(strct, "hash", 1);
+    EXPECT_EQ(get_storage_type(strct_field), "::path::to::Foo");
   }
 
   {
-    t_field ui64_field(ui64, "hash", 1);
-    ui64_field.set_annotation("cpp.ref", "");
+    t_field strct_field(strct, "hash", 1);
+    strct_field.set_annotation("cpp.ref", "");
     EXPECT_EQ(
-        get_storage_type_name(ui64_field),
-        "::std::unique_ptr<::apache::thrift::adapt_detail::adapted_t<HashAdapter, uint64_t>>");
+        get_storage_type(strct_field), "::std::unique_ptr<::path::to::Foo>");
   }
   {
-    t_field ui64_field(ui64, "hash", 1);
-    ui64_field.set_annotation("cpp2.ref", ""); // Works with cpp2.
+    t_field strct_field(strct, "hash", 1);
+    strct_field.set_annotation("cpp2.ref", ""); // Works with cpp2.
     EXPECT_EQ(
-        get_storage_type_name(ui64_field),
-        "::std::unique_ptr<::apache::thrift::adapt_detail::adapted_t<HashAdapter, uint64_t>>");
+        get_storage_type(strct_field), "::std::unique_ptr<::path::to::Foo>");
   }
 
   {
-    t_field ui64_field(ui64, "hash", 1);
-    ui64_field.set_annotation("cpp.ref_type", "unique");
+    t_field strct_field(strct, "hash", 1);
+    strct_field.set_annotation("cpp.ref_type", "unique");
     EXPECT_EQ(
-        get_storage_type_name(ui64_field),
-        "::std::unique_ptr<::apache::thrift::adapt_detail::adapted_t<HashAdapter, uint64_t>>");
+        get_storage_type(strct_field), "::std::unique_ptr<::path::to::Foo>");
   }
 
   {
-    t_field ui64_field(ui64, "hash", 1);
-    ui64_field.set_annotation("cpp2.ref_type", "shared"); // Works with cpp2.
+    t_field strct_field(strct, "hash", 1);
+    strct_field.set_annotation("cpp2.ref_type", "shared"); // Works with cpp2.
     EXPECT_EQ(
-        get_storage_type_name(ui64_field),
-        "::std::shared_ptr<::apache::thrift::adapt_detail::adapted_t<HashAdapter, uint64_t>>");
+        get_storage_type(strct_field), "::std::shared_ptr<::path::to::Foo>");
   }
 
   {
-    t_field ui64_field(ui64, "hash", 1);
-    ui64_field.set_annotation("cpp.ref_type", "shared_mutable");
+    t_field strct_field(strct, "hash", 1);
+    strct_field.set_annotation("cpp.ref_type", "shared_mutable");
     EXPECT_EQ(
-        get_storage_type_name(ui64_field),
-        "::std::shared_ptr<::apache::thrift::adapt_detail::adapted_t<HashAdapter, uint64_t>>");
+        get_storage_type(strct_field), "::std::shared_ptr<::path::to::Foo>");
   }
 
   {
-    t_field ui64_field(ui64, "hash", 1);
-    ui64_field.set_annotation("cpp.ref_type", "shared_const");
+    t_field strct_field(strct, "hash", 1);
+    strct_field.set_annotation("cpp.ref_type", "shared_const");
     EXPECT_EQ(
-        get_storage_type_name(ui64_field),
-        "::std::shared_ptr<const ::apache::thrift::adapt_detail::adapted_t<HashAdapter, uint64_t>>");
+        get_storage_type(strct_field),
+        "::std::shared_ptr<const ::path::to::Foo>");
   }
 
   {
-    t_field ui64_field(ui64, "hash", 1);
-    ui64_field.set_annotation("thrift.box", "");
+    t_field strct_field(strct, "hash", 1);
+    strct_field.set_annotation("thrift.box", "");
     EXPECT_EQ(
-        get_storage_type_name(ui64_field),
-        "::apache::thrift::detail::boxed_value_ptr<::apache::thrift::adapt_detail::adapted_t<HashAdapter, uint64_t>>");
+        get_storage_type(strct_field),
+        "::apache::thrift::detail::boxed_value_ptr<::path::to::Foo>");
   }
 
   {
-    t_field ui64_field(ui64, "hash", 1);
-    ui64_field.add_structured_annotation(
+    t_field strct_field(strct, "hash", 1);
+    strct_field.add_structured_annotation(
         thrift_annotation_builder::box(program_).make());
     EXPECT_EQ(
-        get_storage_type_name(ui64_field),
-        "::apache::thrift::detail::boxed_value_ptr<::apache::thrift::adapt_detail::adapted_t<HashAdapter, uint64_t>>");
+        get_storage_type(strct_field),
+        "::apache::thrift::detail::boxed_value_ptr<::path::to::Foo>");
   }
 
   { // Unrecognized throws an exception.
-    t_field ui64_field(ui64, "hash", 1);
-    ui64_field.set_annotation("cpp.ref_type", "blah");
-    EXPECT_THROW(get_storage_type_name(ui64_field), std::runtime_error);
+    t_field strct_field(strct, "hash", 1);
+    strct_field.set_annotation("cpp.ref_type", "blah");
+    EXPECT_THROW(get_storage_type(strct_field), std::runtime_error);
   }
 }
 
@@ -486,22 +457,21 @@ TEST_F(TypeResolverTest, Typedef_cpptemplate) {
   iumap.set_annotation("cpp.template", "std::unorderd_map");
   t_typedef tiumap(&program_, "tiumap", iumap);
 
-  EXPECT_EQ(get_type_name(imap), "::std::map<::std::int32_t, ::std::string>");
+  EXPECT_EQ(get_native_type(imap), "::std::map<::std::int32_t, ::std::string>");
   EXPECT_EQ(
-      get_standard_type_name(imap),
-      "::std::map<::std::int32_t, ::std::string>");
+      get_standard_type(imap), "::std::map<::std::int32_t, ::std::string>");
   EXPECT_FALSE(can_resolve_to_scalar(imap));
 
   // The 'cpp.template' annotations is applied to the typedef; however, the type
   // resolver only looks for it on container types.
   // TODO(afuller): Consider making the template annotation propagate through
   // the typedef.
-  EXPECT_EQ(get_type_name(iumap), "::path::to::iumap");
-  EXPECT_EQ(get_standard_type_name(iumap), "::path::to::iumap");
+  EXPECT_EQ(get_native_type(iumap), "::path::to::iumap");
+  EXPECT_EQ(get_standard_type(iumap), "::path::to::iumap");
   EXPECT_FALSE(can_resolve_to_scalar(iumap));
 
-  EXPECT_EQ(get_type_name(tiumap), "::path::to::tiumap");
-  EXPECT_EQ(get_standard_type_name(tiumap), "::path::to::tiumap");
+  EXPECT_EQ(get_native_type(tiumap), "::path::to::tiumap");
+  EXPECT_EQ(get_standard_type(tiumap), "::path::to::tiumap");
   EXPECT_FALSE(can_resolve_to_scalar(tiumap));
 }
 
@@ -512,10 +482,9 @@ TEST_F(TypeResolverTest, Typedef_cpptype) {
       "cpp.type", "std::unorderd_map<::std::int32_t, ::std::string>");
   t_typedef tiumap(&program_, "tiumap", iumap);
 
-  EXPECT_EQ(get_type_name(imap), "::std::map<::std::int32_t, ::std::string>");
+  EXPECT_EQ(get_native_type(imap), "::std::map<::std::int32_t, ::std::string>");
   EXPECT_EQ(
-      get_standard_type_name(imap),
-      "::std::map<::std::int32_t, ::std::string>");
+      get_standard_type(imap), "::std::map<::std::int32_t, ::std::string>");
   EXPECT_FALSE(can_resolve_to_scalar(imap));
 
   // The 'cpp.type' annotation is respected on the typedef.
@@ -524,89 +493,29 @@ TEST_F(TypeResolverTest, Typedef_cpptype) {
   // the typedef itself (which should just be the typedef's specified name)
   // which is still code-genend, but with the wrong type!
   EXPECT_EQ(
-      get_type_name(iumap), "std::unorderd_map<::std::int32_t, ::std::string>");
+      get_native_type(iumap),
+      "std::unorderd_map<::std::int32_t, ::std::string>");
   EXPECT_EQ(
-      get_standard_type_name(iumap),
+      get_standard_type(iumap),
       "std::unorderd_map<::std::int32_t, ::std::string>");
   EXPECT_TRUE(can_resolve_to_scalar(iumap));
 
-  EXPECT_EQ(get_type_name(tiumap), "::path::to::tiumap");
-  EXPECT_EQ(get_standard_type_name(tiumap), "::path::to::tiumap");
+  EXPECT_EQ(get_native_type(tiumap), "::path::to::tiumap");
+  EXPECT_EQ(get_standard_type(tiumap), "::path::to::tiumap");
   EXPECT_TRUE(can_resolve_to_scalar(tiumap));
 }
 
-TEST_F(TypeResolverTest, Typedef_cpptype_and_adapter) {
-  // cpp.type sets the 'standard' type.
-  // cpp.adapter adapts the 'standard' type.
-
-  // Switch the standard type to IOBuf:
-  //   iobuf_type = binary (cpp.type="folly::IOBuf")
-  t_base_type iobuf_type = t_base_type::t_binary();
-  iobuf_type.set_annotation("cpp.type", "folly::IOBuf");
-  EXPECT_EQ(get_type_name(iobuf_type), "folly::IOBuf");
-  EXPECT_EQ(get_standard_type_name(iobuf_type), "folly::IOBuf");
-  EXPECT_TRUE(can_resolve_to_scalar(iobuf_type));
-
-  // Then adapt it:
-  //   binary (cpp.type="folly::IOBuf", cpp.adapter="MyAdapter1")
-  t_base_type adapated_type = iobuf_type;
-  adapated_type.set_annotation("cpp.adapter", "MyAdapter1");
-  EXPECT_EQ(
-      get_type_name(adapated_type),
-      "::apache::thrift::adapt_detail::adapted_t<MyAdapter1, folly::IOBuf>");
-  EXPECT_EQ(get_standard_type_name(adapated_type), "folly::IOBuf");
-  EXPECT_TRUE(can_resolve_to_scalar(adapated_type));
-
-  // Give the adapated type an alias:
-  //   typedef binary (cpp.type="folly::IOBuf", cpp.adapter="MyAdapter2")
-  //       AdaptedTypeTypeDef
-  t_typedef adapted_type_typedef(
-      &program_, "AdaptedTypeTypeDef", adapated_type);
-  EXPECT_EQ(
-      get_type_name(adapted_type_typedef), "::path::to::AdaptedTypeTypeDef");
-  EXPECT_EQ(get_standard_type_name(adapted_type_typedef), "folly::IOBuf");
-  EXPECT_TRUE(can_resolve_to_scalar(adapted_type_typedef));
-
-  // Use the alias for the adapted type for field 1.
-  // 1: AdaptedTypeTypeDef field1
-  t_field field1(adapted_type_typedef, "field1", 1);
-  EXPECT_EQ(get_storage_type_name(field1), "::path::to::AdaptedTypeTypeDef");
-  EXPECT_TRUE(can_resolve_to_scalar(*field1.type()));
-
-  // Give the type an alias, then adapt the alias:
-  //   typedef binary (cpp.type="folly::IOBuf") AdaptedTypeDef
-  //       (cpp.adapter="MyAdapter")
-  //
-  // Moving the adapter annotation to the typedef itself (or any user defined
-  // type), is somewhat nonsensical and means the type should be defined
-  // normally, but all uses of the type should be adapted.
-  t_typedef adapted_typedef(&program_, "AdaptedTypeDef", iobuf_type);
-  adapted_typedef.set_annotation("cpp.adapter", "MyAdapter2");
-  EXPECT_EQ(
-      get_type_name(adapted_typedef),
-      "::apache::thrift::adapt_detail::adapted_t<MyAdapter2, ::path::to::AdaptedTypeDef>");
-  EXPECT_EQ(get_standard_type_name(adapted_typedef), "folly::IOBuf");
-  EXPECT_TRUE(can_resolve_to_scalar(adapted_typedef));
-
-  // 2: AdaptedTypeDef field2
-  t_field field2(adapted_typedef, "field2", 2);
-  EXPECT_EQ(
-      get_storage_type_name(field2),
-      "::apache::thrift::adapt_detail::adapted_t<MyAdapter2, ::path::to::AdaptedTypeDef>");
-  EXPECT_TRUE(can_resolve_to_scalar(*field2.type()));
-}
-
 TEST_F(TypeResolverTest, AdaptedFieldType) {
   auto i64 = t_base_type::t_i64();
   auto field = t_field(i64, "n", 42);
   field.add_structured_annotation(
       adapter_builder(program_, "cpp").make("MyAdapter"));
   EXPECT_EQ(
-      get_type_name(field),
+      get_native_type(field),
       "::apache::thrift::adapt_detail::adapted_field_t<"
       "MyAdapter, 42, ::std::int64_t, ThriftStruct>");
   EXPECT_EQ(
-      get_storage_type_name(field),
+      get_storage_type(field),
       "::apache::thrift::adapt_detail::adapted_field_t<"
       "MyAdapter, 42, ::std::int64_t, ThriftStruct>");
 }
@@ -618,7 +527,7 @@ TEST_F(TypeResolverTest, AdaptedFieldStorageType) {
     auto field = t_field(i64, "n", 42);
     field.add_structured_annotation(adapter.make("MyAdapter"));
     EXPECT_EQ(
-        get_storage_type_name(field),
+        get_storage_type(field),
         "::apache::thrift::adapt_detail::adapted_field_t<"
         "MyAdapter, 42, ::std::int64_t, ThriftStruct>");
   }
@@ -628,7 +537,7 @@ TEST_F(TypeResolverTest, AdaptedFieldStorageType) {
     field.add_structured_annotation(adapter.make("MyAdapter"));
     field.set_annotation("cpp.ref", "");
     EXPECT_EQ(
-        get_storage_type_name(field),
+        get_storage_type(field),
         "::std::unique_ptr<::apache::thrift::adapt_detail::adapted_field_t<"
         "MyAdapter, 42, ::std::int64_t, ThriftStruct>>");
   }
@@ -637,7 +546,7 @@ TEST_F(TypeResolverTest, AdaptedFieldStorageType) {
     field.add_structured_annotation(adapter.make("MyAdapter"));
     field.set_annotation("cpp2.ref", ""); // Works with cpp2.
     EXPECT_EQ(
-        get_storage_type_name(field),
+        get_storage_type(field),
         "::std::unique_ptr<::apache::thrift::adapt_detail::adapted_field_t<"
         "MyAdapter, 42, ::std::int64_t, ThriftStruct>>");
   }
@@ -647,7 +556,7 @@ TEST_F(TypeResolverTest, AdaptedFieldStorageType) {
     field.add_structured_annotation(adapter.make("MyAdapter"));
     field.set_annotation("cpp.ref_type", "unique");
     EXPECT_EQ(
-        get_storage_type_name(field),
+        get_storage_type(field),
         "::std::unique_ptr<::apache::thrift::adapt_detail::adapted_field_t<"
         "MyAdapter, 42, ::std::int64_t, ThriftStruct>>");
   }
@@ -657,7 +566,7 @@ TEST_F(TypeResolverTest, AdaptedFieldStorageType) {
     field.add_structured_annotation(adapter.make("MyAdapter"));
     field.set_annotation("cpp.ref_type", "shared");
     EXPECT_EQ(
-        get_storage_type_name(field),
+        get_storage_type(field),
         "::std::shared_ptr<::apache::thrift::adapt_detail::adapted_field_t<"
         "MyAdapter, 42, ::std::int64_t, ThriftStruct>>");
   }
@@ -667,7 +576,7 @@ TEST_F(TypeResolverTest, AdaptedFieldStorageType) {
     field.add_structured_annotation(adapter.make("MyAdapter"));
     field.set_annotation("cpp.ref_type", "shared_mutable");
     EXPECT_EQ(
-        get_storage_type_name(field),
+        get_storage_type(field),
         "::std::shared_ptr<::apache::thrift::adapt_detail::adapted_field_t<"
         "MyAdapter, 42, ::std::int64_t, ThriftStruct>>");
   }
@@ -677,7 +586,7 @@ TEST_F(TypeResolverTest, AdaptedFieldStorageType) {
     field.add_structured_annotation(adapter.make("MyAdapter"));
     field.set_annotation("cpp.ref_type", "shared_const");
     EXPECT_EQ(
-        get_storage_type_name(field),
+        get_storage_type(field),
         "::std::shared_ptr<const ::apache::thrift::adapt_detail::adapted_field_t<"
         "MyAdapter, 42, ::std::int64_t, ThriftStruct>>");
   }
@@ -687,7 +596,7 @@ TEST_F(TypeResolverTest, AdaptedFieldStorageType) {
     field.add_structured_annotation(adapter.make("MyAdapter"));
     field.set_annotation("thrift.box", "");
     EXPECT_EQ(
-        get_storage_type_name(field),
+        get_storage_type(field),
         "::apache::thrift::detail::boxed_value_ptr<::apache::thrift::adapt_detail::adapted_field_t<"
         "MyAdapter, 42, ::std::int64_t, ThriftStruct>>");
   }
@@ -698,7 +607,7 @@ TEST_F(TypeResolverTest, AdaptedFieldStorageType) {
     field.add_structured_annotation(
         thrift_annotation_builder::box(program_).make());
     EXPECT_EQ(
-        get_storage_type_name(field),
+        get_storage_type(field),
         "::apache::thrift::detail::boxed_value_ptr<::apache::thrift::adapt_detail::adapted_field_t<"
         "MyAdapter, 42, ::std::int64_t, ThriftStruct>>");
   }
@@ -707,7 +616,7 @@ TEST_F(TypeResolverTest, AdaptedFieldStorageType) {
     auto field = t_field(i64, "n", 42);
     field.add_structured_annotation(adapter.make("MyAdapter"));
     field.set_annotation("cpp.ref_type", "blah");
-    EXPECT_THROW(get_storage_type_name(field), std::runtime_error);
+    EXPECT_THROW(get_storage_type(field), std::runtime_error);
   }
 }
 
@@ -727,7 +636,7 @@ TEST_F(TypeResolverTest, TransitivelyAdaptedFieldType) {
   field1.add_structured_annotation(
       std::make_unique<t_const>(&program_, &annotation, "", nullptr));
   EXPECT_EQ(
-      get_storage_type_name(field1),
+      get_storage_type(field1),
       "::apache::thrift::adapt_detail::adapted_field_t<"
       "MyAdapter, 1, ::std::int64_t, ThriftStruct>");
 
@@ -735,7 +644,7 @@ TEST_F(TypeResolverTest, TransitivelyAdaptedFieldType) {
   field2.add_structured_annotation(
       std::make_unique<t_const>(&program_, &annotation, "", nullptr));
   EXPECT_EQ(
-      get_storage_type_name(field2),
+      get_storage_type(field2),
       "::apache::thrift::adapt_detail::adapted_field_t<"
       "MyAdapter, 42, ::std::int64_t, ThriftStruct>");
 }
