@@ -24,7 +24,7 @@
 
 #include <stdlib.h>
 #include <sstream>
-#include <thrift/compiler/generate/t_oop_generator.h>
+#include <thrift/compiler/generate/t_concat_generator.h>
 
 using namespace std;
 
@@ -35,29 +35,16 @@ namespace compiler {
 /**
  * JS code generator.
  */
-class t_js_generator : public t_oop_generator {
+class t_js_generator : public t_concat_generator {
  public:
   t_js_generator(
       t_program* program,
       t_generation_context context,
-      const std::map<std::string, std::string>& parsed_options,
-      const std::string& option_string)
-      : t_oop_generator(program, std::move(context)) {
-    (void)option_string;
-
-    std::map<std::string, std::string>::const_iterator iter;
-
-    iter = parsed_options.find("node");
-    gen_node_ = (iter != parsed_options.end());
-
-    iter = parsed_options.find("jquery");
-    gen_jquery_ = (iter != parsed_options.end());
-
-    if (gen_node_) {
-      out_dir_base_ = "gen-nodejs";
-    } else {
-      out_dir_base_ = "gen-js";
-    }
+      const std::map<std::string, std::string>& options)
+      : t_concat_generator(program, std::move(context)) {
+    gen_node_ = options.find("node") != options.end();
+    gen_jquery_ = options.find("jquery") != options.end();
+    out_dir_base_ = gen_node_ ? "gen-nodejs" : "gen-js";
   }
 
   /**
@@ -173,10 +160,11 @@ class t_js_generator : public t_oop_generator {
       const t_struct* tstruct, bool include_callback = false);
   std::string type_to_enum(const t_type* ttype);
 
-  std::string autogen_comment() override {
+  std::string autogen_comment() {
     return std::string("//\n") + "// Autogenerated by Thrift Compiler\n" +
         "//\n" +
-        "// DO NOT EDIT UNLESS YOU ARE SURE THAT YOU KNOW WHAT YOU ARE DOING\n" +
+        "// DO NOT EDIT UNLESS YOU ARE SURE THAT YOU KNOW WHAT YOU ARE "
+        "DOING\n" +
         "//\n";
   }
 
@@ -815,25 +803,28 @@ void t_js_generator::generate_service_processor(const t_service* tservice) {
 
   scope_up(f_service_);
 
-  f_service_
-      << indent() << "var r = input.readMessageBegin();" << endl
-      << indent() << "if (this['process_' + r.fname]) {" << endl
-      << indent()
-      << "  return this['process_' + r.fname].call(this, r.rseqid, input, output);"
-      << endl
-      << indent() << "} else {" << endl
-      << indent() << "  input.skip(Thrift.Type.STRUCT);" << endl
-      << indent() << "  input.readMessageEnd();" << endl
-      << indent()
-      << "  var x = new Thrift.TApplicationException(Thrift.TApplicationExceptionType.UNKNOWN_METHOD, 'Unknown function ' + r.fname);"
-      << endl
-      << indent()
-      << "  output.writeMessageBegin(r.fname, Thrift.MessageType.Exception, r.rseqid);"
-      << endl
-      << indent() << "  x.write(output);" << endl
-      << indent() << "  output.writeMessageEnd();" << endl
-      << indent() << "  output.flush();" << endl
-      << indent() << "}" << endl;
+  f_service_ << indent() << "var r = input.readMessageBegin();" << endl
+             << indent() << "if (this['process_' + r.fname]) {" << endl
+             << indent()
+             << "  return this['process_' + r.fname].call(this, r.rseqid, "
+                "input, output);"
+             << endl
+             << indent() << "} else {" << endl
+             << indent() << "  input.skip(Thrift.Type.STRUCT);" << endl
+             << indent() << "  input.readMessageEnd();" << endl
+             << indent()
+             << "  var x = new "
+                "Thrift.TApplicationException(Thrift.TApplicationExceptionType."
+                "UNKNOWN_METHOD, 'Unknown function ' + r.fname);"
+             << endl
+             << indent()
+             << "  output.writeMessageBegin(r.fname, "
+                "Thrift.MessageType.Exception, r.rseqid);"
+             << endl
+             << indent() << "  x.write(output);" << endl
+             << indent() << "  output.writeMessageEnd();" << endl
+             << indent() << "  output.flush();" << endl
+             << indent() << "}" << endl;
 
   scope_down(f_service_);
   f_service_ << endl;
@@ -1382,7 +1373,8 @@ void t_js_generator::generate_deserialize_container(
     if (!gen_node_) {
       out << indent() << "if (" << i << " > 0 ) {" << endl
           << indent()
-          << "  if (input.rstack.length > input.rpos[input.rpos.length -1] + 1) {"
+          << "  if (input.rstack.length > input.rpos[input.rpos.length -1] + "
+             "1) {"
           << endl
           << indent() << "    input.rstack.pop();" << endl
           << indent() << "  }" << endl
