@@ -25,7 +25,7 @@
 
 #include <thrift/compiler/detail/mustache/mstch.h>
 
-#include <thrift/compiler/filesystem.h>
+#include <thrift/compiler/detail/system.h>
 #include <thrift/compiler/generate/t_generator.h>
 #include <thrift/compiler/generate/templates.h>
 
@@ -64,19 +64,18 @@ t_mstch_generator::t_mstch_generator(
     t_program* program,
     t_generation_context context,
     boost::filesystem::path template_prefix,
-    std::map<std::string, std::string> parsed_options,
+    std::map<std::string, std::string> options,
     bool convert_delimiter)
     : t_generator(program, std::move(context)),
-      parsed_options_(std::move(parsed_options)),
+      options_(std::move(options)),
       convert_delimiter_(convert_delimiter),
-      generators_(std::make_shared<mstch_generators>()),
       cache_(std::make_shared<mstch_cache>()) {
-  cache_->parsed_options_ = parsed_options_;
+  cache_->options_ = options_;
   gen_template_map(template_prefix);
 }
 
 t_mstch_generator::~t_mstch_generator() {
-  // The cache can hold circular references to contexts and other generators,
+  // The cache can hold circular references to contexts and other factories,
   // which results in a memory leak on cleanup. Therefore, we must explicitly
   // clear the cache when we are destroyed, to break the cycles. (Hopefully
   // nobody has kept a reference to us, or this will never be called!)
@@ -290,7 +289,7 @@ mstch::map t_mstch_generator::dump(const t_const_value& value) {
       {"container?", type == cv::CV_MAP || type == cv::CV_LIST},
   };
 
-  auto const format_double_string = [](const double d) {
+  const auto format_double_string = [](const double d) {
     std::ostringstream oss;
     oss << std::setprecision(std::numeric_limits<double>::digits10) << d;
     return oss.str();
@@ -376,7 +375,7 @@ mstch::map t_mstch_generator::dump(const string& value) {
 
 mstch::map t_mstch_generator::dump_options() {
   mstch::map result;
-  for (auto& elem : parsed_options_) {
+  for (auto& elem : options_) {
     result.emplace(elem.first, elem.second);
   }
   return prepend_prefix("option", std::move(result));
@@ -476,7 +475,7 @@ const std::string& t_mstch_generator::get_template(
 void t_mstch_generator::write_output(
     const boost::filesystem::path& path, const std::string& data) {
   auto base_path = boost::filesystem::path{get_out_dir()};
-  auto abs_path = make_abs_path(base_path, path);
+  auto abs_path = detail::make_abs_path(base_path, path);
   boost::filesystem::create_directories(abs_path.parent_path());
   std::ofstream ofs{abs_path.string()};
   if (!ofs) {
@@ -493,12 +492,12 @@ void t_mstch_generator::write_output(
 }
 
 bool t_mstch_generator::has_option(const std::string& option) const {
-  return parsed_options_.find(option) != parsed_options_.end();
+  return options_.find(option) != options_.end();
 }
 
 std::string t_mstch_generator::get_option(const std::string& option) {
-  auto itr = parsed_options_.find(option);
-  if (itr != parsed_options_.end()) {
+  auto itr = options_.find(option);
+  if (itr != options_.end()) {
     return itr->second;
   }
   return {};
@@ -527,15 +526,15 @@ void t_mstch_generator::render_to_file(
 }
 
 const std::shared_ptr<mstch_base>& t_mstch_generator::cached_program(
-    t_program const* program) {
+    const t_program* program) {
   const auto& id = program->path();
   auto itr = cache_->programs_.find(id);
   if (itr == cache_->programs_.end()) {
     itr = cache_->programs_
               .emplace(
                   id,
-                  generators_->program_generator_->generate(
-                      program, generators_, cache_))
+                  factories_.program_factory->make_mstch_object(
+                      program, factories_, cache_))
               .first;
   }
   return itr->second;
