@@ -25,26 +25,17 @@ class t_mstch_html_generator : public t_mstch_generator {
   t_mstch_html_generator(
       t_program* program,
       t_generation_context context,
-      const std::map<std::string, std::string>& parsed_options,
-      const std::string& option_string);
+      const std::map<std::string, std::string>& options)
+      : t_mstch_generator(program, std::move(context), "html", options) {
+    this->out_dir_base_ = "gen-mstch_html";
+  }
 
-  void generate_program() override;
+  void generate_program() override {
+    // Generate index.html.
+    render_to_file(*this->get_program(), "index.html", "index.html");
+  }
 };
 
-t_mstch_html_generator::t_mstch_html_generator(
-    t_program* program,
-    t_generation_context context,
-    const std::map<std::string, std::string>& parsed_options,
-    const std::string& /* option_string */)
-    : t_mstch_generator(program, std::move(context), "html", parsed_options) {
-  this->out_dir_base_ = "gen-mstch_html";
-}
-
-void t_mstch_html_generator::generate_program() {
-  // Generate index.html
-  render_to_file(*this->get_program(), "index.html", "index.html");
-}
-
 THRIFT_REGISTER_GENERATOR(mstch_html, "HTML", "");
 
 } // namespace compiler
