@@ -20,13 +20,48 @@ namespace apache {
 namespace thrift {
 namespace compiler {
 
+std::shared_ptr<mstch_base> make_mstch_program_cached(
+    const t_program* program,
+    const mstch_factories& factories,
+    std::shared_ptr<mstch_cache> cache,
+    mstch_element_position pos) {
+  const auto& id = program->path();
+  auto itr = cache->programs_.find(id);
+  if (itr == cache->programs_.end()) {
+    itr = cache->programs_.emplace_hint(
+        itr,
+        id,
+        factories.program_factory->make_mstch_object(
+            program, factories, cache, pos));
+  }
+  return itr->second;
+}
+
+std::shared_ptr<mstch_base> make_mstch_service_cached(
+    const t_program* program,
+    const t_service* service,
+    const mstch_factories& factories,
+    std::shared_ptr<mstch_cache> cache,
+    mstch_element_position pos) {
+  std::string service_id = program->path() + service->get_name();
+  auto itr = cache->services_.find(service_id);
+  if (itr == cache->services_.end()) {
+    itr = cache->services_.emplace_hint(
+        itr,
+        service_id,
+        factories.service_factory->make_mstch_object(
+            service, factories, cache, pos));
+  }
+  return itr->second;
+}
+
 bool mstch_base::has_option(const std::string& option) const {
-  return cache_->parsed_options_.find(option) != cache_->parsed_options_.end();
+  return cache_->options_.find(option) != cache_->options_.end();
 }
 
 std::string mstch_base::get_option(const std::string& option) const {
-  auto itr = cache_->parsed_options_.find(option);
-  if (itr != cache_->parsed_options_.end()) {
+  auto itr = cache_->options_.find(option);
+  if (itr != cache_->options_.end()) {
     return itr->second;
   }
   return {};
@@ -43,157 +78,32 @@ mstch::node mstch_base::is_struct() {
   return dynamic_cast<mstch_struct*>(this) != nullptr;
 }
 
-std::shared_ptr<mstch_base> enum_value_generator::generate(
-    t_enum_value const* enum_value,
-    std::shared_ptr<mstch_generators const> generators,
-    std::shared_ptr<mstch_cache> cache,
-    ELEMENT_POSITION pos,
-    int32_t /*index*/) const {
-  return std::make_shared<mstch_enum_value>(enum_value, generators, cache, pos);
-}
-
-std::shared_ptr<mstch_base> enum_generator::generate(
-    t_enum const* enm,
-    std::shared_ptr<mstch_generators const> generators,
-    std::shared_ptr<mstch_cache> cache,
-    ELEMENT_POSITION pos,
-    int32_t /*index*/) const {
-  return std::make_shared<mstch_enum>(enm, generators, cache, pos);
-}
-
-std::shared_ptr<mstch_base> const_value_generator::generate(
-    t_const_value const* const_value,
-    std::shared_ptr<mstch_generators const> generators,
-    std::shared_ptr<mstch_cache> cache,
-    ELEMENT_POSITION pos,
-    int32_t index,
-    t_const const* current_const,
-    t_type const* expected_type) const {
-  return std::make_shared<mstch_const_value>(
-      const_value, current_const, expected_type, generators, cache, pos, index);
-}
-
-std::shared_ptr<mstch_base> const_value_generator::generate(
-    std::pair<t_const_value*, t_const_value*> const& value_pair,
-    std::shared_ptr<mstch_generators const> generators,
-    std::shared_ptr<mstch_cache> cache,
-    ELEMENT_POSITION pos,
-    int32_t index,
-    t_const const* current_const,
-    std::pair<const t_type*, const t_type*> const& expected_types) const {
-  return std::make_shared<mstch_const_value_key_mapped_pair>(
-      value_pair, current_const, expected_types, generators, cache, pos, index);
-}
-
-std::shared_ptr<mstch_base> type_generator::generate(
-    t_type const* type,
-    std::shared_ptr<mstch_generators const> generators,
-    std::shared_ptr<mstch_cache> cache,
-    ELEMENT_POSITION pos,
-    int32_t /*index*/) const {
-  return std::make_shared<mstch_type>(type, generators, cache, pos);
-}
-
-std::shared_ptr<mstch_base> field_generator::generate(
-    t_field const* field,
-    std::shared_ptr<mstch_generators const> generators,
-    std::shared_ptr<mstch_cache> cache,
-    ELEMENT_POSITION pos,
-    int32_t index,
-    field_generator_context const* field_context) const {
-  return std::make_shared<mstch_field>(
-      field, generators, cache, pos, index, field_context);
-}
-
-std::shared_ptr<mstch_base> annotation_generator::generate(
-    const t_annotation& annotation,
-    std::shared_ptr<mstch_generators const> generators,
-    std::shared_ptr<mstch_cache> cache,
-    ELEMENT_POSITION pos,
-    int32_t index) const {
-  return std::make_shared<mstch_annotation>(
-      annotation.first, annotation.second, generators, cache, pos, index);
-}
-
-std::shared_ptr<mstch_base> structured_annotation_generator::generate(
-    const t_const* annotValue,
-    std::shared_ptr<mstch_generators const> generators,
-    std::shared_ptr<mstch_cache> cache,
-    ELEMENT_POSITION pos,
-    int32_t index) const {
-  return std::make_shared<mstch_structured_annotation>(
-      *annotValue, generators, cache, pos, index);
-}
-
-std::shared_ptr<mstch_base> struct_generator::generate(
-    t_struct const* strct,
-    std::shared_ptr<mstch_generators const> generators,
-    std::shared_ptr<mstch_cache> cache,
-    ELEMENT_POSITION pos,
-    int32_t /*index*/) const {
-  return std::make_shared<mstch_struct>(strct, generators, cache, pos);
-}
-
-std::shared_ptr<mstch_base> function_generator::generate(
-    t_function const* function,
-    std::shared_ptr<mstch_generators const> generators,
-    std::shared_ptr<mstch_cache> cache,
-    ELEMENT_POSITION pos,
-    int32_t /*index*/) const {
-  return std::make_shared<mstch_function>(function, generators, cache, pos);
-}
-
-std::shared_ptr<mstch_base> service_generator::generate(
-    t_service const* service,
-    std::shared_ptr<mstch_generators const> generators,
-    std::shared_ptr<mstch_cache> cache,
-    ELEMENT_POSITION pos,
-    int32_t /*index*/) const {
-  return std::make_shared<mstch_service>(service, generators, cache, pos);
-}
-
-std::shared_ptr<mstch_base> typedef_generator::generate(
-    t_typedef const* typedf,
-    std::shared_ptr<mstch_generators const> generators,
-    std::shared_ptr<mstch_cache> cache,
-    ELEMENT_POSITION pos,
-    int32_t /*index*/) const {
-  return std::make_shared<mstch_typedef>(typedf, generators, cache, pos);
-}
-
-std::shared_ptr<mstch_base> const_generator::generate(
-    t_const const* cnst,
-    std::shared_ptr<mstch_generators const> generators,
-    std::shared_ptr<mstch_cache> cache,
-    ELEMENT_POSITION pos,
-    int32_t index,
-    t_const const* current_const,
-    t_type const* expected_type,
-    t_field const* field) const {
-  return std::make_shared<mstch_const>(
-      cnst, current_const, expected_type, generators, cache, pos, index, field);
-}
-
-std::shared_ptr<mstch_base> program_generator::generate(
-    t_program const* program,
-    std::shared_ptr<mstch_generators const> generators,
-    std::shared_ptr<mstch_cache> cache,
-    ELEMENT_POSITION pos,
-    int32_t /*index*/) const {
-  return std::make_shared<mstch_program>(program, generators, cache, pos);
+mstch_factories::mstch_factories() {
+  set_program_factory<mstch_program>();
+  set_type_factory<mstch_type>();
+  set_typedef_factory<mstch_typedef>();
+  set_struct_factory<mstch_struct>();
+  set_field_factory<mstch_field>();
+  set_enum_factory<mstch_enum>();
+  set_enum_value_factory<mstch_enum_value>();
+  set_const_factory<mstch_const>();
+  set_const_value_factory<mstch_const_value>();
+  set_const_map_element_factory<mstch_const_map_element>();
+  set_structured_annotation_factory<mstch_structured_annotation>();
+  set_deprecated_annotation_factory<mstch_deprecated_annotation>();
 }
 
 mstch::node mstch_enum::values() {
-  return generate_enum_values(enm_->get_enum_values());
+  return make_mstch_enum_values(enum_->get_enum_values());
 }
 
 mstch::node mstch_type::get_struct() {
   if (type_->is_struct() || type_->is_xception()) {
     std::string id =
         type_->program()->name() + get_type_namespace(type_->program());
-    return generate_elements_cached(
-        std::vector<t_struct const*>{dynamic_cast<t_struct const*>(type_)},
-        generators_->struct_generator_.get(),
+    return make_mstch_array_cached(
+        std::vector<const t_struct*>{dynamic_cast<const t_struct*>(type_)},
+        *factories_.struct_factory,
         cache_->structs_,
         id);
   }
@@ -204,9 +114,9 @@ mstch::node mstch_type::get_enum() {
   if (resolved_type_->is_enum()) {
     std::string id =
         type_->program()->name() + get_type_namespace(type_->program());
-    return generate_elements_cached(
-        std::vector<t_enum const*>{dynamic_cast<t_enum const*>(resolved_type_)},
-        generators_->enum_generator_.get(),
+    return make_mstch_array_cached(
+        std::vector<const t_enum*>{dynamic_cast<const t_enum*>(resolved_type_)},
+        *factories_.enum_factory,
         cache_->enums_,
         id);
   }
@@ -215,9 +125,9 @@ mstch::node mstch_type::get_enum() {
 
 mstch::node mstch_type::get_list_type() {
   if (resolved_type_->is_list()) {
-    return generators_->type_generator_->generate(
+    return factories_.type_factory->make_mstch_object(
         dynamic_cast<const t_list*>(resolved_type_)->get_elem_type(),
-        generators_,
+        factories_,
         cache_,
         pos_);
   }
@@ -226,9 +136,9 @@ mstch::node mstch_type::get_list_type() {
 
 mstch::node mstch_type::get_set_type() {
   if (resolved_type_->is_set()) {
-    return generators_->type_generator_->generate(
+    return factories_.type_factory->make_mstch_object(
         dynamic_cast<const t_set*>(resolved_type_)->get_elem_type(),
-        generators_,
+        factories_,
         cache_,
         pos_);
   }
@@ -237,9 +147,9 @@ mstch::node mstch_type::get_set_type() {
 
 mstch::node mstch_type::get_key_type() {
   if (resolved_type_->is_map()) {
-    return generators_->type_generator_->generate(
+    return factories_.type_factory->make_mstch_object(
         dynamic_cast<const t_map*>(resolved_type_)->get_key_type(),
-        generators_,
+        factories_,
         cache_,
         pos_);
   }
@@ -248,9 +158,9 @@ mstch::node mstch_type::get_key_type() {
 
 mstch::node mstch_type::get_value_type() {
   if (resolved_type_->is_map()) {
-    return generators_->type_generator_->generate(
+    return factories_.type_factory->make_mstch_object(
         dynamic_cast<const t_map*>(resolved_type_)->get_val_type(),
-        generators_,
+        factories_,
         cache_,
         pos_);
   }
@@ -259,9 +169,9 @@ mstch::node mstch_type::get_value_type() {
 
 mstch::node mstch_type::get_typedef_type() {
   if (type_->is_typedef()) {
-    return generators_->type_generator_->generate(
+    return factories_.type_factory->make_mstch_object(
         dynamic_cast<const t_typedef*>(type_)->get_type(),
-        generators_,
+        factories_,
         cache_,
         pos_);
   }
@@ -270,17 +180,17 @@ mstch::node mstch_type::get_typedef_type() {
 
 mstch::node mstch_type::get_typedef() {
   if (type_->is_typedef()) {
-    return generators_->typedef_generator_->generate(
-        dynamic_cast<const t_typedef*>(type_), generators_, cache_, pos_);
+    return factories_.typedef_factory->make_mstch_object(
+        dynamic_cast<const t_typedef*>(type_), factories_, cache_, pos_);
   }
   return mstch::node();
 }
 
 mstch::node mstch_type::get_sink_elem_type() {
   if (type_->is_sink()) {
-    return generators_->type_generator_->generate(
+    return factories_.type_factory->make_mstch_object(
         dynamic_cast<const t_sink*>(type_)->get_sink_type(),
-        generators_,
+        factories_,
         cache_,
         pos_);
   }
@@ -289,9 +199,9 @@ mstch::node mstch_type::get_sink_elem_type() {
 
 mstch::node mstch_type::get_sink_final_reponse_type() {
   if (type_->is_sink()) {
-    return generators_->type_generator_->generate(
+    return factories_.type_factory->make_mstch_object(
         dynamic_cast<const t_sink*>(type_)->get_final_response_type(),
-        generators_,
+        factories_,
         cache_,
         pos_);
   }
@@ -302,8 +212,8 @@ mstch::node mstch_type::get_sink_first_response_type() {
   if (type_->is_sink()) {
     if (const auto sinkresponse =
             dynamic_cast<const t_sink*>(type_)->get_first_response_type()) {
-      return generators_->type_generator_->generate(
-          sinkresponse, generators_, cache_, pos_);
+      return factories_.type_factory->make_mstch_object(
+          sinkresponse, factories_, cache_, pos_);
     }
   }
   return mstch::node();
@@ -311,9 +221,9 @@ mstch::node mstch_type::get_sink_first_response_type() {
 
 mstch::node mstch_type::get_stream_elem_type() {
   if (type_->is_streamresponse()) {
-    return generators_->type_generator_->generate(
+    return factories_.type_factory->make_mstch_object(
         dynamic_cast<const t_stream_response*>(type_)->get_elem_type(),
-        generators_,
+        factories_,
         cache_,
         pos_);
   }
@@ -325,39 +235,37 @@ mstch::node mstch_type::get_stream_first_response_type() {
     if (const auto streamresponse =
             dynamic_cast<const t_stream_response*>(resolved_type_)
                 ->get_first_response_type()) {
-      return generators_->type_generator_->generate(
-          streamresponse, generators_, cache_, pos_);
+      return factories_.type_factory->make_mstch_object(
+          streamresponse, factories_, cache_, pos_);
     }
   }
   return mstch::node();
 }
 
 mstch::node mstch_field::value() {
-  if (field_->get_value()) {
-    return generators_->const_value_generator_->generate(
-        field_->get_value(), generators_, cache_, pos_);
+  if (!field_->get_value()) {
+    return mstch::node();
   }
-  return mstch::node();
+  return factories_.const_value_factory->make_mstch_object(
+      field_->get_value(), factories_, cache_, pos_, nullptr, nullptr);
 }
 
-mstch::node mstch_const_value_key_mapped_pair::element_key() {
-  return generators_->const_value_generator_->generate(
-      pair_.first,
-      generators_,
+mstch::node mstch_const_map_element::element_key() {
+  return factories_.const_value_factory->make_mstch_object(
+      element_.first,
+      factories_,
       cache_,
       pos_,
-      index_,
       current_const_,
       expected_types_.first);
 }
 
-mstch::node mstch_const_value_key_mapped_pair::element_value() {
-  return generators_->const_value_generator_->generate(
-      pair_.second,
-      generators_,
+mstch::node mstch_const_map_element::element_value() {
+  return factories_.const_value_factory->make_mstch_object(
+      element_.second,
+      factories_,
       cache_,
       pos_,
-      index_,
       current_const_,
       expected_types_.second);
 }
@@ -419,7 +327,8 @@ mstch::node mstch_const_value::enum_name() {
 }
 
 mstch::node mstch_const_value::enum_value_name() {
-  if (type_ == cv::CV_INTEGER && const_value_->is_enum()) {
+  if (type_ == cv::CV_INTEGER && const_value_->is_enum() &&
+      const_value_->get_enum_value() != nullptr) {
     return const_value_->get_enum_value()->get_name();
   }
   return mstch::node();
@@ -451,23 +360,26 @@ mstch::node mstch_const_value::list_elems() {
             dynamic_cast<const t_set*>(expected_type_)->get_elem_type();
       }
     }
-    return generate_consts(
+    return make_mstch_consts(
         const_value_->get_list(), current_const_, expected_type);
   }
   return mstch::node();
 }
 
 mstch::node mstch_const_value::map_elems() {
-  if (type_ == cv::CV_MAP) {
-    std::pair<const t_type*, const t_type*> expected_types;
-    if (expected_type_ && expected_type_->is_map()) {
-      const auto* m = dynamic_cast<const t_map*>(expected_type_);
-      expected_types = {m->get_key_type(), m->get_val_type()};
-    }
-    return generate_consts(
-        const_value_->get_map(), current_const_, expected_types);
+  if (type_ != cv::CV_MAP) {
+    return mstch::node();
   }
-  return mstch::node();
+  std::pair<const t_type*, const t_type*> expected_types;
+  if (expected_type_ && expected_type_->is_map()) {
+    const auto* m = dynamic_cast<const t_map*>(expected_type_);
+    expected_types = {m->get_key_type(), m->get_val_type()};
+  }
+  return make_mstch_array(
+      const_value_->get_map(),
+      *factories_.const_map_element_factory,
+      current_const_,
+      expected_types);
 }
 
 mstch::node mstch_const_value::is_const_struct() {
@@ -485,7 +397,7 @@ mstch::node mstch_const_value::const_struct_type() {
 
   const auto* type = const_value_->ttype()->get_true_type();
   if (type->is_struct() || type->is_xception()) {
-    return generators_->type_generator_->generate(type, generators_, cache_);
+    return factories_.type_factory->make_mstch_object(type, factories_, cache_);
   }
 
   return {};
@@ -498,9 +410,9 @@ mstch::node mstch_const_value::const_struct() {
 
   const auto* type = const_value_->ttype()->get_true_type();
   if (type->is_struct() || type->is_xception()) {
-    auto const* strct = dynamic_cast<t_struct const*>(type);
+    const auto* strct = dynamic_cast<const t_struct*>(type);
     for (auto member : const_value_->get_map()) {
-      auto const* field = strct->get_field_by_name(member.first->get_string());
+      const auto* field = strct->get_field_by_name(member.first->get_string());
       assert(field != nullptr);
       constants.push_back(new t_const(
           nullptr,
@@ -511,14 +423,12 @@ mstch::node mstch_const_value::const_struct() {
     }
   }
 
-  for (size_t i = 0; i < constants.size(); ++i) {
-    auto pos = element_position(i, constants.size());
-    a.push_back(generators_->const_generator_->generate(
+  for (size_t i = 0, size = constants.size(); i < size; ++i) {
+    a.push_back(factories_.const_factory->make_mstch_object(
         constants[i],
-        generators_,
+        factories_,
         cache_,
-        pos,
-        fields[i]->get_key(),
+        mstch_element_position(i, size),
         current_const_,
         constants[i]->get_type(),
         fields[i]));
@@ -527,25 +437,31 @@ mstch::node mstch_const_value::const_struct() {
 }
 
 mstch::node mstch_const_value::owning_const() {
-  return generators_->const_generator_->generate(
-      const_value_->get_owner(), generators_, cache_, pos_);
+  return factories_.const_factory->make_mstch_object(
+      const_value_->get_owner(),
+      factories_,
+      cache_,
+      pos_,
+      nullptr,
+      nullptr,
+      nullptr);
 }
 
 mstch::node mstch_field::type() {
-  return generators_->type_generator_->generate(
-      field_->get_type(), generators_, cache_, pos_);
+  return factories_.type_factory->make_mstch_object(
+      field_->get_type(), factories_, cache_, pos_);
 }
 
 mstch::node mstch_struct::fields() {
-  return generate_fields(strct_->get_members());
+  return make_mstch_fields(struct_->get_members());
 }
 
 mstch::node mstch_struct::exception_safety() {
-  if (!strct_->is_xception()) {
+  if (!struct_->is_xception()) {
     return std::string("");
   }
 
-  const auto* t_ex_ptr = dynamic_cast<const t_exception*>(strct_);
+  const auto* t_ex_ptr = dynamic_cast<const t_exception*>(struct_);
   auto s = t_ex_ptr->safety();
 
   switch (s) {
@@ -557,11 +473,11 @@ mstch::node mstch_struct::exception_safety() {
 }
 
 mstch::node mstch_struct::exception_blame() {
-  if (!strct_->is_xception()) {
+  if (!struct_->is_xception()) {
     return std::string("");
   }
 
-  const auto* t_ex_ptr = dynamic_cast<const t_exception*>(strct_);
+  const auto* t_ex_ptr = dynamic_cast<const t_exception*>(struct_);
   auto s = t_ex_ptr->blame();
 
   switch (s) {
@@ -575,11 +491,11 @@ mstch::node mstch_struct::exception_blame() {
 }
 
 mstch::node mstch_struct::exception_kind() {
-  if (!strct_->is_xception()) {
+  if (!struct_->is_xception()) {
     return std::string("");
   }
 
-  const auto* t_ex_ptr = dynamic_cast<const t_exception*>(strct_);
+  const auto* t_ex_ptr = dynamic_cast<const t_exception*>(struct_);
   auto s = t_ex_ptr->kind();
 
   switch (s) {
@@ -595,12 +511,12 @@ mstch::node mstch_struct::exception_kind() {
 }
 
 const std::vector<const t_field*>& mstch_struct::get_members_in_key_order() {
-  if (strct_->fields().size() == fields_in_key_order_.size()) {
+  if (struct_->fields().size() == fields_in_key_order_.size()) {
     // Already reordered.
     return fields_in_key_order_;
   }
 
-  fields_in_key_order_ = strct_->fields().copy();
+  fields_in_key_order_ = struct_->fields().copy();
   // Sort by increasing key.
   std::sort(
       fields_in_key_order_.begin(),
@@ -613,29 +529,29 @@ const std::vector<const t_field*>& mstch_struct::get_members_in_key_order() {
 }
 
 mstch::node mstch_function::return_type() {
-  return generators_->type_generator_->generate(
-      function_->get_returntype(), generators_, cache_, pos_);
+  return factories_.type_factory->make_mstch_object(
+      function_->get_returntype(), factories_, cache_, pos_);
 }
 
 mstch::node mstch_function::exceptions() {
-  return generate_fields(function_->get_xceptions()->get_members());
+  return make_mstch_fields(function_->get_xceptions()->get_members());
 }
 
 mstch::node mstch_function::stream_exceptions() {
-  return generate_fields(function_->get_stream_xceptions()->get_members());
+  return make_mstch_fields(function_->get_stream_xceptions()->get_members());
 }
 
 mstch::node mstch_function::sink_exceptions() {
-  return generate_fields(function_->get_sink_xceptions()->get_members());
+  return make_mstch_fields(function_->get_sink_xceptions()->get_members());
 }
 
 mstch::node mstch_function::sink_final_response_exceptions() {
-  return generate_fields(
+  return make_mstch_fields(
       function_->get_sink_final_response_xceptions()->get_members());
 }
 
 mstch::node mstch_function::arg_list() {
-  return generate_fields(function_->get_paramlist()->get_members());
+  return make_mstch_fields(function_->get_paramlist()->get_members());
 }
 
 mstch::node mstch_function::returns_stream() {
@@ -643,50 +559,43 @@ mstch::node mstch_function::returns_stream() {
 }
 
 mstch::node mstch_service::functions() {
-  return generate_functions(get_functions());
+  return make_mstch_functions(get_functions());
 }
 
 mstch::node mstch_service::extends() {
-  auto const* extends = service_->get_extends();
+  const auto* extends = service_->get_extends();
   if (extends) {
-    return generate_cached_extended_service(extends);
+    return make_mstch_extended_service_cached(extends);
   }
   return mstch::node();
 }
 
-mstch::node mstch_service::generate_cached_extended_service(
+mstch::node mstch_service::make_mstch_extended_service_cached(
     const t_service* service) {
   std::string id =
       service->program()->name() + get_service_namespace(service->program());
-  size_t element_index = 0;
-  size_t element_count = 1;
-  return generate_element_cached(
-      service,
-      generators_->service_generator_.get(),
-      cache_->services_,
-      id,
-      element_index,
-      element_count);
+  return make_mstch_element_cached(
+      service, *factories_.service_factory, cache_->services_, id, 0, 1);
 }
 
 mstch::node mstch_typedef::type() {
-  return generators_->type_generator_->generate(
-      typedf_->get_type(), generators_, cache_, pos_);
+  return factories_.type_factory->make_mstch_object(
+      typedef_->get_type(), factories_, cache_, pos_);
 }
 
 mstch::node mstch_const::type() {
-  return generators_->type_generator_->generate(
-      cnst_->get_type(), generators_, cache_, pos_);
+  return factories_.type_factory->make_mstch_object(
+      const_->get_type(), factories_, cache_, pos_);
 }
 
 mstch::node mstch_const::value() {
-  return generators_->const_value_generator_->generate(
-      cnst_->get_value(), generators_, cache_, pos_, 0, cnst_, expected_type_);
+  return factories_.const_value_factory->make_mstch_object(
+      const_->get_value(), factories_, cache_, pos_, const_, expected_type_);
 }
 
 mstch::node mstch_const::program() {
-  return generators_->program_generator_->generate(
-      cnst_->get_program(), generators_, cache_, pos_);
+  return factories_.program_factory->make_mstch_object(
+      const_->get_program(), factories_, cache_, pos_);
 }
 
 mstch::node mstch_program::has_thrift_uris() {
@@ -700,48 +609,38 @@ mstch::node mstch_program::has_thrift_uris() {
 
 mstch::node mstch_program::structs() {
   std::string id = program_->name() + get_program_namespace(program_);
-  return generate_elements_cached(
-      get_program_objects(),
-      generators_->struct_generator_.get(),
-      cache_->structs_,
-      id);
+  return make_mstch_array_cached(
+      get_program_objects(), *factories_.struct_factory, cache_->structs_, id);
 }
 
 mstch::node mstch_program::enums() {
   std::string id = program_->name() + get_program_namespace(program_);
-  return generate_elements_cached(
-      get_program_enums(),
-      generators_->enum_generator_.get(),
-      cache_->enums_,
-      id);
+  return make_mstch_array_cached(
+      get_program_enums(), *factories_.enum_factory, cache_->enums_, id);
 }
 
 mstch::node mstch_program::services() {
   std::string id = program_->name() + get_program_namespace(program_);
-  return generate_elements_cached(
-      program_->services(),
-      generators_->service_generator_.get(),
-      cache_->services_,
-      id);
+  return make_mstch_array_cached(
+      program_->services(), *factories_.service_factory, cache_->services_, id);
 }
 
 mstch::node mstch_program::typedefs() {
-  return generate_typedefs(program_->typedefs());
+  return make_mstch_typedefs(program_->typedefs());
 }
 
 mstch::node mstch_program::constants() {
   mstch::array a;
   const auto& container = program_->consts();
-  for (size_t i = 0; i < container.size(); ++i) {
-    auto pos = element_position(i, container.size());
-    a.push_back(generators_->const_generator_->generate(
+  for (size_t i = 0, size = container.size(); i < size; ++i) {
+    a.push_back(factories_.const_factory->make_mstch_object(
         container[i],
-        generators_,
+        factories_,
         cache_,
-        pos,
-        i,
+        mstch_element_position(i, size),
         container[i],
-        container[i]->get_type()));
+        container[i]->get_type(),
+        nullptr));
   }
   return a;
 }
