@@ -18,9 +18,11 @@
 #include <boost/algorithm/string/join.hpp>
 #include <boost/algorithm/string/replace.hpp>
 #include <boost/filesystem.hpp>
+#include <fmt/format.h>
 
 #include <thrift/compiler/ast/t_service.h>
 #include <thrift/compiler/gen/cpp/reference_type.h>
+#include <thrift/compiler/gen/cpp/type_resolver.h>
 #include <thrift/compiler/generate/common.h>
 #include <thrift/compiler/generate/t_mstch_generator.h>
 #include <thrift/compiler/generate/t_mstch_objects.h>
@@ -35,8 +37,10 @@ namespace {
 
 std::vector<t_function*> lifecycleFunctions() {
   static t_function onStartServing_{
-      nullptr, t_base_type::t_void(), "onStartServing"},
-      onStopRequested_{nullptr, t_base_type::t_void(), "onStopRequested"};
+      nullptr, t_base_type::t_void(), "onStartServing"};
+  static t_function onStopRequested_{
+      nullptr, t_base_type::t_void(), "onStopRequested"};
+
   return {&onStartServing_, &onStopRequested_};
 }
 
@@ -116,12 +120,12 @@ class mstch_py3_type : public mstch_type {
   };
   mstch_py3_type(
       const t_type* type,
-      std::shared_ptr<const mstch_generators> generators,
+      const mstch_factories& factories,
       std::shared_ptr<mstch_cache> cache,
-      ELEMENT_POSITION const pos,
+      mstch_element_position pos,
       const t_program* prog,
       CachedProperties& cachedProps)
-      : mstch_type(type, generators, cache, pos),
+      : mstch_type(type, factories, cache, pos),
         prog_{prog},
         cachedProps_{cachedProps} {
     strip_cpp_comments_and_newlines(cachedProps_.cppType);
@@ -340,15 +344,14 @@ class mstch_py3_type : public mstch_type {
 };
 
 template <bool ForContainers = false>
-class type_py3_generator : public type_generator {
+class py3_type_factory : public mstch_type_factory {
  public:
-  explicit type_py3_generator(const t_program* prog) : prog_{prog} {}
-  std::shared_ptr<mstch_base> generate(
+  explicit py3_type_factory(const t_program* prog) : prog_{prog} {}
+  std::shared_ptr<mstch_base> make_mstch_object(
       const t_type* type,
-      std::shared_ptr<const mstch_generators> generators,
+      const mstch_factories& factories,
       std::shared_ptr<mstch_cache> cache,
-      ELEMENT_POSITION pos,
-      int32_t index) const override;
+      mstch_element_position pos) const override;
 
  protected:
   const t_program* prog_;
@@ -358,10 +361,10 @@ class mstch_py3_program : public mstch_program {
  public:
   mstch_py3_program(
       const t_program* program,
-      std::shared_ptr<const mstch_generators> generators,
+      const mstch_factories& factories,
       std::shared_ptr<mstch_cache> cache,
-      ELEMENT_POSITION const pos)
-      : mstch_program{program, generators, cache, pos} {
+      mstch_element_position pos)
+      : mstch_program(program, factories, cache, pos) {
     register_methods(
         this,
         {
@@ -384,6 +387,8 @@ class mstch_py3_program : public mstch_program {
             {"program:stream_exceptions",
              &mstch_py3_program::getStreamExceptions},
             {"program:cpp_gen_path", &mstch_py3_program::getCppGenPath},
+            {"program:py_deprecated_module_path",
+             &mstch_py3_program::py_deprecated_module_path},
         });
     gather_included_program_namespaces();
     visit_types_for_services_and_interactions();
@@ -392,7 +397,7 @@ class mstch_py3_program : public mstch_program {
     visit_types_for_typedefs();
     visit_types_for_mixin_fields();
 
-    for (auto func : lifecycleFunctions()) {
+    for (const auto* func : lifecycleFunctions()) {
       addFunctionByUniqueReturnType(func);
     }
   }
@@ -402,8 +407,7 @@ class mstch_py3_program : public mstch_program {
   }
 
   mstch::node getContainerTypes() {
-    type_py3_generator<true> generator{program_};
-    return generate_elements(containers_, &generator);
+    return make_mstch_array(containers_, py3_type_factory<true>(program_));
   }
 
   mstch::node getCppIncludes() {
@@ -422,7 +426,7 @@ class mstch_py3_program : public mstch_program {
       }
     }
 
-    return generate_functions(functions);
+    return make_mstch_functions(functions);
   }
 
   mstch::node getCustomTemplates() {
@@ -484,6 +488,14 @@ class mstch_py3_program : public mstch_program {
     return !has_option("no_stream") && !streamTypes_.empty();
   }
 
+  mstch::node py_deprecated_module_path() {
+    std::string module_path = program_->get_namespace("py");
+    if (module_path.empty()) {
+      return program_->name();
+    }
+    return module_path;
+  }
+
  protected:
   struct Namespace {
     std::vector<std::string> ns;
@@ -492,7 +504,7 @@ class mstch_py3_program : public mstch_program {
   };
 
   mstch::array generate_types_array(const std::vector<const t_type*>& types) {
-    return generate_types(types);
+    return make_mstch_types(types);
   }
 
   void gather_included_program_namespaces() {
@@ -528,7 +540,7 @@ class mstch_py3_program : public mstch_program {
 
   void addFunctionByUniqueReturnType(const t_function* function) {
     const t_type* return_type = function->get_returntype();
-    auto sa = cpp2::is_stack_arguments(cache_->parsed_options_, *function);
+    auto sa = cpp2::is_stack_arguments(cache_->options_, *function);
     uniqueFunctionsByReturnType_.insert(
         {{visit_type(return_type), sa}, function});
   }
@@ -560,6 +572,9 @@ class mstch_py3_program : public mstch_program {
       for (auto&& field : object->fields()) {
         visit_type(field.get_type());
       }
+      if (!gen::cpp::type_resolver::is_directly_adapted(*object)) {
+        objects_.push_back(object);
+      }
     }
   }
 
@@ -592,8 +607,8 @@ class mstch_py3_program : public mstch_program {
   std::string visit_type_with_typedef(
       const t_type* orig_type, TypeDef isTypedef) {
     auto trueType = orig_type->get_true_type();
-    auto baseType =
-        generators_->type_generator_->generate(trueType, generators_, cache_);
+    auto baseType = factories_.type_factory->make_mstch_object(
+        trueType, factories_, cache_);
     mstch_py3_type* type = dynamic_cast<mstch_py3_type*>(baseType.get());
     const std::string& flatName = type->get_flat_name();
     // Import all types either beneath a typedef, even if the current type is
@@ -664,27 +679,32 @@ class mstch_py3_program : public mstch_program {
     return flatName;
   }
 
+  const std::vector<t_struct*>& get_program_objects() override {
+    return objects_;
+  }
+
   std::vector<const t_type*> containers_;
   std::map<std::string, const t_type*> moveContainers_;
   std::vector<const t_type*> customTemplates_;
   std::vector<const t_type*> customTypes_;
   std::unordered_set<std::string> seenTypeNames_;
   std::map<std::string, Namespace> includeNamespaces_;
-  std::map<std::tuple<std::string, bool>, t_function const*>
+  std::map<std::tuple<std::string, bool>, const t_function*>
       uniqueFunctionsByReturnType_;
   std::vector<const t_type*> responseAndStreamTypes_;
   std::map<std::string, const t_type*> streamTypes_;
   std::map<std::string, const t_type*> streamExceptions_;
+  std::vector<t_struct*> objects_;
 };
 
 class mstch_py3_function : public mstch_function {
  public:
   mstch_py3_function(
-      t_function const* function,
-      std::shared_ptr<mstch_generators const> generators,
+      const t_function* function,
+      const mstch_factories& factories,
       std::shared_ptr<mstch_cache> cache,
-      ELEMENT_POSITION const pos)
-      : mstch_function(function, generators, cache, pos),
+      mstch_element_position pos)
+      : mstch_function(function, factories, cache, pos),
         cppName_{cpp2::get_name(function)} {
     register_methods(
         this,
@@ -700,7 +720,7 @@ class mstch_py3_function : public mstch_function {
   }
 
   mstch::node stack_arguments() {
-    return cpp2::is_stack_arguments(cache_->parsed_options_, *function_);
+    return cpp2::is_stack_arguments(cache_->options_, *function_);
   }
 
  protected:
@@ -711,11 +731,11 @@ class mstch_py3_service : public mstch_service {
  public:
   mstch_py3_service(
       const t_service* service,
-      std::shared_ptr<const mstch_generators> generators,
+      const mstch_factories& factories,
       std::shared_ptr<mstch_cache> cache,
-      ELEMENT_POSITION const pos,
+      mstch_element_position pos,
       const t_program* prog)
-      : mstch_service(service, generators, cache, pos), prog_{prog} {
+      : mstch_service(service, factories, cache, pos), prog_{prog} {
     register_methods(
         this,
         {
@@ -762,11 +782,11 @@ class mstch_py3_service : public mstch_service {
   }
 
   mstch::node parent_service_name() {
-    return cache_->parsed_options_.at("parent_service_name");
+    return cache_->options_.at("parent_service_name");
   }
 
   mstch::node parent_service_cpp_name() {
-    return cache_->parsed_options_.at("parent_service_cpp_name");
+    return cache_->options_.at("parent_service_cpp_name");
   }
 
   std::vector<t_function*> supportedFunctions() {
@@ -781,11 +801,11 @@ class mstch_py3_service : public mstch_service {
   }
 
   mstch::node get_lifecycle_functions() {
-    return generate_functions(lifecycleFunctions());
+    return make_mstch_functions(lifecycleFunctions());
   }
 
   mstch::node get_supported_functions() {
-    return generate_functions(supportedFunctions());
+    return make_mstch_functions(supportedFunctions());
   }
 
   mstch::node get_supported_functions_with_lifecycle() {
@@ -793,7 +813,7 @@ class mstch_py3_service : public mstch_service {
     for (auto func : lifecycleFunctions()) {
       funcs.push_back(func);
     }
-    return generate_functions(funcs);
+    return make_mstch_functions(funcs);
   }
 
  protected:
@@ -811,14 +831,13 @@ class mstch_py3_field : public mstch_field {
   };
   mstch_py3_field(
       const t_field* field,
-      std::shared_ptr<const mstch_generators> generators,
+      const mstch_factories& factories,
       std::shared_ptr<mstch_cache> cache,
-      ELEMENT_POSITION const pos,
-      int32_t index,
-      field_generator_context const* field_context)
-      : mstch_field(field, generators, cache, pos, index, field_context),
-        pyName_{py3::get_py3_name(*field)},
-        cppName_{cpp2::get_name(field)} {
+      mstch_element_position pos,
+      const field_generator_context* field_context)
+      : mstch_field(field, factories, cache, pos, field_context),
+        pyName_(py3::get_py3_name(*field)),
+        cppName_(cpp2::get_name(field)) {
     register_methods(
         this,
         {
@@ -836,6 +855,7 @@ class mstch_py3_field : public mstch_field {
             {"field:hasModifiedName?", &mstch_py3_field::hasModifiedName},
             {"field:hasPyName?", &mstch_py3_field::hasPyName},
             {"field:boxed_ref?", &mstch_py3_field::boxed_ref},
+            {"field:has_ref_api?", &mstch_py3_field::hasRefApi},
         });
   }
 
@@ -879,6 +899,20 @@ class mstch_py3_field : public mstch_field {
     return gen::cpp::find_ref_type(*field_) == gen::cpp::reference_type::boxed;
   }
 
+  mstch::node hasRefApi() {
+    // Mixin is a special case (T126232678) because it does not contain
+    // a valid pointer to the top level struct
+    bool isMixin =
+        ((mstch_field::field_context_ == nullptr) ||
+         (mstch_field::field_context_->strct == nullptr));
+    if (isMixin) {
+      return false;
+    }
+
+    const t_struct* parentStruct = mstch_field::field_context_->strct;
+    return parentStruct->is_union() || generate_legacy_api(*parentStruct);
+  }
+
  protected:
   RefType get_ref_type() {
     if (ref_type_cached_) {
@@ -905,11 +939,6 @@ class mstch_py3_field : public mstch_field {
         }
         return ref_type_ = RefType::NotRef;
       }
-      case gen::cpp::reference_type::unrecognized: {
-        // It is legal to get here but hopefully nobody will in practice, since
-        // we're not set up to handle other kinds of refs:
-        throw std::runtime_error{"Unrecognized ref_type"};
-      }
     }
     // Suppress "control reaches end of non-void function" warning
     throw std::logic_error{"Unhandled ref_type"};
@@ -931,10 +960,10 @@ class mstch_py3_struct : public mstch_struct {
  public:
   mstch_py3_struct(
       const t_struct* strct,
-      std::shared_ptr<const mstch_generators> generators,
+      const mstch_factories& factories,
       std::shared_ptr<mstch_cache> cache,
-      ELEMENT_POSITION const pos)
-      : mstch_struct(strct, generators, cache, pos) {
+      mstch_element_position pos)
+      : mstch_struct(strct, factories, cache, pos) {
     register_methods(
         this,
         {
@@ -951,12 +980,12 @@ class mstch_py3_struct : public mstch_struct {
             {"struct:py3_fields", &mstch_py3_struct::py3_fields},
             {"struct:py3_fields?", &mstch_py3_struct::has_py3_fields},
         });
-    py3_fields_ = strct_->fields().copy();
+    py3_fields_ = struct_->fields().copy();
     py3_fields_.erase(
         std::remove_if(
             py3_fields_.begin(),
             py3_fields_.end(),
-            [](t_field const* field) {
+            [](const t_field* field) {
               return field->has_annotation("py3.hidden");
             }),
         py3_fields_.end());
@@ -965,48 +994,49 @@ class mstch_py3_struct : public mstch_struct {
   mstch::node getSize() { return std::to_string(py3_fields_.size()); }
 
   mstch::node isStructOrderable() {
-    return cpp2::is_orderable(*strct_) &&
-        !strct_->has_annotation("no_default_comparators");
+    return cpp2::is_orderable(*struct_) &&
+        !struct_->has_annotation("no_default_comparators");
   }
 
   mstch::node cppNonComparable() {
-    return strct_->has_annotation({"cpp.noncomparable", "cpp2.noncomparable"});
+    return struct_->has_annotation({"cpp.noncomparable", "cpp2.noncomparable"});
   }
 
   mstch::node cppNonCopyable() {
-    return strct_->has_annotation({"cpp.noncopyable", "cpp2.noncopyable"});
+    return struct_->has_annotation({"cpp.noncopyable", "cpp2.noncopyable"});
   }
 
   mstch::node hasExceptionMessage() {
-    return strct_->has_annotation("message");
+    return struct_->has_annotation("message");
   }
 
-  mstch::node exceptionMessage() { return strct_->get_annotation("message"); }
+  mstch::node exceptionMessage() { return struct_->get_annotation("message"); }
 
-  mstch::node py3_fields() { return generate_fields(py3_fields_); }
+  mstch::node py3_fields() { return make_mstch_fields(py3_fields_); }
 
   mstch::node has_py3_fields() { return !py3_fields_.empty(); }
 
   mstch::node fields_and_mixin_fields() {
-    std::vector<t_field const*> fields = py3_fields_;
-    for (auto m : cpp2::get_mixins_and_members(*strct_)) {
+    std::vector<const t_field*> fields = py3_fields_;
+    for (const auto& m : cpp2::get_mixins_and_members(*struct_)) {
       fields.push_back(m.member);
     }
-    return generate_fields(fields);
+
+    return make_mstch_fields(fields);
   }
 
  private:
-  std::vector<t_field const*> py3_fields_;
+  std::vector<const t_field*> py3_fields_;
 };
 
 class mstch_py3_enum : public mstch_enum {
  public:
   mstch_py3_enum(
       const t_enum* enm,
-      std::shared_ptr<const mstch_generators> generators,
+      const mstch_factories& factories,
       std::shared_ptr<mstch_cache> cache,
-      ELEMENT_POSITION const pos)
-      : mstch_enum(enm, generators, cache, pos) {
+      mstch_element_position pos)
+      : mstch_enum(enm, factories, cache, pos) {
     register_methods(
         this,
         {
@@ -1015,18 +1045,18 @@ class mstch_py3_enum : public mstch_enum {
         });
   }
 
-  mstch::node hasFlags() { return enm_->has_annotation("py3.flags"); }
-  mstch::node cpp_name() { return cpp2::get_name(enm_); }
+  mstch::node hasFlags() { return enum_->has_annotation("py3.flags"); }
+  mstch::node cpp_name() { return cpp2::get_name(enum_); }
 };
 
 class mstch_py3_enum_value : public mstch_enum_value {
  public:
   mstch_py3_enum_value(
-      const t_enum_value* enm_value,
-      std::shared_ptr<const mstch_generators> generators,
+      const t_enum_value* enum_value,
+      const mstch_factories& factories,
       std::shared_ptr<mstch_cache> cache,
-      ELEMENT_POSITION const pos)
-      : mstch_enum_value(enm_value, generators, cache, pos) {
+      mstch_element_position pos)
+      : mstch_enum_value(enum_value, factories, cache, pos) {
     register_methods(
         this,
         {
@@ -1036,12 +1066,12 @@ class mstch_py3_enum_value : public mstch_enum_value {
         });
   }
 
-  mstch::node pyName() { return py3::get_py3_name(*enm_value_); }
+  mstch::node pyName() { return py3::get_py3_name(*enum_value_); }
 
-  mstch::node cppName() { return cpp2::get_name(enm_value_); }
+  mstch::node cppName() { return cpp2::get_name(enum_value_); }
 
   mstch::node hasPyName() {
-    return py3::get_py3_name(*enm_value_) != enm_value_->get_name();
+    return py3::get_py3_name(*enum_value_) != enum_value_->get_name();
   }
 };
 
@@ -1049,12 +1079,12 @@ class mstch_py3_container_type : public mstch_py3_type {
  public:
   mstch_py3_container_type(
       const t_type* type,
-      std::shared_ptr<const mstch_generators> generators,
+      const mstch_factories& factories,
       std::shared_ptr<mstch_cache> cache,
-      ELEMENT_POSITION const pos,
+      mstch_element_position pos,
       const t_program* prog,
       CachedProperties& cachedProps)
-      : mstch_py3_type(type, generators, cache, pos, prog, cachedProps) {
+      : mstch_py3_type(type, factories, cache, pos, prog, cachedProps) {
     register_methods(
         this,
         {
@@ -1070,36 +1100,28 @@ class mstch_py3_container_type : public mstch_py3_type {
   }
 };
 
-class mstch_py3_annotation : public mstch_annotation {
+class mstch_py3_deprecated_annotation : public mstch_deprecated_annotation {
  public:
-  mstch_py3_annotation(
-      const t_annotation& annotation,
-      std::shared_ptr<const mstch_generators> generators,
+  mstch_py3_deprecated_annotation(
+      const t_annotation* annotation,
+      const mstch_factories& factories,
       std::shared_ptr<mstch_cache> cache,
-      ELEMENT_POSITION pos,
-      int32_t index)
-      : mstch_annotation(
-            annotation.first,
-            annotation.second,
-            generators,
-            cache,
-            pos,
-            index) {
+      mstch_element_position pos)
+      : mstch_deprecated_annotation(annotation, factories, cache, pos) {
     register_methods(
         this,
         {
-            {"annotation:value?", &mstch_py3_annotation::hasValue},
-            {"annotation:py_quoted_key", &mstch_py3_annotation::pyQuotedKey},
+            {"annotation:value?", &mstch_py3_deprecated_annotation::has_value},
+            {"annotation:py_quoted_key",
+             &mstch_py3_deprecated_annotation::py_quoted_key},
             {"annotation:py_quoted_value",
-             &mstch_py3_annotation::pyQuotedValue},
+             &mstch_py3_deprecated_annotation::py_quoted_value},
         });
   }
 
-  mstch::node hasValue() { return !val_.value.empty(); }
-
-  mstch::node pyQuotedKey() { return to_python_string_literal(key_); }
-
-  mstch::node pyQuotedValue() { return to_python_string_literal(val_.value); }
+  mstch::node has_value() { return !val_.value.empty(); }
+  mstch::node py_quoted_key() { return to_python_string_literal(key_); }
+  mstch::node py_quoted_value() { return to_python_string_literal(val_.value); }
 
  protected:
   std::string to_python_string_literal(std::string val) const {
@@ -1110,14 +1132,13 @@ class mstch_py3_annotation : public mstch_annotation {
   }
 };
 
-class program_py3_generator : public program_generator {
+class py3_program_factory : public mstch_program_factory {
  public:
-  std::shared_ptr<mstch_base> generate(
+  std::shared_ptr<mstch_base> make_mstch_object(
       const t_program* program,
-      std::shared_ptr<const mstch_generators> generators,
+      const mstch_factories& factories,
       std::shared_ptr<mstch_cache> cache,
-      ELEMENT_POSITION pos,
-      int32_t /*index*/) const override {
+      mstch_element_position pos) const override {
     const std::string& id = program->path();
     auto it = cache->programs_.find(id);
     if (it != cache->programs_.end()) {
@@ -1125,158 +1146,73 @@ class program_py3_generator : public program_generator {
     }
     auto r = cache->programs_.emplace(
         id,
-        std::make_shared<mstch_py3_program>(program, generators, cache, pos));
+        std::make_shared<mstch_py3_program>(program, factories, cache, pos));
     return r.first->second;
   }
   std::unordered_map<const t_type*, mstch_py3_type::CachedProperties>
       typePropsCache;
 };
 
-class struct_py3_generator : public struct_generator {
+class py3_service_factory : public mstch_service_factory {
  public:
-  std::shared_ptr<mstch_base> generate(
-      const t_struct* strct,
-      std::shared_ptr<const mstch_generators> generators,
-      std::shared_ptr<mstch_cache> cache,
-      ELEMENT_POSITION pos,
-      int32_t /*index*/) const override {
-    return std::make_shared<mstch_py3_struct>(strct, generators, cache, pos);
-  }
-};
+  explicit py3_service_factory(const t_program* prog) : prog_{prog} {}
 
-class function_py3_generator : public function_generator {
- public:
-  function_py3_generator() = default;
-  ~function_py3_generator() override = default;
-  std::shared_ptr<mstch_base> generate(
-      t_function const* function,
-      std::shared_ptr<mstch_generators const> generators,
-      std::shared_ptr<mstch_cache> cache,
-      ELEMENT_POSITION pos,
-      int32_t /*index*/) const override {
-    return std::make_shared<mstch_py3_function>(
-        function, generators, cache, pos);
-  }
-};
-
-class service_py3_generator : public service_generator {
- public:
-  explicit service_py3_generator(const t_program* prog) : prog_{prog} {}
-  std::shared_ptr<mstch_base> generate(
+  std::shared_ptr<mstch_base> make_mstch_object(
       const t_service* service,
-      std::shared_ptr<const mstch_generators> generators,
+      const mstch_factories& factories,
       std::shared_ptr<mstch_cache> cache,
-      ELEMENT_POSITION pos,
-      int32_t /*index*/) const override {
+      mstch_element_position pos) const override {
     return std::make_shared<mstch_py3_service>(
-        service, generators, cache, pos, prog_);
+        service, factories, cache, pos, prog_);
   }
 
  protected:
   const t_program* prog_;
 };
 
-class field_py3_generator : public field_generator {
- public:
-  std::shared_ptr<mstch_base> generate(
-      const t_field* field,
-      std::shared_ptr<const mstch_generators> generators,
-      std::shared_ptr<mstch_cache> cache,
-      ELEMENT_POSITION pos,
-      int32_t index,
-      field_generator_context const* field_context) const override {
-    return std::make_shared<mstch_py3_field>(
-        field, generators, cache, pos, index, field_context);
-  }
-};
-
-class enum_py3_generator : public enum_generator {
- public:
-  std::shared_ptr<mstch_base> generate(
-      const t_enum* enm,
-      std::shared_ptr<const mstch_generators> generators,
-      std::shared_ptr<mstch_cache> cache,
-      ELEMENT_POSITION pos,
-      int32_t /*index*/) const override {
-    return std::make_shared<mstch_py3_enum>(enm, generators, cache, pos);
-  }
-};
-
-class enum_value_py3_generator : public enum_value_generator {
- public:
-  std::shared_ptr<mstch_base> generate(
-      const t_enum_value* enm_value,
-      std::shared_ptr<const mstch_generators> generators,
-      std::shared_ptr<mstch_cache> cache,
-      ELEMENT_POSITION pos,
-      int32_t /*index*/) const override {
-    return std::make_shared<mstch_py3_enum_value>(
-        enm_value, generators, cache, pos);
-  }
-};
-
-class annotation_py3_generator : public annotation_generator {
- public:
-  std::shared_ptr<mstch_base> generate(
-      const t_annotation& annotation,
-      std::shared_ptr<const mstch_generators> generators,
-      std::shared_ptr<mstch_cache> cache,
-      ELEMENT_POSITION pos,
-      int32_t index) const override {
-    return std::make_shared<mstch_py3_annotation>(
-        annotation, generators, cache, pos, index);
-  }
-};
-
-/**
- * Generator-specific validator that enforces that reserved key is
- * not used as namespace field name.
- *
- */
-class no_reserved_key_in_namespace_validator : virtual public validator {
+// Generator-specific validator that enforces that a reserved key is not used as
+// a namespace component.
+class no_reserved_key_in_namespace_validator : public validator {
  public:
   using validator::visit;
 
-  bool visit(t_program* const prog) override {
-    set_program(prog);
+  bool visit(t_program* prog) override {
     validate(prog);
     return true;
   }
 
  private:
-  void validate(t_program* const prog) {
+  void validate(t_program* prog) {
     auto namespace_tokens = get_py3_namespace(prog);
     if (namespace_tokens.empty()) {
       return;
     }
-    for (const auto& field_name : namespace_tokens) {
-      if (get_python_reserved_names().find(field_name) !=
+    for (const auto& component : namespace_tokens) {
+      if (get_python_reserved_names().find(component) !=
           get_python_reserved_names().end()) {
-        std::ostringstream ss;
-        ss << "Namespace '" << boost::algorithm::join(namespace_tokens, ".")
-           << "' contains reserved keyword '" << field_name << "'";
-        add_error(boost::none, ss.str());
+        report_error(
+            *prog,
+            "Namespace '{}' contains reserved keyword '{}'",
+            fmt::join(namespace_tokens, "."),
+            component);
       }
     }
 
-    std::string filepath_delimiters("\\/.");
-    std::vector<std::string> fields;
-    boost::split(fields, prog->path(), boost::is_any_of(filepath_delimiters));
-    for (const auto& field : fields) {
-      if (field == "include") {
-        std::ostringstream ss;
-        ss << "Path '" << prog->path()
-           << "' contains reserved keyword 'include'";
-        add_error(boost::none, ss.str());
+    std::vector<std::string> components;
+    boost::split(components, prog->path(), boost::is_any_of("\\/."));
+    for (const auto& component : components) {
+      if (component == "include") {
+        report_error(
+            *prog,
+            "Path '{}' contains reserved keyword 'include'",
+            prog->path());
       }
     }
   }
 };
 
-/**
- * Generator-specific validator that enforces "name" and "value" is not used as
- * enum member or union field names (thrift-py3)
- */
+// Generator-specific validator that enforces "name" and "value" are not used as
+// enum member or union field names (thrift-py3).
 class enum_member_union_field_names_validator : virtual public validator {
  public:
   using validator::visit;
@@ -1302,11 +1238,12 @@ class enum_member_union_field_names_validator : virtual public validator {
   void validate(const t_named* node, const std::string& name) {
     const auto& pyname = node->get_annotation("py3.name", &name);
     if (pyname == "name" || pyname == "value") {
-      std::ostringstream ss;
-      ss << "'" << pyname
-         << "' should not be used as an enum/union field name in thrift-py3. "
-         << "Use a different name or annotate the field with `(py3.name=\"<new_py_name>\")`";
-      add_error(node->get_lineno(), ss.str());
+      report_error(
+          *node,
+          "'{}' should not be used as an enum/union field name in thrift-py3. "
+          "Use a different name or annotate the field with "
+          "`(py3.name=\"<new_py_name>\")`",
+          pyname);
     }
   }
 };
@@ -1316,10 +1253,9 @@ class t_mstch_py3_generator : public t_mstch_generator {
   t_mstch_py3_generator(
       t_program* program,
       t_generation_context context,
-      const std::map<std::string, std::string>& parsed_options,
-      const std::string& /* option_string unused */)
-      : t_mstch_generator(program, std::move(context), "py3", parsed_options),
-        generateRootPath_{package_to_path()} {
+      const std::map<std::string, std::string>& options)
+      : t_mstch_generator(program, std::move(context), "py3", options),
+        generateRootPath_(package_to_path()) {
     out_dir_base_ = "gen-py3";
     auto include_prefix = get_option("include_prefix");
     if (!include_prefix.empty()) {
@@ -1328,7 +1264,7 @@ class t_mstch_py3_generator : public t_mstch_generator {
   }
 
   void generate_program() override {
-    set_mstch_generators();
+    set_mstch_factories();
     generate_init_files();
     generate_types();
     generate_services();
@@ -1343,7 +1279,7 @@ class t_mstch_py3_generator : public t_mstch_generator {
 
  protected:
   bool should_resolve_typedefs() const override { return true; }
-  void set_mstch_generators();
+  void set_mstch_factories();
   void generate_init_files();
   void generate_file(
       const std::string& file,
@@ -1359,50 +1295,47 @@ class t_mstch_py3_generator : public t_mstch_generator {
 } // namespace
 
 template <bool ForContainers>
-std::shared_ptr<mstch_base> type_py3_generator<ForContainers>::generate(
+std::shared_ptr<mstch_base> py3_type_factory<ForContainers>::make_mstch_object(
     const t_type* type,
-    std::shared_ptr<const mstch_generators> generators,
+    const mstch_factories& factories,
     std::shared_ptr<mstch_cache> cache,
-    ELEMENT_POSITION pos,
-    int32_t /*index*/) const {
+    mstch_element_position pos) const {
   using T = std::
       conditional_t<ForContainers, mstch_py3_container_type, mstch_py3_type>;
   auto trueType = type->get_true_type();
   auto& propsCache =
-      dynamic_cast<program_py3_generator*>(generators->program_generator_.get())
-          ->typePropsCache;
+      dynamic_cast<py3_program_factory&>(*factories.program_factory)
+          .typePropsCache;
   auto it = propsCache.find(trueType);
   if (it == propsCache.end()) {
     propsCache.emplace(
         trueType,
         mstch_py3_type::CachedProperties{
-            get_cpp_template(*trueType), cpp2::get_type(trueType), {}});
+            get_cpp_template(*trueType),
+            fmt::to_string(cpp2::get_type(trueType)),
+            {}});
   }
   return std::make_shared<T>(
-      trueType, generators, cache, pos, prog_, propsCache.at(trueType));
+      trueType, factories, cache, pos, prog_, propsCache.at(trueType));
 }
 
-void t_mstch_py3_generator::set_mstch_generators() {
-  generators_->set_program_generator(std::make_unique<program_py3_generator>());
-  generators_->set_struct_generator(std::make_unique<struct_py3_generator>());
-  generators_->set_function_generator(
-      std::make_unique<function_py3_generator>());
-  generators_->set_service_generator(
-      std::make_unique<service_py3_generator>(get_program()));
-  generators_->set_field_generator(std::make_unique<field_py3_generator>());
-  generators_->set_enum_generator(std::make_unique<enum_py3_generator>());
-  generators_->set_enum_value_generator(
-      std::make_unique<enum_value_py3_generator>());
-  generators_->set_type_generator(
-      std::make_unique<type_py3_generator<false>>(get_program()));
-  generators_->set_annotation_generator(
-      std::make_unique<annotation_py3_generator>());
+void t_mstch_py3_generator::set_mstch_factories() {
+  factories_.program_factory = std::make_unique<py3_program_factory>();
+  factories_.service_factory = std::make_unique<py3_service_factory>(program_);
+  factories_.set_function_factory<mstch_py3_function>();
+  factories_.type_factory = std::make_unique<py3_type_factory<false>>(program_);
+  factories_.set_struct_factory<mstch_py3_struct>();
+  factories_.set_field_factory<mstch_py3_field>();
+  factories_.set_enum_factory<mstch_py3_enum>();
+  factories_.set_enum_value_factory<mstch_py3_enum_value>();
+  factories_
+      .set_deprecated_annotation_factory<mstch_py3_deprecated_annotation>();
 }
 
 void t_mstch_py3_generator::generate_init_files() {
   boost::filesystem::path p = generateRootPath_;
-  auto nodePtr = generators_->program_generator_->generate(
-      get_program(), generators_, cache_);
+  auto nodePtr = factories_.program_factory->make_mstch_object(
+      get_program(), factories_, cache_);
   while (!p.empty()) {
     render_to_file(nodePtr, "common/auto_generated_py", p / "__init__.py");
     p = p.parent_path();
@@ -1425,12 +1358,12 @@ void t_mstch_py3_generator::generate_file(
   auto program = get_program();
   const auto& name = program->name();
   if (is_types_file == IsTypesFile) {
-    cache_->parsed_options_["is_types_file"] = "";
+    cache_->options_["is_types_file"] = "";
   } else {
-    cache_->parsed_options_.erase("is_types_file");
+    cache_->options_.erase("is_types_file");
   }
-  auto nodePtr =
-      generators_->program_generator_->generate(program, generators_, cache_);
+  auto nodePtr = factories_.program_factory->make_mstch_object(
+      program, factories_, cache_);
   render_to_file(nodePtr, file, base / name / file);
 }
 
@@ -1478,9 +1411,11 @@ void t_mstch_py3_generator::generate_types() {
 }
 
 void t_mstch_py3_generator::generate_services() {
-  if (get_program()->services().empty()) {
+  if (get_program()->services().empty() && !has_option("single_file_service")) {
     // There is no need to generate empty / broken code for non existent
-    // services.
+    // services. However, in single_file_service mode, the build system may not
+    // know ahead of time if these files can exist - so we should always
+    // generate them.
     return;
   }
 
