@@ -40,19 +40,6 @@ namespace compiler {
 
 namespace {
 
-std::string get_py3_namespace_with_name_and_prefix(
-    const t_program* prog, const std::string& prefix) {
-  std::ostringstream ss;
-  if (!prefix.empty()) {
-    ss << prefix << ".";
-  }
-  for (const auto& name : get_py3_namespace(prog)) {
-    ss << name << ".";
-  }
-  ss << prog->name();
-  return ss.str();
-}
-
 bool is_type_iobuf(const std::string& name) {
   return name == "folly::IOBuf" || name == "std::unique_ptr<folly::IOBuf>";
 }
@@ -86,12 +73,12 @@ const std::string extract_module_path(const std::string& fully_qualified_name) {
 class mstch_python_typedef : public mstch_typedef {
  public:
   mstch_python_typedef(
-      t_typedef const* typedf,
-      std::shared_ptr<mstch_generators const> generators,
+      const t_typedef* t,
+      const mstch_factories& factories,
       std::shared_ptr<mstch_cache> cache,
-      ELEMENT_POSITION pos)
-      : mstch_typedef(typedf, std::move(generators), std::move(cache), pos),
-        adapter_annotation_(find_structured_adapter_annotation(*typedf)) {
+      mstch_element_position pos)
+      : mstch_typedef(t, factories, std::move(cache), pos),
+        adapter_annotation_(find_structured_adapter_annotation(*t)) {
     register_methods(
         this,
         {
@@ -111,33 +98,16 @@ class mstch_python_typedef : public mstch_typedef {
   const t_const* adapter_annotation_;
 };
 
-class typedef_python_generator : public typedef_generator {
- public:
-  std::shared_ptr<mstch_base> generate(
-      t_typedef const* typedf,
-      std::shared_ptr<mstch_generators const> generators,
-      std::shared_ptr<mstch_cache> cache,
-      ELEMENT_POSITION pos,
-      int32_t /*index*/) const override {
-    return std::make_shared<mstch_python_typedef>(
-        typedf, std::move(generators), std::move(cache), pos);
-  }
-};
-
 class mstch_python_type : public mstch_type {
  public:
   mstch_python_type(
       const t_type* type,
-      std::shared_ptr<const mstch_generators> generators,
+      const mstch_factories& factories,
       std::shared_ptr<mstch_cache> cache,
-      ELEMENT_POSITION pos,
+      mstch_element_position pos,
       const t_program* prog)
-      : mstch_type(
-            type->get_true_type(),
-            std::move(generators),
-            std::move(cache),
-            pos),
-        prog_{prog},
+      : mstch_type(type->get_true_type(), factories, std::move(cache), pos),
+        prog_(prog),
         adapter_annotation_(find_structured_adapter_annotation(*type)) {
     register_methods(
         this,
@@ -239,25 +209,25 @@ class mstch_python_type : public mstch_type {
 class mstch_python_const_value : public mstch_const_value {
  public:
   mstch_python_const_value(
-      t_const_value const* const_value,
-      t_const const* current_const,
-      t_type const* expected_type,
-      std::shared_ptr<mstch_generators const> generators,
+      const t_const_value* const_value,
+      const mstch_factories& factories,
       std::shared_ptr<mstch_cache> cache,
-      ELEMENT_POSITION pos,
-      int32_t index)
+      mstch_element_position pos,
+      const t_const* current_const,
+      const t_type* expected_type)
       : mstch_const_value(
             const_value,
-            current_const,
-            expected_type,
-            std::move(generators),
+            factories,
             std::move(cache),
             pos,
-            index) {
+            current_const,
+            expected_type) {
     register_methods(
         this,
         {
             {"value:py3_string_value", &mstch_python_const_value::string_value},
+            {"value:py3_enum_value_name",
+             &mstch_python_const_value::py3_enum_value_name},
             {"value:py3_string?", &mstch_python_const_value::is_string},
             {"value:py3_binary?", &mstch_python_const_value::is_binary},
             {"value:const_enum_type",
@@ -291,7 +261,8 @@ class mstch_python_const_value : public mstch_const_value {
     }
     const auto* type = const_value_->ttype()->get_true_type();
     if (type->is_enum()) {
-      return generators_->type_generator_->generate(type, generators_, cache_);
+      return factories_.type_factory->make_mstch_object(
+          type, factories_, cache_);
     }
     return {};
   }
@@ -310,6 +281,13 @@ class mstch_python_const_value : public mstch_const_value {
     return false;
   }
 
+  mstch::node py3_enum_value_name() {
+    if (const_value_->is_enum() && const_value_->get_enum_value() != nullptr) {
+      return py3::get_py3_name(*const_value_->get_enum_value());
+    }
+    return mstch_const_value::enum_value_name();
+  }
+
   mstch::node string_value() {
     if (type_ != cv::CV_STRING) {
       return mstch::node();
@@ -352,8 +330,8 @@ class mstch_python_const_value : public mstch_const_value {
       } else {
         return {};
       }
-      return generators_->type_generator_->generate(
-          elem_type, generators_, cache_, pos_);
+      return factories_.type_factory->make_mstch_object(
+          elem_type, factories_, cache_, pos_);
     }
     return {};
   }
@@ -369,9 +347,9 @@ class mstch_python_const_value : public mstch_const_value {
     if (auto ttype = const_value_->ttype()) {
       const auto* type = ttype->get_true_type();
       if (type->is_map()) {
-        return generators_->type_generator_->generate(
+        return factories_.type_factory->make_mstch_object(
             dynamic_cast<const t_map*>(type)->get_key_type(),
-            generators_,
+            factories_,
             cache_,
             pos_);
       }
@@ -383,9 +361,9 @@ class mstch_python_const_value : public mstch_const_value {
     if (auto ttype = const_value_->ttype()) {
       const auto* type = ttype->get_true_type();
       if (type->is_map()) {
-        return generators_->type_generator_->generate(
+        return factories_.type_factory->make_mstch_object(
             dynamic_cast<const t_map*>(type)->get_val_type(),
-            generators_,
+            factories_,
             cache_,
             pos_);
       }
@@ -394,17 +372,17 @@ class mstch_python_const_value : public mstch_const_value {
   }
 };
 
-class type_python_generator : public type_generator {
+class python_type_factory : public mstch_type_factory {
  public:
-  explicit type_python_generator(const t_program* prog) : prog_{prog} {}
-  std::shared_ptr<mstch_base> generate(
+  explicit python_type_factory(const t_program* prog) : prog_(prog) {}
+
+  std::shared_ptr<mstch_base> make_mstch_object(
       const t_type* type,
-      std::shared_ptr<const mstch_generators> generators,
+      const mstch_factories& factories,
       std::shared_ptr<mstch_cache> cache,
-      ELEMENT_POSITION pos,
-      int32_t /*index*/) const override {
+      mstch_element_position pos) const override {
     return std::make_shared<mstch_python_type>(
-        type, generators, cache, pos, prog_);
+        type, factories, cache, pos, prog_);
   }
 
  protected:
@@ -415,14 +393,16 @@ class mstch_python_program : public mstch_program {
  public:
   mstch_python_program(
       const t_program* program,
-      std::shared_ptr<const mstch_generators> generators,
+      const mstch_factories& factories,
       std::shared_ptr<mstch_cache> cache,
-      ELEMENT_POSITION pos)
-      : mstch_program{program, std::move(generators), std::move(cache), pos} {
+      mstch_element_position pos)
+      : mstch_program{program, factories, std::move(cache), pos} {
     register_methods(
         this,
         {
             {"program:module_path", &mstch_python_program::module_path},
+            {"program:py_deprecated_module_path",
+             &mstch_python_program::py_deprecated_module_path},
             {"program:is_types_file?", &mstch_python_program::is_types_file},
             {"program:include_namespaces",
              &mstch_python_program::include_namespaces},
@@ -434,6 +414,7 @@ class mstch_python_program : public mstch_program {
             {"program:adapter_type_hint_modules",
              &mstch_python_program::adapter_type_hint_modules},
         });
+    register_has_option("program:import_static?", "import_static");
     gather_included_program_namespaces();
     visit_types_for_services_and_interactions();
     visit_types_for_objects();
@@ -469,6 +450,14 @@ class mstch_python_program : public mstch_program {
         program_, get_option("root_module_prefix"));
   }
 
+  mstch::node py_deprecated_module_path() {
+    std::string module_path = program_->get_namespace("py");
+    if (module_path.empty()) {
+      return program_->name();
+    }
+    return module_path;
+  }
+
   mstch::node base_library_package() {
     auto option = get_option("base_library_package");
     return option.empty() ? "thrift.python" : option;
@@ -576,6 +565,15 @@ class mstch_python_program : public mstch_program {
 
   void visit_types_for_adapters() {
     for (const auto& strct : program_->structs()) {
+      if (auto annotation = find_structured_adapter_annotation(*strct)) {
+        extract_module_and_insert_to(
+            get_annotation_property(annotation, "name"), adapter_modules_);
+        extract_module_and_insert_to(
+            get_annotation_property(annotation, "typeHint"), adapter_modules_);
+        extract_module_and_insert_to(
+            get_annotation_property(annotation, "typeHint"),
+            adapter_type_hint_modules_);
+      }
       for (const auto& field : strct->fields()) {
         if (auto annotation = find_structured_adapter_annotation(field)) {
           extract_module_and_insert_to(
@@ -667,18 +665,11 @@ class mstch_python_field : public mstch_field {
  public:
   mstch_python_field(
       const t_field* field,
-      std::shared_ptr<const mstch_generators> generators,
+      const mstch_factories& factories,
       std::shared_ptr<mstch_cache> cache,
-      ELEMENT_POSITION pos,
-      int32_t index,
-      field_generator_context const* field_context)
-      : mstch_field(
-            field,
-            std::move(generators),
-            std::move(cache),
-            pos,
-            index,
-            field_context),
+      mstch_element_position pos,
+      const field_generator_context* field_context)
+      : mstch_field(field, factories, std::move(cache), pos, field_context),
         py_name_{py3::get_py3_name(*field)},
         adapter_annotation_(find_structured_adapter_annotation(*field)) {
     register_methods(
@@ -725,8 +716,8 @@ class mstch_python_field : public mstch_field {
         const_cast<t_const_value*>(value)->set_map();
       }
     }
-    return generators_->const_value_generator_->generate(
-        value, generators_, cache_, pos_);
+    return factories_.const_value_factory->make_mstch_object(
+        value, factories_, cache_, pos_, nullptr, nullptr);
   }
   mstch::node has_adapter() { return adapter_annotation_ != nullptr; }
   mstch::node adapter_name() {
@@ -744,11 +735,12 @@ class mstch_python_field : public mstch_field {
 class mstch_python_struct : public mstch_struct {
  public:
   mstch_python_struct(
-      const t_struct* strct,
-      std::shared_ptr<const mstch_generators> generators,
+      const t_struct* s,
+      const mstch_factories& factories,
       std::shared_ptr<mstch_cache> cache,
-      ELEMENT_POSITION pos)
-      : mstch_struct(strct, std::move(generators), std::move(cache), pos) {
+      mstch_element_position pos)
+      : mstch_struct(s, factories, std::move(cache), pos),
+        adapter_annotation_(find_structured_adapter_annotation(*s)) {
     register_methods(
         this,
         {
@@ -758,31 +750,51 @@ class mstch_python_struct : public mstch_struct {
              &mstch_python_struct::has_exception_message},
             {"struct:exception_message",
              &mstch_python_struct::exception_message},
+            {"struct:has_adapter?", &mstch_python_struct::has_adapter},
+            {"struct:adapter_name", &mstch_python_struct::adapter_name},
+            {"struct:adapter_type_hint",
+             &mstch_python_struct::adapter_type_hint},
         });
   }
 
   mstch::node fields_and_mixin_fields() {
-    std::vector<t_field const*> fields = strct_->fields().copy();
-    for (auto m : cpp2::get_mixins_and_members(*strct_)) {
+    std::vector<const t_field*> fields = struct_->fields().copy();
+    for (auto m : cpp2::get_mixins_and_members(*struct_)) {
       fields.push_back(m.member);
     }
-    return generate_fields(fields);
+    std::sort(fields.begin(), fields.end(), [](const auto* m, const auto* n) {
+      return m->id() < n->id();
+    });
+    return make_mstch_fields(fields);
   }
 
   mstch::node has_exception_message() {
-    return strct_->has_annotation("message");
+    return struct_->has_annotation("message");
   }
-  mstch::node exception_message() { return strct_->get_annotation("message"); }
+  mstch::node exception_message() { return struct_->get_annotation("message"); }
+
+  mstch::node has_adapter() { return adapter_annotation_ != nullptr; }
+
+  mstch::node adapter_name() {
+    return get_annotation_property(adapter_annotation_, "name");
+  }
+
+  mstch::node adapter_type_hint() {
+    return get_annotation_property(adapter_annotation_, "typeHint");
+  }
+
+ private:
+  const t_const* adapter_annotation_;
 };
 
 class mstch_python_enum : public mstch_enum {
  public:
   mstch_python_enum(
-      const t_enum* enm,
-      std::shared_ptr<const mstch_generators> generators,
+      const t_enum* e,
+      const mstch_factories& factories,
       std::shared_ptr<mstch_cache> cache,
-      ELEMENT_POSITION pos)
-      : mstch_enum(enm, std::move(generators), std::move(cache), pos) {
+      mstch_element_position pos)
+      : mstch_enum(e, factories, std::move(cache), pos) {
     register_methods(
         this,
         {
@@ -790,18 +802,17 @@ class mstch_python_enum : public mstch_enum {
         });
   }
 
-  mstch::node has_flags() { return enm_->has_annotation("py3.flags"); }
+  mstch::node has_flags() { return enum_->has_annotation("py3.flags"); }
 };
 
 class mstch_python_enum_value : public mstch_enum_value {
  public:
   mstch_python_enum_value(
-      const t_enum_value* enm_value,
-      std::shared_ptr<const mstch_generators> generators,
+      const t_enum_value* enum_value,
+      const mstch_factories& factories,
       std::shared_ptr<mstch_cache> cache,
-      ELEMENT_POSITION pos)
-      : mstch_enum_value(
-            enm_value, std::move(generators), std::move(cache), pos) {
+      mstch_element_position pos)
+      : mstch_enum_value(enum_value, factories, std::move(cache), pos) {
     register_methods(
         this,
         {
@@ -809,17 +820,16 @@ class mstch_python_enum_value : public mstch_enum_value {
         });
   }
 
-  mstch::node py_name() { return py3::get_py3_name(*enm_value_); }
+  mstch::node py_name() { return py3::get_py3_name(*enum_value_); }
 };
 
-class program_python_generator : public program_generator {
+class python_program_factory : public mstch_program_factory {
  public:
-  std::shared_ptr<mstch_base> generate(
+  std::shared_ptr<mstch_base> make_mstch_object(
       const t_program* program,
-      std::shared_ptr<const mstch_generators> generators,
+      const mstch_factories& factories,
       std::shared_ptr<mstch_cache> cache,
-      ELEMENT_POSITION pos,
-      int32_t /*index*/) const override {
+      mstch_element_position pos) const override {
     const std::string& id = program->path();
     auto it = cache->programs_.find(id);
     if (it != cache->programs_.end()) {
@@ -827,32 +837,19 @@ class program_python_generator : public program_generator {
     }
     auto r = cache->programs_.emplace(
         id,
-        std::make_shared<mstch_python_program>(
-            program, generators, cache, pos));
+        std::make_shared<mstch_python_program>(program, factories, cache, pos));
     return r.first->second;
   }
 };
 
-class struct_python_generator : public struct_generator {
- public:
-  std::shared_ptr<mstch_base> generate(
-      const t_struct* strct,
-      std::shared_ptr<const mstch_generators> generators,
-      std::shared_ptr<mstch_cache> cache,
-      ELEMENT_POSITION pos,
-      int32_t /*index*/) const override {
-    return std::make_shared<mstch_python_struct>(strct, generators, cache, pos);
-  }
-};
-
 class mstch_python_function : public mstch_function {
  public:
   mstch_python_function(
-      t_function const* function,
-      std::shared_ptr<mstch_generators const> generators,
+      const t_function* function,
+      const mstch_factories& factories,
       std::shared_ptr<mstch_cache> cache,
-      ELEMENT_POSITION const pos)
-      : mstch_function(function, generators, cache, pos) {
+      mstch_element_position pos)
+      : mstch_function(function, factories, cache, pos) {
     register_methods(
         this,
         {
@@ -879,8 +876,8 @@ class mstch_python_function : public mstch_function {
       rettype = stream->has_first_response() ? stream->get_first_response_type()
                                              : &t_base_type::t_void();
     }
-    return generators_->type_generator_->generate(
-        rettype, generators_, cache_, pos_);
+    return factories_.type_factory->make_mstch_object(
+        rettype, factories_, cache_, pos_);
   }
 
   mstch::node return_stream_elem_type() {
@@ -891,9 +888,9 @@ class mstch_python_function : public mstch_function {
     if (!rettype->is_streamresponse()) {
       return {};
     }
-    return generators_->type_generator_->generate(
+    return factories_.type_factory->make_mstch_object(
         dynamic_cast<const t_stream_response*>(rettype)->get_elem_type(),
-        generators_,
+        factories_,
         cache_,
         pos_);
   }
@@ -906,30 +903,15 @@ class mstch_python_function : public mstch_function {
   const std::string cppName_;
 };
 
-class function_python_generator : public function_generator {
- public:
-  function_python_generator() = default;
-  ~function_python_generator() override = default;
-  std::shared_ptr<mstch_base> generate(
-      t_function const* function,
-      std::shared_ptr<mstch_generators const> generators,
-      std::shared_ptr<mstch_cache> cache,
-      ELEMENT_POSITION pos,
-      int32_t /*index*/) const override {
-    return std::make_shared<mstch_python_function>(
-        function, generators, cache, pos);
-  }
-};
-
 class mstch_python_service : public mstch_service {
  public:
   mstch_python_service(
       const t_service* service,
-      std::shared_ptr<const mstch_generators> generators,
+      const mstch_factories& factories,
       std::shared_ptr<mstch_cache> cache,
-      ELEMENT_POSITION const pos,
+      mstch_element_position pos,
       const t_program* prog)
-      : mstch_service(service, generators, cache, pos), prog_{prog} {
+      : mstch_service(service, factories, cache, pos), prog_{prog} {
     register_methods(
         this,
         {
@@ -954,7 +936,7 @@ class mstch_python_service : public mstch_service {
   mstch::node program_name() { return service_->program()->name(); }
 
   mstch::node parent_service_name() {
-    return cache_->parsed_options_.at("parent_service_name");
+    return cache_->options_.at("parent_service_name");
   }
 
   std::vector<t_function*> get_supported_functions(
@@ -969,14 +951,14 @@ class mstch_python_service : public mstch_service {
   }
 
   mstch::node supported_functions() {
-    return generate_functions(
+    return make_mstch_functions(
         get_supported_functions([](const t_function* func) -> bool {
           return !func->returns_sink() && !func->get_returntype()->is_service();
         }));
   }
 
   mstch::node supported_service_functions() {
-    return generate_functions(
+    return make_mstch_functions(
         get_supported_functions([](const t_function* func) -> bool {
           return !func->returns_stream() && !func->returns_sink() &&
               !func->get_returntype()->is_service();
@@ -989,101 +971,36 @@ class mstch_python_service : public mstch_service {
   const t_program* prog_;
 };
 
-class service_python_generator : public service_generator {
+class python_service_factory : public mstch_service_factory {
  public:
-  explicit service_python_generator(const t_program* prog) : prog_{prog} {}
-  std::shared_ptr<mstch_base> generate(
+  explicit python_service_factory(const t_program* prog) : prog_{prog} {}
+
+  std::shared_ptr<mstch_base> make_mstch_object(
       const t_service* service,
-      std::shared_ptr<const mstch_generators> generators,
+      const mstch_factories& factories,
       std::shared_ptr<mstch_cache> cache,
-      ELEMENT_POSITION pos,
-      int32_t /*index*/) const override {
+      mstch_element_position pos) const override {
     return std::make_shared<mstch_python_service>(
-        service, generators, cache, pos, prog_);
+        service, factories, cache, pos, prog_);
   }
 
  protected:
   const t_program* prog_;
 };
 
-class field_python_generator : public field_generator {
- public:
-  std::shared_ptr<mstch_base> generate(
-      const t_field* field,
-      std::shared_ptr<const mstch_generators> generators,
-      std::shared_ptr<mstch_cache> cache,
-      ELEMENT_POSITION pos,
-      int32_t index,
-      field_generator_context const* field_context) const override {
-    return std::make_shared<mstch_python_field>(
-        field, generators, cache, pos, index, field_context);
-  }
-};
-
-class enum_python_generator : public enum_generator {
- public:
-  std::shared_ptr<mstch_base> generate(
-      const t_enum* enm,
-      std::shared_ptr<const mstch_generators> generators,
-      std::shared_ptr<mstch_cache> cache,
-      ELEMENT_POSITION pos,
-      int32_t /*index*/) const override {
-    return std::make_shared<mstch_python_enum>(enm, generators, cache, pos);
-  }
-};
-
-class enum_value_python_generator : public enum_value_generator {
- public:
-  std::shared_ptr<mstch_base> generate(
-      const t_enum_value* enm_value,
-      std::shared_ptr<const mstch_generators> generators,
-      std::shared_ptr<mstch_cache> cache,
-      ELEMENT_POSITION pos,
-      int32_t /*index*/) const override {
-    return std::make_shared<mstch_python_enum_value>(
-        enm_value, generators, cache, pos);
-  }
-};
-
-class const_value_python_generator : public const_value_generator {
- public:
-  const_value_python_generator() = default;
-  ~const_value_python_generator() override = default;
-  std::shared_ptr<mstch_base> generate(
-      t_const_value const* const_value,
-      std::shared_ptr<mstch_generators const> generators,
-      std::shared_ptr<mstch_cache> cache,
-      ELEMENT_POSITION pos,
-      int32_t index,
-      t_const const* current_const,
-      t_type const* expected_type) const override {
-    return std::make_shared<mstch_python_const_value>(
-        const_value,
-        current_const,
-        expected_type,
-        generators,
-        cache,
-        pos,
-        index);
-  }
-};
-
-/**
- * Generator-specific validator that enforces that reserved key is
- * not used as namespace field name.
- */
+// Generator-specific validator that enforces that a reserved key is not used as
+// a namespace component.
 class no_reserved_key_in_namespace_validator : virtual public validator {
  public:
   using validator::visit;
 
-  bool visit(t_program* const prog) override {
-    set_program(prog);
+  bool visit(t_program* prog) override {
     validate(prog);
     return true;
   }
 
  private:
-  void validate(t_program* const prog) {
+  void validate(t_program* prog) {
     auto namespace_tokens = get_py3_namespace(prog);
     if (namespace_tokens.empty()) {
       return;
@@ -1091,31 +1008,29 @@ class no_reserved_key_in_namespace_validator : virtual public validator {
     for (const auto& field_name : namespace_tokens) {
       if (get_python_reserved_names().find(field_name) !=
           get_python_reserved_names().end()) {
-        std::ostringstream ss;
-        ss << "Namespace '" << boost::algorithm::join(namespace_tokens, ".")
-           << "' contains reserved keyword '" << field_name << "'";
-        add_error(boost::none, ss.str());
+        report_error(
+            *prog,
+            "Namespace '{}' contains reserved keyword '{}'",
+            fmt::join(namespace_tokens, "."),
+            field_name);
       }
     }
 
-    std::string filepath_delimiters("\\/.");
     std::vector<std::string> fields;
-    boost::split(fields, prog->path(), boost::is_any_of(filepath_delimiters));
+    boost::split(fields, prog->path(), boost::is_any_of("\\/."));
     for (const auto& field : fields) {
       if (field == "include") {
-        std::ostringstream ss;
-        ss << "Path '" << prog->path()
-           << "' contains reserved keyword 'include'";
-        add_error(boost::none, ss.str());
+        report_error(
+            *prog,
+            "Path '{}' contains reserved keyword 'include'",
+            prog->path());
       }
     }
   }
 };
 
-/**
- * Generator-specific validator that enforces "name" and "value" is not used
- * as enum member or union field names (thrift-py3)
- */
+// Generator-specific validator that enforces "name" and "value" are not used as
+// enum member or union field names (thrift-py3).
 class enum_member_union_field_names_validator : virtual public validator {
  public:
   using validator::visit;
@@ -1141,11 +1056,12 @@ class enum_member_union_field_names_validator : virtual public validator {
   void validate(const t_named* node, const std::string& name) {
     const auto& pyname = node->get_annotation("py3.name", &name);
     if (pyname == "name" || pyname == "value") {
-      std::ostringstream ss;
-      ss << "'" << pyname
-         << "' should not be used as an enum/union field name in thrift-py3. "
-         << "Use a different name or annotate the field with `(py3.name=\"<new_py_name>\")`";
-      add_error(node->get_lineno(), ss.str());
+      report_error(
+          *node,
+          "'{}' should not be used as an enum/union field name in thrift-py3. "
+          "Use a different name or annotate the field with "
+          "`(py3.name=\"<new_py_name>\")`",
+          pyname);
     }
   }
 };
@@ -1155,10 +1071,8 @@ class t_mstch_python_generator : public t_mstch_generator {
   t_mstch_python_generator(
       t_program* program,
       t_generation_context context,
-      const std::map<std::string, std::string>& parsed_options,
-      const std::string& /* option_string unused */)
-      : t_mstch_generator(
-            program, std::move(context), "python", parsed_options),
+      const std::map<std::string, std::string>& options)
+      : t_mstch_generator(program, std::move(context), "python", options),
         generate_root_path_{package_to_path()} {
     out_dir_base_ = "gen-python";
     auto include_prefix = get_option("include_prefix");
@@ -1168,7 +1082,7 @@ class t_mstch_python_generator : public t_mstch_generator {
   }
 
   void generate_program() override {
-    set_mstch_generators();
+    set_mstch_factories();
     generate_types();
     generate_metadata();
     generate_clients();
@@ -1184,7 +1098,7 @@ class t_mstch_python_generator : public t_mstch_generator {
 
  protected:
   bool should_resolve_typedefs() const override { return true; }
-  void set_mstch_generators();
+  void set_mstch_factories();
   void generate_file(
       const std::string& file,
       TypesFile is_types_file,
@@ -1201,25 +1115,18 @@ class t_mstch_python_generator : public t_mstch_generator {
 
 } // namespace
 
-void t_mstch_python_generator::set_mstch_generators() {
-  generators_->set_program_generator(
-      std::make_unique<program_python_generator>());
-  generators_->set_struct_generator(
-      std::make_unique<struct_python_generator>());
-  generators_->set_function_generator(
-      std::make_unique<function_python_generator>());
-  generators_->set_service_generator(
-      std::make_unique<service_python_generator>(get_program()));
-  generators_->set_field_generator(std::make_unique<field_python_generator>());
-  generators_->set_enum_generator(std::make_unique<enum_python_generator>());
-  generators_->set_enum_value_generator(
-      std::make_unique<enum_value_python_generator>());
-  generators_->set_const_value_generator(
-      std::make_unique<const_value_python_generator>());
-  generators_->set_type_generator(
-      std::make_unique<type_python_generator>(get_program()));
-  generators_->set_typedef_generator(
-      std::make_unique<typedef_python_generator>());
+void t_mstch_python_generator::set_mstch_factories() {
+  factories_.program_factory = std::make_unique<python_program_factory>();
+  factories_.service_factory =
+      std::make_unique<python_service_factory>(program_);
+  factories_.set_function_factory<mstch_python_function>();
+  factories_.type_factory = std::make_unique<python_type_factory>(program_);
+  factories_.set_typedef_factory<mstch_python_typedef>();
+  factories_.set_struct_factory<mstch_python_struct>();
+  factories_.set_field_factory<mstch_python_field>();
+  factories_.set_enum_factory<mstch_python_enum>();
+  factories_.set_enum_value_factory<mstch_python_enum_value>();
+  factories_.set_const_value_factory<mstch_python_const_value>();
 }
 
 boost::filesystem::path t_mstch_python_generator::package_to_path() {
@@ -1234,12 +1141,12 @@ void t_mstch_python_generator::generate_file(
   auto program = get_program();
   const auto& name = program->name();
   if (is_types_file == IsTypesFile) {
-    cache_->parsed_options_["is_types_file"] = "";
+    cache_->options_["is_types_file"] = "";
   } else {
-    cache_->parsed_options_.erase("is_types_file");
+    cache_->options_.erase("is_types_file");
   }
-  auto node_ptr =
-      generators_->program_generator_->generate(program, generators_, cache_);
+  auto node_ptr = factories_.program_factory->make_mstch_object(
+      program, factories_, cache_);
   render_to_file(node_ptr, file, base / name / file);
 }
 
@@ -1276,11 +1183,6 @@ THRIFT_REGISTER_GENERATOR(
     "Python",
     "    include_prefix:  Use full include paths in generated files.\n");
 
-static t_generator_factory_impl<t_mstch_python_generator> alias(
-    "mstch_py3lite",
-    "Py3 Lite",
-    "    include_prefix:  Use full include paths in generated files.\n");
-
 } // namespace compiler
 } // namespace thrift
 } // namespace apache
