@@ -18,6 +18,7 @@
 #include "thrift/compiler/ast/t_program.h"
 
 #include <algorithm>
+#include <memory>
 #include <queue>
 #include <stdexcept>
 
@@ -37,7 +38,6 @@
 #include <thrift/compiler/ast/t_struct.h>
 #include <thrift/compiler/ast/t_typedef.h>
 #include <thrift/compiler/gen/cpp/type_resolver.h>
-#include <thrift/compiler/util.h>
 
 namespace apache {
 namespace thrift {
@@ -45,8 +45,12 @@ namespace compiler {
 namespace cpp2 {
 namespace {
 
-const std::string& value_or_empty(const std::string* value) {
-  return value ? *value : empty_instance<std::string>();
+bool contains(fmt::string_view s, fmt::string_view what) {
+  return std::search(s.begin(), s.end(), what.begin(), what.end()) != s.end();
+}
+
+fmt::string_view value_or_empty(const std::string* value) {
+  return value ? *value : std::string_view("");
 }
 
 int checked_stoi(const std::string& s, std::string msg) {
@@ -60,17 +64,17 @@ int checked_stoi(const std::string& s, std::string msg) {
 
 } // namespace
 
-// TODO: check @cpp.Adapter
 bool is_custom_type(const t_type& type) {
   return t_typedef::get_first_annotation_or_null(
-      &type,
-      {
-          "cpp.template",
-          "cpp2.template",
-          "cpp.type",
-          "cpp2.type",
-          "cpp.adapter",
-      });
+             &type,
+             {
+                 "cpp.template",
+                 "cpp2.template",
+                 "cpp.type",
+                 "cpp2.type",
+             }) ||
+      t_typedef::get_first_structured_annotation_or_null(
+             &type, "facebook.com/thrift/annotation/cpp/Adapter");
 }
 
 std::unordered_map<t_struct*, std::vector<t_struct*>>
@@ -87,27 +91,36 @@ gen_struct_dependency_graph(
       }
 
       auto add_dependency = [&](const t_type* type) {
-        type = type->get_true_type();
-        if (const auto strct = dynamic_cast<const t_struct*>(type)) {
-          // We're only interested in types defined in the current program.
-          if (!strct->is_exception() && strct->program() == program) {
-            // TODO(afuller): Remove const cast, once the return type also has
-            // const elements.
-            deps.emplace_back(const_cast<t_struct*>(strct));
+        if (type != nullptr) {
+          type = type->get_true_type();
+          if (const auto strct = dynamic_cast<const t_struct*>(type)) {
+            // We're only interested in types defined in the current program.
+            if (!strct->is_exception() && strct->program() == program) {
+              // TODO(afuller): Remove const cast, once the return type also has
+              // const elements.
+              deps.emplace_back(const_cast<t_struct*>(strct));
+            }
           }
         }
       };
 
-      auto t = f.type()->get_true_type();
-      if (auto map = dynamic_cast<t_map const*>(t)) {
-        // We don't add non-custom type `std::map` to dependency graph since it
-        // supports incomplete types
-        if (cpp2::is_custom_type(*map)) {
-          add_dependency(map->get_key_type());
-          add_dependency(map->get_val_type());
+      // This awkward "resolve-then-check-if-resolved" formulation is to handle
+      // the (malformed) case where a type definition exists for the field but
+      // doesn't actually resolve to an actual type.
+      auto ftype = f.type();
+      ftype.resolve();
+      if (ftype.resolved()) {
+        auto t = ftype->get_true_type();
+        if (auto map = dynamic_cast<t_map const*>(t)) {
+          // We don't add non-custom type `std::map` to dependency graph since
+          // it supports incomplete types
+          if (cpp2::is_custom_type(*map)) {
+            add_dependency(map->get_key_type());
+            add_dependency(map->get_val_type());
+          }
+        } else {
+          add_dependency(t);
         }
-      } else {
-        add_dependency(t);
       }
     }
 
@@ -142,7 +155,10 @@ gen_adapter_dependency_graph(
 
           // Make sure it's still a type we care about
           if (!dynamic_cast<t_typedef const*>(type) &&
-              !(dynamic_cast<t_struct const*>(type) && include_structs)) {
+              !(dynamic_cast<t_struct const*>(type) &&
+                (include_structs ||
+                 gen::cpp::type_resolver::find_first_adapter(
+                     *type->get_true_type())))) {
             return;
           }
         }
@@ -159,11 +175,24 @@ gen_adapter_dependency_graph(
     if (auto* typedf = dynamic_cast<t_typedef const*>(obj)) {
       // The adjacency list of a typedef is the list of structs and typedefs
       // named in its underlying type, but we only care about structs if the
-      // typedef has an adapter annotation.
+      // typedef or struct has an adapter annotation without adaptedType
+      // specified.
       const auto* type = &*typedf->type();
       bool has_adapter = gen::cpp::type_resolver::find_first_adapter(*typedf);
+      if (has_adapter) {
+        if (auto* annotation = typedf->find_structured_annotation_or_null(
+                "facebook.com/thrift/annotation/cpp/Adapter")) {
+          if (annotation->get_value_from_structured_annotation_or_null(
+                  "adaptedType")) {
+            has_adapter = false;
+          }
+        }
+      }
       if (dynamic_cast<t_typedef const*>(type) ||
-          (dynamic_cast<t_struct const*>(type) && has_adapter)) {
+          (dynamic_cast<t_struct const*>(type) &&
+           (has_adapter ||
+            gen::cpp::type_resolver::find_first_adapter(
+                *typedf->get_true_type())))) {
         add_dependency(type, has_adapter);
       }
     } else if (auto* strct = dynamic_cast<t_struct const*>(obj)) {
@@ -199,6 +228,10 @@ bool is_orderable(
   if (!seen.insert(&type).second) {
     return true;
   }
+  auto make_scope_guard = [](auto f) {
+    auto deleter = [=](void*) { f(); };
+    return std::unique_ptr<void, decltype(deleter)>(&f, deleter);
+  };
   auto g = make_scope_guard([&] { seen.erase(&type); });
   // TODO: Consider why typedef is not resolved in this method
   if (type.is_base_type()) {
@@ -254,23 +287,21 @@ bool is_orderable(t_type const& type) {
   return is_orderable(seen, memo, type);
 }
 
-std::string const& get_type(const t_type* type) {
+fmt::string_view get_type(const t_type* type) {
   return value_or_empty(gen::cpp::type_resolver::find_type(*type));
 }
 
 bool is_implicit_ref(const t_type* type) {
   auto const* resolved_typedef = type->get_true_type();
   return resolved_typedef != nullptr && resolved_typedef->is_binary() &&
-      get_type(resolved_typedef).find("std::unique_ptr") != std::string::npos &&
-      get_type(resolved_typedef).find("folly::IOBuf") != std::string::npos;
+      contains(get_type(resolved_typedef), "std::unique_ptr") &&
+      contains(get_type(resolved_typedef), "folly::IOBuf");
 }
 
 bool field_transitively_refers_to_unique(const t_field* field) {
   switch (gen::cpp::find_ref_type(*field)) {
     case gen::cpp::reference_type::none:
-    case gen::cpp::reference_type::unrecognized: {
       break;
-    }
     case gen::cpp::reference_type::unique: {
       return true;
     }
@@ -430,7 +461,6 @@ bool has_ref_annotation(const t_field& field) {
     case gen::cpp::reference_type::unique:
     case gen::cpp::reference_type::shared_const:
     case gen::cpp::reference_type::shared_mutable:
-    case gen::cpp::reference_type::unrecognized:
       return true;
     case gen::cpp::reference_type::none:
     case gen::cpp::reference_type::boxed:
