@@ -18,6 +18,7 @@
 
 #include <initializer_list>
 #include <map>
+#include <stack>
 #include <string>
 #include <unordered_map>
 #include <unordered_set>
@@ -34,6 +35,8 @@
 #include <thrift/compiler/gen/cpp/namespace_resolver.h>
 #include <thrift/compiler/gen/cpp/reference_type.h>
 
+#include <fmt/format.h>
+
 namespace apache {
 namespace thrift {
 namespace compiler {
@@ -68,19 +71,19 @@ gen_adapter_dependency_graph(
     const std::vector</* const */ t_typedef*>& typedefs);
 
 inline std::vector<std::string> get_gen_namespace_components(
-    t_program const& program) {
+    const t_program& program) {
   return gen::cpp::namespace_resolver::gen_namespace_components(program);
 }
 
-inline std::string get_gen_namespace(t_program const& program) {
+inline std::string get_gen_namespace(const t_program& program) {
   return gen::cpp::namespace_resolver::gen_namespace(program);
 }
 
-inline std::string get_gen_unprefixed_namespace(t_program const& program) {
+inline std::string get_gen_unprefixed_namespace(const t_program& program) {
   return gen::cpp::namespace_resolver::gen_unprefixed_namespace(program);
 }
 
-inline std::string get_service_qualified_name(t_service const& service) {
+inline std::string get_service_qualified_name(const t_service& service) {
   return get_gen_namespace(*service.program()) + "::" + get_name(&service);
 }
 
@@ -90,16 +93,16 @@ inline std::string get_service_qualified_name(t_service const& service) {
  * its not considered orderable, and we don't need to generate operator< methods
  */
 bool is_orderable(
-    std::unordered_set<t_type const*>& seen,
-    std::unordered_map<t_type const*, bool>& memo,
-    t_type const& type);
-bool is_orderable(t_type const& type);
+    std::unordered_set<const t_type*>& seen,
+    std::unordered_map<const t_type*, bool>& memo,
+    const t_type& type);
+bool is_orderable(const t_type& type);
 
 /**
  * Return the cpp.type/cpp2.type attribute or empty string if nothing set.
  */
 // TODO(afuller): Replace with type_resolver::get_type_name.
-std::string const& get_type(const t_type* type);
+fmt::string_view get_type(const t_type* type);
 
 /**
  * If the cpp_type is std::unique_ptr<folly::IOBuf> the C++ compiler implicitly
@@ -199,14 +202,14 @@ inline bool is_unique_ref(const t_field* f) {
 }
 
 bool is_stack_arguments(
-    std::map<std::string, std::string> const& options,
-    t_function const& function);
+    const std::map<std::string, std::string>& options,
+    const t_function& function);
 
-int32_t get_split_count(std::map<std::string, std::string> const& options);
+int32_t get_split_count(const std::map<std::string, std::string>& options);
 
 // return client name -- client split count
 std::unordered_map<std::string, int32_t> get_client_name_to_split_count(
-    std::map<std::string, std::string> const& options);
+    const std::map<std::string, std::string>& options);
 
 bool is_mixin(const t_field& field);
 
@@ -246,17 +249,66 @@ std::vector<mixin_member> get_mixins_and_members(const t_struct& strct);
 //
 //  The _with_indirection variant intersperses indirection_tag wherever the
 //  annotation cpp.indirection appears in the corresponding definitions.
-std::string get_gen_type_class(t_type const& type);
-std::string get_gen_type_class_with_indirection(t_type const& type);
+std::string get_gen_type_class(const t_type& type);
+std::string get_gen_type_class_with_indirection(const t_type& type);
 
-std::string sha256_hex(std::string const& in);
+std::string sha256_hex(const std::string& in);
 
 bool is_cpp_ref_unique_either(const t_field* f);
 
 bool deprecated_terse_writes(const t_field* field);
 
-// an implementation of the LPT scheduling
-// Greedy Multiway Partitioning algorithm
+//  topological_sort
+//
+//  Given a container of objects and a function to obtain dependencies,
+//  produces a vector of those nodes in a topologicaly sorted order.
+template <typename T, typename ForwardIt, typename Edges>
+std::vector<T> topological_sort(
+    ForwardIt begin, ForwardIt end, const Edges& edges) {
+  struct iter_state {
+    T node;
+    std::vector<T> edges;
+    typename std::vector<T>::const_iterator pos;
+
+    iter_state(T n, std::vector<T> e)
+        : node(std::move(n)), edges(std::move(e)), pos(edges.begin()) {}
+
+    // Prevent an accidental move/copy, because the iterator needs to be
+    // properly updated.
+    iter_state(const iter_state&) = delete;
+    iter_state& operator=(const iter_state&) = delete;
+  };
+
+  std::unordered_set<T> visited;
+  std::vector<T> output;
+
+  for (auto it = begin; it != end; ++it) {
+    if (visited.count(*it) != 0) {
+      continue;
+    }
+    std::stack<iter_state> st;
+    st.emplace(*it, edges.at(*it));
+    visited.insert(*it);
+    while (!st.empty()) {
+      iter_state& s = st.top();
+      if (s.pos == s.edges.end()) {
+        output.emplace_back(s.node);
+        st.pop();
+        continue;
+      }
+
+      if (visited.find(*s.pos) == visited.end()) {
+        st.emplace(*s.pos, edges.at(*s.pos));
+        visited.insert(*s.pos);
+      }
+      ++s.pos;
+    }
+  }
+  return output;
+}
+
+// An implementation of the LPT scheduling Greedy Multiway Partitioning
+// algorithm.
 template <class T, class F>
 auto lpt_split(std::vector<T> vec, size_t k, F size) {
   std::vector<std::vector<T>> ret(k);
