@@ -1,5 +1,5 @@
 /*
- * Copyright (c) Facebook, Inc. and its affiliates.
+ * Copyright (c) Meta Platforms, Inc. and affiliates.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -16,16 +16,15 @@
 
 #pragma once
 
-#include <thrift/compiler/ast/diagnostic.h>
 #include <thrift/compiler/ast/visitor.h>
-#include <thrift/compiler/parse/parsing_driver.h>
+#include <thrift/compiler/diagnostic.h>
 
 namespace apache {
 namespace thrift {
 namespace compiler {
 
-struct MutatorException : std::exception {
-  explicit MutatorException(diagnostic msg) : message(std::move(msg)) {}
+struct mutator_exception : std::exception {
+  explicit mutator_exception(diagnostic msg) : message(std::move(msg)) {}
 
   diagnostic message;
 };
