@@ -80,6 +80,10 @@ const char* getOptionalPatchTypeName(t_base_type::type base_type) {
   }
 }
 
+std::string getSibName(const std::string& sibling, const std::string& name) {
+  return sibling.substr(0, sibling.find_last_of("/")) + name;
+}
+
 // A fluent function to set the doc string on a given node.
 template <typename N>
 N& doc(std::string txt, N& node) {
@@ -119,11 +123,26 @@ struct StructGen {
   operator t_struct&() { return generated; }
   operator t_type_ref() { return generated; }
 
-  void set_adapter(std::string name) {
-    generated.set_annotation(
-        "cpp.adapter", "::apache::thrift::op::detail::" + std::move(name));
-    generated.set_annotation("cpp.name", generated.name() + "Struct");
-    generated.set_annotation("cpp.detail.adapted_alias");
+  void set_adapter(std::string name, t_program& program) {
+    auto annotation = dynamic_cast<const t_type*>(program.scope()->find_def(
+        "facebook.com/thrift/annotation/cpp/Adapter"));
+    assert(annotation); // transitive include from patch.thrift
+    auto value = std::make_unique<t_const_value>();
+    value->set_map();
+    value->add_map(
+        std::make_unique<t_const_value>("name"),
+        std::make_unique<t_const_value>(
+            "::apache::thrift::op::detail::" + std::move(name)));
+    value->add_map(
+        std::make_unique<t_const_value>("underlyingName"),
+        std::make_unique<t_const_value>(generated.name() + "Struct"));
+    value->add_map(
+        std::make_unique<t_const_value>("extraNamespace"),
+        std::make_unique<t_const_value>(""));
+    value->set_ttype(*annotation);
+    auto adapter =
+        std::make_unique<t_const>(&program, annotation, "", std::move(value));
+    generated.add_structured_annotation(std::move(adapter));
   }
 };
 
@@ -131,25 +150,23 @@ struct StructGen {
 struct PatchGen : StructGen {
   // Standardized patch field ids.
   enum t_patch_field_id : t_field_id {
+    // All values.
     kAssignId = 1,
     kClearId = 2,
-    kPatchId = 3,
 
-    // Optional, Union
+    // Optional, Union, Struct
+    kPatchId = 3,
     kEnsureId = 4,
-    kPatchAfterId = 5,
+    kPatchAfterId = 6,
+
+    // Set, Map, Number
+    kRemoveId = 7,
+    kAddId = 8,
+    kPutId = 9,
 
     // List, String, Binary
-    kPrependId = 4,
-    kAppendId = 5,
-
-    // Set, Map
-    kRemoveId = 4,
-    kAddId = 5,
-
-    // TODO(afuller): Add 'replace' op.
-    // kReplaceId = 6,
-    kPutId = 7, // Map Patch
+    kAppendId = 9,
+    kPrependId = 10,
   };
 
   // {kAssignId}: optional {type} assign (thrift.box);
@@ -248,7 +265,7 @@ void generate_optional_patch(
       patch_generator::get_for(ctx, mctx).add_optional_patch(
           *annot, value_type, node);
     } else {
-      ctx.failure(
+      ctx.error(
           "Could not resolve the 'value' type, needed to generate the optional patch struct.");
     }
   }
@@ -262,13 +279,12 @@ void generate_struct_patch(
           ctx.program().inherit_annotation_or_null(node, kGeneratePatchUri)) {
     auto& generator = patch_generator::get_for(ctx, mctx);
 
-    // Add a 'structured patch' and 'struct value patch' using it.
-    auto& struct_patch =
-        generator.add_structured_patch(*annot, node, "StructPatchAdapter");
-    auto& patch = generator.add_struct_value_patch(*annot, node, struct_patch);
+    // Add a 'field patch' and 'struct patch' using it.
+    auto& patch = generator.add_field_patch(*annot, node);
+    auto& struct_patch = generator.add_struct_patch(*annot, node, patch);
 
     // Add an 'optional patch' based on the added patch type.
-    generator.add_optional_patch(*annot, node, patch);
+    generator.add_optional_patch(*annot, node, struct_patch);
   }
 }
 
@@ -279,12 +295,11 @@ void generate_union_patch(
     auto& generator = patch_generator::get_for(ctx, mctx);
 
     // Add a 'structured patch' and 'union value patch' using it.
-    auto& struct_patch =
-        generator.add_structured_patch(*annot, node, "UnionPatchAdapter");
-    auto& patch = generator.add_union_value_patch(*annot, node, struct_patch);
+    auto& patch = generator.add_field_patch(*annot, node);
+    auto& union_patch = generator.add_union_patch(*annot, node, patch);
 
     // Add an 'optional patch' based on the added patch type.
-    generator.add_optional_patch(*annot, node, patch);
+    generator.add_optional_patch(*annot, node, union_patch);
   }
 }
 
@@ -312,26 +327,13 @@ t_struct& patch_generator::add_optional_patch(
   gen.patchOpt(patch_type);
   box(gen.ensure(value_type));
   gen.patchAfter(patch_type);
-  gen.set_adapter("OptionalPatchAdapter");
+  gen.set_adapter("OptionalPatchAdapter", program_);
   return gen;
 }
 
-t_struct& patch_generator::add_union_value_patch(
-    const t_node& annot, t_union& value_type, t_type_ref patch_type) {
-  PatchGen gen{{annot, gen_suffix_struct(annot, value_type, "ValuePatch")}};
-  // TODO(afuller): Add 'assign`.
-  gen.clearOpt();
-  gen.patchOpt(patch_type);
-  gen.ensure(value_type);
-  // TODO(afuller): Add 'maybeEnsure'.
-  gen.patchAfter(patch_type);
-  gen.set_adapter("UnionValuePatchAdapter");
-  return gen;
-}
-
-t_struct& patch_generator::add_structured_patch(
-    const t_const& annot, t_structured& orig, const char* adapter) {
-  StructGen gen{annot, gen_suffix_struct(annot, orig, "Patch")};
+t_struct& patch_generator::add_field_patch(
+    const t_const& annot, t_structured& orig) {
+  StructGen gen{annot, gen_suffix_struct(annot, orig, "FieldPatch")};
   for (const auto& field : orig.fields()) {
     if (t_type_ref patch_type = find_patch_type(annot, orig, field)) {
       gen.field(field.id(), patch_type, field.name());
@@ -339,17 +341,30 @@ t_struct& patch_generator::add_structured_patch(
       ctx_.warning(field, "Could not resolve patch type for field.");
     }
   }
-  gen.set_adapter(adapter);
+  gen.set_adapter("FieldPatchAdapter", program_);
   return gen;
 }
 
-t_struct& patch_generator::add_struct_value_patch(
+t_struct& patch_generator::add_union_patch(
+    const t_node& annot, t_union& value_type, t_type_ref patch_type) {
+  PatchGen gen{{annot, gen_suffix_struct(annot, value_type, "Patch")}};
+  // TODO(afuller): Add 'assign`.
+  gen.clearOpt();
+  gen.patchOpt(patch_type);
+  gen.ensure(value_type);
+  // TODO(afuller): Add 'maybeEnsure'.
+  gen.patchAfter(patch_type);
+  gen.set_adapter("UnionPatchAdapter", program_);
+  return gen;
+}
+
+t_struct& patch_generator::add_struct_patch(
     const t_node& annot, t_struct& value_type, t_type_ref patch_type) {
-  PatchGen gen{{annot, gen_suffix_struct(annot, value_type, "ValuePatch")}};
+  PatchGen gen{{annot, gen_suffix_struct(annot, value_type, "Patch")}};
   gen.assign(value_type);
   gen.clear();
   gen.patch(patch_type);
-  gen.set_adapter("StructValuePatchAdapter");
+  gen.set_adapter("StructPatchAdapter", program_);
   return gen;
 }
 
@@ -364,32 +379,61 @@ t_type_ref patch_generator::find_patch_type(
     if (const auto* result = program_.scope()->find_type(name)) {
       return t_type_ref::from_ptr(result);
     }
+
     // TODO(afuller): This look up hack only works for 'built-in' patch types.
     // Use a shared uri type registry instead.
-    if (const auto* result =
-            annot.type()->program()->scope()->find_type(name)) {
-      return t_type_ref::from_ptr(result);
+    t_type_ref result = annot.type()->program()->scope()->ref_type(
+        *annot.type()->program(), name, field.src_range());
+    if (auto* ph = result.get_unresolved_type()) {
+      // Set the location info, in case the type can't be resolved later.
+      ph->set_lineno(field.lineno());
+      ph->set_src_range(field.src_range());
+      ph->set_generated();
     }
 
-    // These type should always be availabile because the are defined along
-    // side the annoation used to trigger patch generation.
-    ctx_.failure(field, [&](auto& os) {
-      os << "Could not find expected patch type: " << name;
-    });
-    return {};
-  } else if (auto* structured = dynamic_cast<const t_structured*>(type)) {
-    // Try to find the generated patch type.
-    std::string name = structured->name() + "ValuePatch";
+    return result;
+  }
+
+  // Check the field for a custom patch type.
+  if (auto* custom = field.find_annotation_or_null("thrift.patch.uri")) {
+    if (const auto* result =
+            dynamic_cast<const t_type*>(program_.scope()->find_def(*custom))) {
+      return t_type_ref::from_ptr(result);
+    }
+    ctx_.warning(field, "Could not find custom type: {}", *custom);
+  }
+
+  // Check the field type for a custom patch type.
+  if (auto* custom = t_typedef::get_first_annotation_or_null(
+          field.type().get_type(), {"thrift.patch.uri"})) {
+    if (const auto* result =
+            dynamic_cast<const t_type*>(program_.scope()->find_def(*custom))) {
+      return t_type_ref::from_ptr(result);
+    }
+    ctx_.warning(*field.type(), "Could not find custom type: {}", *custom);
+  }
+
+  if (auto* structured = dynamic_cast<const t_structured*>(type)) {
+    std::string name = structured->name() + "Patch";
     if (field.qualifier() == t_field_qualifier::optional) {
       name = "Optional" + std::move(name);
     }
-    // It should be in the same program as the type itself.
-    t_type_ref result =
-        program_.scope()->ref_type(*structured->program(), name);
+
+    if (!structured->uri().empty()) { // Try to look up by URI.
+      if (auto* result = dynamic_cast<const t_type*>(program_.scope()->find_def(
+              getSibName(structured->uri(), name)))) {
+        return t_type_ref::from_ptr(result);
+      }
+    }
+
+    // Try to look up by Name.
+    // Look for it in the same program as the type itself.
+    t_type_ref result = program_.scope()->ref_type(
+        *structured->program(), name, field.src_range());
     if (auto* ph = result.get_unresolved_type()) {
       // Set the location info, in case the type can't be resolved later.
-      ph->set_src_range(field.src_range());
       ph->set_lineno(field.lineno());
+      ph->set_src_range(field.src_range());
       ph->set_generated();
     }
     return result;
@@ -410,13 +454,13 @@ t_type_ref patch_generator::find_patch_type(
         // TODO(afuller): support 'removeIf' op.
         gen.prepend(field.type());
         gen.append(field.type());
-        gen.set_adapter("ListPatchAdapter");
+        gen.set_adapter("ListPatchAdapter", program_);
         break;
       case t_container::type::t_set:
         // TODO(afuller): support 'replace' op.
         gen.remove(field.type());
         gen.add(field.type());
-        gen.set_adapter("SetPatchAdapter");
+        gen.set_adapter("SetPatchAdapter", program_);
         break;
       case t_container::type::t_map:
         // TODO(afuller): support 'patch' op.
@@ -425,11 +469,11 @@ t_type_ref patch_generator::find_patch_type(
         // TODO(afuller): support 'removeIf' op.
         gen.put(field.type());
         // TODO(afuller): support 'add' op.
-        gen.set_adapter("MapPatchAdapter");
+        gen.set_adapter("MapPatchAdapter", program_);
         break;
     }
   } else {
-    gen.set_adapter("AssignPatchAdapter");
+    gen.set_adapter("AssignPatchAdapter", program_);
   }
 
   if (field.qualifier() == t_field_qualifier::optional) {
@@ -441,20 +485,17 @@ t_type_ref patch_generator::find_patch_type(
 t_struct& patch_generator::gen_struct(
     const t_node& annot, std::string name, std::string uri) {
   auto generated = std::make_unique<t_struct>(&program_, std::move(name));
-  t_struct* ptr = generated.get();
   generated->set_generated();
   generated->set_uri(std::move(uri));
   // Attribute the new struct to the anntation.
   generated->set_lineno(annot.lineno());
   program_.scope()->add_type(program_.scope_name(*generated), generated.get());
-  program_.add_definition(std::move(generated));
-  return *ptr;
+  return program_.add_def(std::move(generated));
 }
 
 t_struct& patch_generator::gen_suffix_struct(
     const t_node& annot, const t_named& orig, const char* suffix) {
-  ctx_.failure_if(
-      orig.uri().empty(), annot, "URI required to support patching.");
+  ctx_.check(!orig.uri().empty(), annot, "URI required to support patching.");
   t_struct& generated =
       gen_struct(annot, orig.name() + suffix, orig.uri() + suffix);
   if (const auto* cpp_name = orig.find_annotation_or_null("cpp.name")) {
