@@ -17,9 +17,12 @@
 #include <thrift/compiler/sema/standard_validator.h>
 
 #include <algorithm>
+#include <set>
 #include <string>
 #include <unordered_map>
 
+#include <fmt/ranges.h>
+
 #include <thrift/compiler/ast/name_index.h>
 #include <thrift/compiler/ast/t_enum.h>
 #include <thrift/compiler/ast/t_enum_value.h>
@@ -48,7 +51,10 @@ namespace {
 
 constexpr auto kCppRefUri = "facebook.com/thrift/annotation/cpp/Ref";
 constexpr auto kCppAdapterUri = "facebook.com/thrift/annotation/cpp/Adapter";
+constexpr auto kCppFieldInterceptorUri =
+    "facebook.com/thrift/annotation/cpp/FieldInterceptor";
 constexpr auto kHackAdapterUri = "facebook.com/thrift/annotation/hack/Adapter";
+constexpr auto kReserveIdsUri = "facebook.com/thrift/annotation/ReserveIds";
 constexpr auto kCppUnstructuredAdapter = "cpp.adapter";
 constexpr auto kHackUnstructuredAdapter = "hack.adapter";
 
@@ -69,39 +75,30 @@ bool has_lazy_field(const t_structured& node) {
 }
 
 // Reports an existing name was redefined within the given parent node.
-void report_redef_failure(
-    diagnostic_context& ctx,
+void report_redef_error(
+    diagnostics_engine& diags,
     const char* kind,
     const std::string& name,
-    const std::string& path,
     const t_named& parent,
     const t_node& child,
     const t_node& /*existing*/) {
   // TODO(afuller): Use `existing` to provide more detail in the
   // diagnostic.
-  ctx.failure(child, path, [&](auto& o) {
-    o << kind << " `" << name << "` is already defined for `" << parent.name()
-      << "`.";
-  });
-}
-
-void report_redef_failure(
-    diagnostic_context& ctx,
-    const char* kind,
-    const std::string& name,
-    const t_named& parent,
-    const t_node& child,
-    const t_node& existing) {
-  report_redef_failure(
-      ctx, kind, name, ctx.program().path(), parent, child, existing);
+  diags.report(
+      child,
+      diagnostic_level::error,
+      "{} `{}` is already defined for `{}`.",
+      kind,
+      name,
+      parent.name());
 }
 
 // Helper for checking for the redefinition of a name in the context of a node.
 class redef_checker {
  public:
   redef_checker(
-      diagnostic_context& ctx, const char* kind, const t_named& parent)
-      : ctx_(ctx), kind_(kind), parent_(parent) {}
+      diagnostics_engine& diags, const char* kind, const t_named& parent)
+      : diags_(diags), kind_(kind), parent_(parent) {}
 
   // Checks if the given `name`, derived from `node` via `child`, has already
   // been defined.
@@ -115,13 +112,17 @@ class redef_checker {
     if (const auto* existing = seen_.put(name, node)) {
       if (&node == &parent_ && existing == &parent_) {
         // The degenerate case where parent_ is conflicting with itself.
-        report_redef_failure(ctx_, kind_, name, parent_, child, *existing);
+        report_redef_error(diags_, kind_, name, parent_, child, *existing);
       } else {
-        ctx_.failure(child, [&](auto& o) {
-          o << kind_ << " `" << node.name() << "." << name << "` and `"
-            << existing->name() << "." << name
-            << "` can not have same name in `" << parent_.name() << "`.";
-        });
+        diags_.error(
+            child,
+            "{} `{}.{}` and `{}.{}` can not have same name in `{}`.",
+            kind_,
+            node.name(),
+            name,
+            existing->name(),
+            name,
+            parent_.name());
       }
     }
   }
@@ -134,8 +135,8 @@ class redef_checker {
   // For example, all functions in an interface.
   void check(const t_named& child) {
     if (const auto* existing = seen_.put(child)) {
-      report_redef_failure(
-          ctx_, kind_, child.name(), parent_, child, *existing);
+      report_redef_error(
+          diags_, kind_, child.name(), parent_, child, *existing);
     }
   }
   void check(t_named&& child) = delete;
@@ -148,7 +149,7 @@ class redef_checker {
   }
 
  private:
-  diagnostic_context& ctx_;
+  diagnostics_engine& diags_;
   const char* kind_;
   const t_named& parent_;
 
@@ -175,20 +176,18 @@ class adapter_checker {
     try {
       adapter_annotation->get_value_from_structured_annotation("name");
     } catch (const std::exception& e) {
-      ctx_.failure([&](auto& o) { o << e.what(); });
+      ctx_.error("{}", e.what());
       return;
     }
 
-    // TODO (dokwon): Do not allow composing unstructured adapter annotation on
-    // typedef as well.
     if (const auto* typedf = dynamic_cast<const t_typedef*>(&node)) {
       if (t_typedef::get_first_structured_annotation_or_null(
               &*typedf->type(), structured_adapter_annotation)) {
-        ctx_.failure([&](auto& o) {
-          o << "The `" << structured_adapter_annotation_error_name
-            << "` annotation cannot be annotated more than once in all typedef levels in `"
-            << node.name() << "`.";
-        });
+        ctx_.error(
+            "The `{}` annotation cannot be annotated more than once in all "
+            "typedef levels in `{}`.",
+            structured_adapter_annotation_error_name,
+            node.name());
       }
     }
   }
@@ -220,21 +219,20 @@ class adapter_checker {
 
     if (disallow_structured_annotations_on_both_field_and_typedef &&
         structured_annotation_on_field && structured_annotation_on_typedef) {
-      ctx_.failure([&](auto& o) {
-        o << "`" << structured_adapter_annotation_error_name
-          << "` cannot be applied on both field and typedef in `"
-          << field.name() << "`.";
-      });
+      ctx_.error(
+          "`{}` cannot be applied on both field and typedef in `{}`.",
+          structured_adapter_annotation_error_name,
+          field.name());
     }
 
     if (structured_annotation_on_typedef || structured_annotation_on_field) {
       if (t_typedef::get_first_annotation_or_null(
               type, {unstructured_adapter_annotation})) {
-        ctx_.failure([&](auto& o) {
-          o << "`" << structured_adapter_annotation_error_name
-            << "` cannot be combined with `" << unstructured_adapter_annotation
-            << "` in `" << field.name() << "`.";
-        });
+        ctx_.error(
+            "`{}` cannot be combined with `{}` in `{}`.",
+            structured_adapter_annotation_error_name,
+            unstructured_adapter_annotation,
+            field.name());
       }
     }
   }
@@ -300,11 +298,12 @@ void validate_extends_service_function_name_uniqueness(
   for (const auto& function : node.functions()) {
     if (const auto* existing_service =
             extends_metadata.function_name_to_service.find(function.name())) {
-      ctx.failure(function, [&](auto& o) {
-        o << "Function `" << node.name() << "." << function.name()
-          << "` redefines `" << existing_service->get_full_name() << "."
-          << function.name() << "`.";
-      });
+      ctx.error(
+          function,
+          "Function `{0}.{2}` redefines `{1}.{2}`.",
+          node.name(),
+          existing_service->get_full_name(),
+          function.name());
     }
   }
 }
@@ -347,10 +346,12 @@ void validate_union_field_attributes(
     if (field.qualifier() != t_field_qualifier::none) {
       auto qual = field.qualifier() == t_field_qualifier::required ? "required"
                                                                    : "optional";
-      ctx.failure(field, [&](auto& o) {
-        o << "Unions cannot contain qualified fields. Remove `" << qual
-          << "` qualifier from field `" << field.name() << "`.";
-      });
+      ctx.error(
+          field,
+          "Unions cannot contain qualified fields. Remove `{}` qualifier from "
+          "field `{}`.",
+          qual,
+          field.name());
     }
   }
 }
@@ -364,20 +365,20 @@ void validate_boxed_field_attributes(
   ctx.check(
       dynamic_cast<const t_union*>(ctx.parent()) ||
           node.qualifier() == t_field_qualifier::optional,
-      "The `cpp.box` annotation can only be used with optional fields. Make sure `{}` is optional.",
+      "The `cpp.box` annotation can only be used with optional fields. "
+      "Make sure `{}` is optional.",
       node.name());
 
-  ctx.failure_if(
-      node.has_annotation({
+  ctx.check(
+      !node.has_annotation({
           "cpp.ref",
           "cpp2.ref",
           "cpp.ref_type",
           "cpp2.ref_type",
       }),
-      [&](auto& o) {
-        o << "The `cpp.box` annotation cannot be combined with the `cpp.ref` or `cpp.ref_type` annotations. Remove one of the annotations from `"
-          << node.name() << "`.";
-      });
+      "The `cpp.box` annotation cannot be combined with the `cpp.ref` or "
+      "`cpp.ref_type` annotations. Remove one of the annotations from `{}`.",
+      node.name());
 }
 
 // Checks the attributes of a mixin field.
@@ -395,18 +396,16 @@ void validate_mixin_field_attributes(
       ttype->get_name());
 
   if (const auto* parent = dynamic_cast<const t_union*>(ctx.parent())) {
-    ctx.failure([&](auto& o) {
-      o << "Union `" << parent->name() << "` cannot contain mixin field `"
-        << node.name() << "`.";
-    });
+    ctx.error(
+        "Union `{}` cannot contain mixin field `{}`.",
+        parent->name(),
+        node.name());
   } else if (node.qualifier() == t_field_qualifier::optional) {
     // Nothing technically stops us from marking optional field mixin.
     // However, this will bring surprising behavior. e.g. `foo.bar_ref()`
     // might throw `bad_field_access` if `bar` is inside optional mixin
     // field.
-    ctx.failure([&](auto& o) {
-      o << "Mixin field `" << node.name() << "` cannot be optional.";
-    });
+    ctx.error("Mixin field `{}` cannot be optional.", node.name());
   }
 }
 
@@ -451,15 +450,19 @@ void validate_enum_value_uniqueness(
 
 void validate_enum_value(diagnostic_context& ctx, const t_enum_value& node) {
   if (!node.has_value()) {
-    ctx.failure([&](auto& o) {
-      o << "The enum value, `" << node.name()
-        << "`, must have an explicitly assigned value.";
-    });
+    ctx.error(
+        "The enum value, `{}`, must have an explicitly assigned value.",
+        node.name());
   }
 }
 
 void validate_const_type_and_value(diagnostic_context& ctx, const t_const& c) {
   check_const_rec(ctx, c, &c.type().deref(), c.value());
+  ctx.check(
+      !c.find_structured_annotation_or_null(kCppAdapterUri) ||
+          ctx.has(context_type::experimental),
+      "Using adapters on const `{}` is only allowed in the experimental mode.",
+      c.name());
 }
 
 void validate_field_default_value(
@@ -475,7 +478,7 @@ void validate_structured_annotation(
   for (const auto* annot : node.structured_annotations()) {
     auto result = seen.emplace(&annot->type().deref(), annot);
     if (!result.second) {
-      report_redef_failure(
+      report_redef_error(
           ctx,
           "Structured annotation",
           result.first->first->name(),
@@ -490,55 +493,55 @@ void validate_structured_annotation(
 void validate_uri_uniqueness(diagnostic_context& ctx, const t_program& prog) {
   // TODO: use string_view as map key
   std::unordered_map<std::string, const t_named*> uri_to_node;
-  basic_ast_visitor<true, const std::string&> visit;
-  visit.add_definition_visitor(
-      [&](const std::string& path, const t_named& node) {
-        const auto& uri = node.uri();
-        if (uri.empty() || uri == t_named::kTransitiveUri) {
-          return;
-        }
-        auto result = uri_to_node.emplace(uri, &node);
-        if (!result.second) {
-          report_redef_failure(
-              ctx, "Thrift URI", uri, path, node, node, *result.first->second);
-        }
-      });
+  basic_ast_visitor<true> visit;
+  visit.add_definition_visitor([&](const t_named& node) {
+    const auto& uri = node.uri();
+    if (uri.empty() || uri == t_named::kTransitiveUri) {
+      return;
+    }
+    auto result = uri_to_node.emplace(uri, &node);
+    if (!result.second) {
+      report_redef_error(
+          ctx, "Thrift URI", uri, node, node, *result.first->second);
+    }
+  });
   for (const auto* p : prog.get_included_programs()) {
-    visit(p->path(), *p);
+    visit(*p);
   }
-  visit(prog.path(), prog);
+  visit(prog);
 }
 
 void validate_field_id(diagnostic_context& ctx, const t_field& node) {
-  if (node.explicit_id() != node.id()) {
-    ctx.warning(
+  if (node.explicit_id() != node.id() && !ctx.has(context_type::testing)) {
+    ctx.report(
+        node,
+        ctx.has(context_type::no_legacy) && !ctx.has(context_type::experimental)
+            ? diagnostic_level::error
+            : diagnostic_level::warning,
         "No field id specified for `{}`, resulting protocol may have conflicts "
         "or not be backwards compatible!",
         node.name());
   }
 
-  ctx.failure_if(
-      node.id() == 0 &&
-          !node.has_annotation("cpp.deprecated_allow_zero_as_field_id"),
-      [&](auto& o) {
-        o << "Zero value (0) not allowed as a field id for `" << node.get_name()
-          << "`";
-      });
+  ctx.check(
+      node.id() != 0 ||
+          node.has_annotation("cpp.deprecated_allow_zero_as_field_id"),
+      "Zero value (0) not allowed as a field id for `{}`",
+      node.get_name());
 
-  ctx.failure_if(
-      !node.is_injected() && node.id() < t_field::min_id, [&](auto& o) {
-        o << "Reserved field id (" << node.id() << ") cannot be used for `"
-          << node.name() << "`.";
-      });
+  ctx.check(
+      node.id() >= t_field::min_id || node.is_injected(),
+      "Reserved field id ({}) cannot be used for `{}`.",
+      node.id(),
+      node.name());
 }
 
 void validate_compatibility_with_lazy_field(
     diagnostic_context& ctx, const t_structured& node) {
   if (has_lazy_field(node) && node.has_annotation("cpp.methods")) {
-    ctx.failure([&](auto& o) {
-      o << "cpp.methods is incompatible with lazy deserialization in struct `"
-        << node.get_name() << "`";
-    });
+    ctx.error(
+        "cpp.methods is incompatible with lazy deserialization in struct `{}`",
+        node.get_name());
   }
 }
 
@@ -546,10 +549,10 @@ void validate_ref_annotation(diagnostic_context& ctx, const t_field& node) {
   if (node.find_structured_annotation_or_null(kCppRefUri) &&
       node.has_annotation(
           {"cpp.ref", "cpp2.ref", "cpp.ref_type", "cpp2.ref_type"})) {
-    ctx.failure([&](auto& o) {
-      o << "The @cpp.Ref annotation cannot be combined with the `cpp.ref` or `cpp.ref_type` annotations. Remove one of the annotations from `"
-        << node.name() << "`.";
-    });
+    ctx.error(
+        "The @cpp.Ref annotation cannot be combined with the `cpp.ref` or "
+        "`cpp.ref_type` annotations. Remove one of the annotations from `{}`.",
+        node.name());
   }
 }
 
@@ -588,11 +591,10 @@ void validate_ref_unique_and_box_annotation(
   if (cpp2::is_unique_ref(&node)) {
     if (node.has_annotation({"cpp.ref", "cpp2.ref"})) {
       if (adapter_annotation) {
-        ctx.failure([&](auto& o) {
-          o << "cpp.ref, cpp2.ref "
-            << "are deprecated. Please use @thrift.Box annotation instead in `"
-            << node.name() << "` with @cpp.Adapter.";
-        });
+        ctx.error(
+            "cpp.ref, cpp2.ref are deprecated. Please use @thrift.Box "
+            "annotation instead in `{}` with @cpp.Adapter.",
+            node.name());
       } else {
         ctx.warning(
             "cpp.ref, cpp2.ref are deprecated. Please use @thrift.Box "
@@ -602,26 +604,24 @@ void validate_ref_unique_and_box_annotation(
     }
     if (node.has_annotation({"cpp.ref_type", "cpp2.ref_type"})) {
       if (adapter_annotation) {
-        ctx.failure([&](auto& o) {
-          o << "cpp.ref_type = `unique`, cpp2.ref_type = `unique` "
-            << "are deprecated. Please use @thrift.Box annotation instead in `"
-            << node.name() << "` with @cpp.Adapter.";
-        });
+        ctx.error(
+            "cpp.ref_type = `unique`, cpp2.ref_type = `unique` are deprecated. "
+            "Please use @thrift.Box annotation instead in `{}` with "
+            "@cpp.Adapter.",
+            node.name());
       } else {
         ctx.warning(
-            "cpp.ref_type = `unique`, cpp2.ref_type = `unique` "
-            "are deprecated. Please use @thrift.Box annotation instead in "
-            "`{}`.",
+            "cpp.ref_type = `unique`, cpp2.ref_type = `unique` are deprecated. "
+            "Please use @thrift.Box annotation instead in `{}`.",
             node.name());
       }
     }
     if (node.find_structured_annotation_or_null(kCppRefUri) != nullptr) {
       if (adapter_annotation) {
-        ctx.failure([&](auto& o) {
-          o << "@cpp.Ref{type = cpp.RefType.Unique} "
-            << "is deprecated. Please use @thrift.Box annotation instead in `"
-            << node.name() << "` with @cpp.Adapter.";
-        });
+        ctx.error(
+            "@cpp.Ref{{type = cpp.RefType.Unique}} is deprecated. Please use "
+            "@thrift.Box annotation instead in `{}` with @cpp.Adapter.",
+            node.name());
       } else {
         ctx.warning(
             "@cpp.Ref{{type = cpp.RefType.Unique}} is deprecated. Please use "
@@ -635,18 +635,14 @@ void validate_ref_unique_and_box_annotation(
 void validate_function_priority_annotation(
     diagnostic_context& ctx, const t_node& node) {
   if (auto* priority = node.find_annotation_or_null("priority")) {
-    std::string choices[] = {
+    const std::string choices[] = {
         "HIGH_IMPORTANT", "HIGH", "IMPORTANT", "NORMAL", "BEST_EFFORT"};
     auto* end = choices + sizeof(choices) / sizeof(choices[0]);
-    ctx.failure_if(std::find(choices, end, *priority) == end, [&](auto& o) {
-      o << "Bad priority '" << *priority << "'. Choose one of '";
-      auto delim = "";
-      for (const auto& choice : choices) {
-        o << delim << choice;
-        delim = "','";
-      }
-      o << "'.";
-    });
+    ctx.check(
+        std::find(choices, end, *priority) != end,
+        "Bad priority '{}'. Choose one of {}.",
+        *priority,
+        choices);
   }
 }
 
@@ -670,15 +666,17 @@ void validate_exception_php_annotations(
     const std::string& v = node.get_annotation("message");
     const auto* field = node.get_field_by_name(v);
     if (field == nullptr) {
-      ctx.failure([&](auto& o) {
-        o << "member specified as exception 'message' should be a valid"
-          << " struct member, '" << v << "' in '" << node.name() << "' is not";
-      });
+      ctx.error(
+          "member specified as exception 'message' should be a valid "
+          "struct member, '{}' in '{}' is not",
+          v,
+          node.name());
     } else if (!field->get_type()->is_string_or_binary()) {
-      ctx.failure([&](auto& o) {
-        o << "member specified as exception 'message' should be of type "
-          << "STRING, '" << v << "' in '" << node.name() << "' is not";
-      });
+      ctx.error(
+          "member specified as exception 'message' should be of type "
+          "STRING, '{}' in '{}' is not",
+          v,
+          node.name());
     }
   }
 }
@@ -716,9 +714,22 @@ void validate_stream_exceptions_return_type(
 void validate_interaction_factories(
     diagnostic_context& ctx, const t_function& node) {
   if (node.is_interaction_member() && node.returned_interaction()) {
-    ctx.failure([&](auto& o) {
-      o << "Nested interactions are forbidden: " << node.name();
-    });
+    ctx.error("Nested interactions are forbidden: {}", node.name());
+  }
+}
+
+void validate_cpp_field_interceptor_annotation(
+    diagnostic_context& ctx, const t_field& field) {
+  if (const t_const* annot =
+          field.find_structured_annotation_or_null(kCppFieldInterceptorUri)) {
+    try {
+      annot->get_value_from_structured_annotation("name");
+    } catch (const std::exception&) {
+      ctx.error(
+          "`@cpp.FieldInterceptor` cannot be used without `name` specified in `{}`.",
+          field.name());
+      return;
+    }
   }
 }
 
@@ -741,6 +752,124 @@ void validate_hack_field_adapter_annotation(
       "@hack.Adapter",
       true /* disallow_structured_annotations_on_both_field_and_typedef */);
 }
+
+class reserved_ids_checker {
+ public:
+  explicit reserved_ids_checker(diagnostic_context& ctx) : ctx_(ctx) {}
+
+  void check(const t_structured& node) {
+    auto reserved_ids = get_reserved_ids(node);
+    check_out_of_range_ids(node, reserved_ids);
+    for (const auto& field : node.fields()) {
+      ctx_.check(
+          reserved_ids.count(field.id()) == 0,
+          "Fields in {} cannot use reserved ids: {}",
+          node.name(),
+          field.id());
+    }
+  }
+
+  void check(const t_enum& node) {
+    auto reserved_ids = get_reserved_ids(node);
+    for (const auto& enum_value : node.values()) {
+      ctx_.check(
+          reserved_ids.count(enum_value.get_value()) == 0,
+          "Enum values in {} cannot use reserved ids: {}",
+          node.name(),
+          enum_value.get_value());
+    }
+  }
+
+ private:
+  diagnostic_context& ctx_;
+
+  // Gets all the reserved ids annotated on this node. Returns
+  // empty set if the annotation is not present.
+  std::unordered_set<int32_t> get_reserved_ids(const t_type& node) {
+    std::unordered_set<int32_t> reserved_ids;
+
+    auto* annotation = node.find_structured_annotation_or_null(kReserveIdsUri);
+    if (annotation == nullptr) {
+      return reserved_ids;
+    }
+
+    // Take the union of the list of tag values in `ids` and the range of
+    // of values from `id_ranges`
+    if (auto ids =
+            annotation->get_value_from_structured_annotation_or_null("ids");
+        ids != nullptr) {
+      ctx_.check(
+          ids->get_type() == t_const_value::t_const_value_type::CV_LIST,
+          "Field ids must be a list of integers, annotated on {}",
+          node.name());
+      for (const auto* id : ids->get_list()) {
+        ctx_.check(
+            id->get_type() == t_const_value::t_const_value_type::CV_INTEGER,
+            "Field ids must be a list of integers, annotated on {}",
+            node.name());
+        reserved_ids.insert(id->get_integer());
+      }
+    }
+    if (auto id_ranges =
+            annotation->get_value_from_structured_annotation_or_null(
+                "id_ranges");
+        id_ranges != nullptr) {
+      ctx_.check(
+          id_ranges->get_type() == t_const_value::t_const_value_type::CV_MAP,
+          "Field id_ranges must be a map of integer to integer, annotated on {}",
+          node.name());
+      for (const auto& [id_range_begin, id_range_end] : id_ranges->get_map()) {
+        ctx_.check(
+            id_range_begin->get_type() ==
+                    t_const_value::t_const_value_type::CV_INTEGER &&
+                id_range_begin->get_type() ==
+                    t_const_value::t_const_value_type::CV_INTEGER,
+            "Field id_ranges must be a map of integer to integer, annotated on {}",
+            node.name());
+        ctx_.check(
+            id_range_begin->get_integer() < id_range_end->get_integer(),
+            "For each (start: end) in id_ranges, we must have start < end. Got ({}: {}), annotated on {}",
+            id_range_begin->get_integer(),
+            id_range_end->get_integer(),
+            node.name());
+        for (int i = id_range_begin->get_integer();
+             i <= id_range_end->get_integer();
+             ++i) {
+          reserved_ids.insert(i);
+        }
+      }
+    }
+    return reserved_ids;
+  }
+
+  void check_out_of_range_ids(
+      const t_structured& node,
+      const std::unordered_set<int32_t>& reserved_ids) {
+    // Insert into std::set to make sure error message is deterministic
+    std::set<int32_t> out_of_range_ids;
+    for (auto id : reserved_ids) {
+      if (id < std::numeric_limits<std::int16_t>::min() ||
+          std::numeric_limits<std::int16_t>::max() < id) {
+        out_of_range_ids.insert(id);
+      }
+    }
+    for (auto id : out_of_range_ids) {
+      ctx_.error(
+          "Struct `{}` cannot have reserved id that is out of range: {}",
+          node.name(),
+          id);
+    }
+  }
+};
+
+void validate_reserved_ids_structured(
+    diagnostic_context& ctx, const t_structured& node) {
+  reserved_ids_checker(ctx).check(node);
+}
+
+void validate_reserved_ids_enum(diagnostic_context& ctx, const t_enum& node) {
+  reserved_ids_checker(ctx).check(node);
+}
 } // namespace
 
 ast_validator standard_validator() {
@@ -759,6 +888,8 @@ ast_validator standard_validator() {
   validator.add_structured_definition_visitor(
       &validate_compatibility_with_lazy_field);
   validator.add_structured_definition_visitor(&validate_box_annotation);
+  validator.add_structured_definition_visitor(
+      &validate_reserved_ids_structured);
   validator.add_union_visitor(&validate_union_field_attributes);
   validator.add_exception_visitor(&validate_exception_php_annotations);
   validator.add_field_visitor(&validate_field_id);
@@ -770,9 +901,11 @@ ast_validator standard_validator() {
   validator.add_field_visitor(&validate_ref_unique_and_box_annotation);
   validator.add_field_visitor(&validate_cpp_field_adapter_annotation);
   validator.add_field_visitor(&validate_hack_field_adapter_annotation);
+  validator.add_field_visitor(&validate_cpp_field_interceptor_annotation);
 
   validator.add_enum_visitor(&validate_enum_value_name_uniqueness);
   validator.add_enum_visitor(&validate_enum_value_uniqueness);
+  validator.add_enum_visitor(&validate_reserved_ids_enum);
   validator.add_enum_value_visitor(&validate_enum_value);
 
   validator.add_definition_visitor(&validate_structured_annotation);
