@@ -45,7 +45,7 @@ using ::testing::UnorderedElementsAre;
 class StandardValidatorTest : public ::testing::Test {
  public:
   StandardValidatorTest()
-      : loc(source_mgr.add_string("/path/to/file.thrift", "").start) {}
+      : loc(source_mgr.add_string("/path/to/file.thrift", "\n\n\n\n").start) {}
 
  protected:
   std::vector<diagnostic> validate(
@@ -66,11 +66,11 @@ class StandardValidatorTest : public ::testing::Test {
     return result;
   }
 
-  diagnostic failure(int lineno, const std::string& msg) {
-    return {diagnostic_level::failure, msg, "/path/to/file.thrift", lineno};
+  diagnostic error(int lineno, const std::string& msg) {
+    return {diagnostic_level::error, msg, "/path/to/file.thrift", lineno};
   }
-  diagnostic failure(const std::string& msg) {
-    return {diagnostic_level::failure, msg, "", 0};
+  diagnostic error(const std::string& msg) {
+    return {diagnostic_level::error, msg, "", 0};
   }
 
   diagnostic warning(int lineno, const std::string& msg) {
@@ -118,14 +118,12 @@ TEST_F(StandardValidatorTest, InterfaceNamesUniqueNoError) {
 TEST_F(StandardValidatorTest, BadPriority) {
   {
     auto service = std::make_unique<t_service>(&program_, "Service");
-    service->set_lineno(1);
     service->set_annotation("priority", "bad1");
     auto fn = std::make_unique<t_function>(
         &t_base_type::t_void(),
         "foo",
         std::make_unique<t_paramlist>(&program_));
     fn->set_annotation("priority", "bad2");
-    fn->set_lineno(2);
     service->add_function(std::move(fn));
     program_.add_service(std::move(service));
   }
@@ -133,14 +131,12 @@ TEST_F(StandardValidatorTest, BadPriority) {
   {
     auto interaction =
         std::make_unique<t_interaction>(&program_, "Interaction");
-    interaction->set_lineno(3);
     interaction->set_annotation("priority", "bad3");
     auto fn = std::make_unique<t_function>(
         &t_base_type::t_void(),
         "foo",
         std::make_unique<t_paramlist>(&program_));
     fn->set_annotation("priority", "bad4");
-    fn->set_lineno(4);
     interaction->add_function(std::move(fn));
     program_.add_interaction(std::move(interaction));
   }
@@ -148,18 +144,14 @@ TEST_F(StandardValidatorTest, BadPriority) {
   EXPECT_THAT(
       validate(),
       ::testing::UnorderedElementsAre(
-          failure(
-              1,
-              "Bad priority 'bad1'. Choose one of 'HIGH_IMPORTANT','HIGH','IMPORTANT','NORMAL','BEST_EFFORT'."),
-          failure(
-              2,
-              "Bad priority 'bad2'. Choose one of 'HIGH_IMPORTANT','HIGH','IMPORTANT','NORMAL','BEST_EFFORT'."),
-          failure(
-              3,
-              "Bad priority 'bad3'. Choose one of 'HIGH_IMPORTANT','HIGH','IMPORTANT','NORMAL','BEST_EFFORT'."),
-          failure(
-              4,
-              "Bad priority 'bad4'. Choose one of 'HIGH_IMPORTANT','HIGH','IMPORTANT','NORMAL','BEST_EFFORT'.")));
+          error(
+              "Bad priority 'bad1'. Choose one of [\"HIGH_IMPORTANT\", \"HIGH\", \"IMPORTANT\", \"NORMAL\", \"BEST_EFFORT\"]."),
+          error(
+              "Bad priority 'bad2'. Choose one of [\"HIGH_IMPORTANT\", \"HIGH\", \"IMPORTANT\", \"NORMAL\", \"BEST_EFFORT\"]."),
+          error(
+              "Bad priority 'bad3'. Choose one of [\"HIGH_IMPORTANT\", \"HIGH\", \"IMPORTANT\", \"NORMAL\", \"BEST_EFFORT\"]."),
+          error(
+              "Bad priority 'bad4'. Choose one of [\"HIGH_IMPORTANT\", \"HIGH\", \"IMPORTANT\", \"NORMAL\", \"BEST_EFFORT\"].")));
 }
 
 TEST_F(StandardValidatorTest, ReapeatedNamesInService) {
@@ -175,7 +167,7 @@ TEST_F(StandardValidatorTest, ReapeatedNamesInService) {
         &t_base_type::t_void(),
         "foo",
         std::make_unique<t_paramlist>(&program_));
-    fn2->set_lineno(2);
+    fn2->set_src_range({loc + 1, loc + 1});
     service->add_function(std::move(fn1));
     service->add_function(std::move(fn2));
     service->add_function(std::make_unique<t_function>(
@@ -192,12 +184,12 @@ TEST_F(StandardValidatorTest, ReapeatedNamesInService) {
         &t_base_type::t_void(),
         "bar",
         std::make_unique<t_paramlist>(&program_));
-    fn1->set_lineno(3);
+    fn1->set_src_range({loc + 2, loc + 2});
     auto fn2 = std::make_unique<t_function>(
         &t_base_type::t_void(),
         "bar",
         std::make_unique<t_paramlist>(&program_));
-    fn2->set_lineno(4);
+    fn2->set_src_range({loc + 3, loc + 3});
 
     interaction->add_function(std::make_unique<t_function>(
         &t_base_type::t_void(),
@@ -211,8 +203,8 @@ TEST_F(StandardValidatorTest, ReapeatedNamesInService) {
   EXPECT_THAT(
       validate(),
       ::testing::UnorderedElementsAre(
-          failure(2, "Function `foo` is already defined for `Service`."),
-          failure(4, "Function `bar` is already defined for `Interaction`.")));
+          error(2, "Function `foo` is already defined for `Service`."),
+          error(4, "Function `bar` is already defined for `Interaction`.")));
 }
 
 TEST_F(StandardValidatorTest, RepeatedNameInExtendedService) {
@@ -235,13 +227,13 @@ TEST_F(StandardValidatorTest, RepeatedNameInExtendedService) {
   // Add an overlapping function in the derived service.
   auto dupe = std::make_unique<t_function>(
       &t_base_type::t_void(), "baz", std::make_unique<t_paramlist>(&program_));
-  dupe->set_lineno(1);
+  dupe->set_src_range({loc, loc});
   derived_ptr->add_function(std::move(dupe));
 
   // An error will be found
   EXPECT_THAT(
       validate(),
-      ::testing::UnorderedElementsAre(failure(
+      ::testing::UnorderedElementsAre(error(
           1, "Function `Derived.baz` redefines `service file.Base.baz`.")));
 }
 
@@ -264,7 +256,7 @@ TEST_F(StandardValidatorTest, DuplicatedEnumValues) {
   EXPECT_THAT(
       validate(),
       UnorderedElementsAre(
-          failure("Duplicate value `foo=1` with value `bar` in enum `foo`.")));
+          error("Duplicate value `foo=1` with value `bar` in enum `foo`.")));
 }
 
 TEST_F(StandardValidatorTest, RepeatedNamesInEnumValues) {
@@ -280,14 +272,14 @@ TEST_F(StandardValidatorTest, RepeatedNamesInEnumValues) {
 
   // Add enum_value with repeated name.
   auto enum_value_3 = std::make_unique<t_enum_value>("bar", 3);
-  enum_value_3->set_lineno(1);
+  enum_value_3->set_src_range({loc, loc});
   tenum_ptr->append(std::move(enum_value_3));
 
   // An error will be found.
   EXPECT_THAT(
       validate(),
       UnorderedElementsAre(
-          failure(1, "Enum value `bar` is already defined for `foo`.")));
+          error(1, "Enum value `bar` is already defined for `foo`.")));
 }
 
 TEST_F(StandardValidatorTest, UnsetEnumValues) {
@@ -298,9 +290,9 @@ TEST_F(StandardValidatorTest, UnsetEnumValues) {
   auto enum_value_1 = std::make_unique<t_enum_value>("Foo", 1);
   auto enum_value_2 = std::make_unique<t_enum_value>("Bar");
   auto enum_value_3 = std::make_unique<t_enum_value>("Baz");
-  enum_value_1->set_lineno(1);
-  enum_value_2->set_lineno(2);
-  enum_value_3->set_lineno(3);
+  enum_value_1->set_src_range({loc, loc});
+  enum_value_2->set_src_range({loc + 1, loc + 1});
+  enum_value_3->set_src_range({loc + 2, loc + 2});
   tenum_ptr->append(std::move(enum_value_1));
   tenum_ptr->append(std::move(enum_value_2));
   tenum_ptr->append(std::move(enum_value_3));
@@ -309,10 +301,10 @@ TEST_F(StandardValidatorTest, UnsetEnumValues) {
   EXPECT_THAT(
       validate(),
       UnorderedElementsAre(
-          failure(
+          error(
               2,
               "The enum value, `Bar`, must have an explicitly assigned value."),
-          failure(
+          error(
               3,
               "The enum value, `Baz`, must have an explicitly assigned value.")));
 }
@@ -320,27 +312,27 @@ TEST_F(StandardValidatorTest, UnsetEnumValues) {
 TEST_F(StandardValidatorTest, UnionFieldAttributes) {
   auto tstruct = std::make_unique<t_struct>(&program_, "Struct");
   auto tunion = std::make_unique<t_union>(&program_, "Union");
-  tunion->set_lineno(1);
+  tunion->set_src_range({loc, loc});
   {
     auto field = std::make_unique<t_field>(&t_base_type::t_i64(), "req", 1);
-    field->set_lineno(2);
+    field->set_src_range({loc + 1, loc + 1});
     field->set_qualifier(t_field_qualifier::required);
     tunion->append(std::move(field));
   }
   {
     auto field = std::make_unique<t_field>(&t_base_type::t_i64(), "op", 2);
-    field->set_lineno(3);
+    field->set_src_range({loc + 2, loc + 2});
     field->set_qualifier(t_field_qualifier::optional);
     tunion->append(std::move(field));
   }
   {
     auto field = std::make_unique<t_field>(&t_base_type::t_i64(), "non", 3);
-    field->set_lineno(4);
+    field->set_src_range({loc + 3, loc + 3});
     tunion->append(std::move(field));
   }
   {
     auto field = std::make_unique<t_field>(tstruct.get(), "mixin", 4);
-    field->set_lineno(5);
+    field->set_src_range({loc + 4, loc + 4});
     field->set_annotation("cpp.mixin");
     tunion->append(std::move(field));
   }
@@ -351,14 +343,14 @@ TEST_F(StandardValidatorTest, UnionFieldAttributes) {
       validate(),
       UnorderedElementsAre(
           // Qualified fields will have errors.
-          failure(
+          error(
               2,
               "Unions cannot contain qualified fields. Remove `required` qualifier from field `req`."),
-          failure(
+          error(
               3,
               "Unions cannot contain qualified fields. Remove `optional` qualifier from field `op`."),
           // Fields with cpp.mixing have errors.
-          failure(5, "Union `Union` cannot contain mixin field `mixin`.")));
+          error(5, "Union `Union` cannot contain mixin field `mixin`.")));
 }
 
 TEST_F(StandardValidatorTest, FieldId) {
@@ -378,7 +370,7 @@ TEST_F(StandardValidatorTest, FieldId) {
   EXPECT_THAT(
       validate(),
       UnorderedElementsAre(
-          failure(-1, "Zero value (0) not allowed as a field id for `zero_id`"),
+          error("Zero value (0) not allowed as a field id for `zero_id`"),
           warning(
               1,
               "No field id specified for `implicit_id`, resulting protocol may have conflicts or not be backwards compatible!")));
@@ -392,7 +384,7 @@ TEST_F(StandardValidatorTest, MixinFieldType) {
   auto foo = std::make_unique<t_struct>(&program_, "Foo");
   {
     auto field = std::make_unique<t_field>(tstruct.get(), "struct_field", 1);
-    field->set_lineno(1);
+    field->set_src_range({loc, loc});
     field->set_annotation("cpp.mixin");
     field->set_qualifier(t_field_qualifier::optional);
     foo->append(std::move(field));
@@ -421,12 +413,12 @@ TEST_F(StandardValidatorTest, MixinFieldType) {
   program_.add_struct(std::move(foo));
 
   EXPECT_THAT(
-      validate(diagnostic_params::only_failures()),
+      validate(diagnostic_params::only_errors()),
       UnorderedElementsAre(
-          failure(1, "Mixin field `struct_field` cannot be optional."),
-          failure(
+          error(1, "Mixin field `struct_field` cannot be optional."),
+          error(
               "Mixin field `except_field` type must be a struct or union. Found `Exception`."),
-          failure(
+          error(
               "Mixin field `other_field` type must be a struct or union. Found `i32`.")));
 }
 
@@ -443,16 +435,18 @@ TEST_F(StandardValidatorTest, RepeatedStructuredAnnotation) {
       &program_, &t_base_type::t_i32(), "Bar", &scope);
   bar->add_structured_annotation(inst(other_foo.get(), 1));
   bar->add_structured_annotation(inst(foo.get(), 2));
-  bar->add_structured_annotation(inst(foo.get(), 3));
+  auto annot = inst(foo.get(), 3);
+  annot->set_src_range({loc + 2, loc + 2});
+  bar->add_structured_annotation(std::move(annot));
 
-  program_.add_struct(std::move(foo));
-  program_.add_typedef(std::move(bar));
-  other_program.add_struct(std::move(other_foo));
+  program_.add_def(std::move(foo));
+  program_.add_def(std::move(bar));
+  other_program.add_def(std::move(other_foo));
 
   // Only the third annotation is a duplicate.
   EXPECT_THAT(
-      validate(diagnostic_params::only_failures()),
-      UnorderedElementsAre(failure(
+      validate(diagnostic_params::only_errors()),
+      UnorderedElementsAre(error(
           3, "Structured annotation `Foo` is already defined for `Bar`.")));
 }
 
@@ -504,19 +498,19 @@ TEST_F(StandardValidatorTest, CustomDefaultValue) {
   EXPECT_THAT(
       validate(),
       UnorderedElementsAre(
-          failure(
+          error(
               1,
               "value error: const `const_byte` has an invalid custom default value."),
-          failure(
+          error(
               1,
               "value error: const `const_short` has an invalid custom default value."),
-          failure(
+          error(
               1,
               "value error: const `const_integer` has an invalid custom default value."),
-          failure(
+          error(
               1,
               "value error: const `const_float` has an invalid custom default value."),
-          failure(
+          error(
               1,
               "value error: const `const_float_precision_loss` cannot be represented precisely as `float` or `double`.")));
 }
