@@ -22,6 +22,7 @@ import shutil
 import subprocess
 import sys
 
+import pkg_resources
 
 """
 * Invoke from the `/fbsource/fbcode/` directory using `buck run`:
@@ -47,7 +48,6 @@ to the current working directory) and $FIXTURENAMES is a list of
 fixture names to build specifically (default is to build all fixtures).
 """
 FIXTURE_ROOT = "."
-THRIFT_REL = "thrift/compiler/thrift"
 
 
 def parsed_args():
@@ -60,7 +60,7 @@ def parsed_args():
             "For relative path, we will search script's path and all parents"
         ),
         type=str,
-        default=THRIFT_REL,
+        default=None,
     )
     parser.add_argument(
         "--fixture-root",
@@ -104,8 +104,11 @@ def read_lines(path):
 args = parsed_args()
 fixture_root = args.fixture_root
 exe = os.path.join(os.getcwd(), sys.argv[0])
-thrift = ascend_find_exe(exe, args.thrift_bin)
-
+thrift = args.thrift_bin
+if args.thrift_bin is None:
+    thrift = pkg_resources.resource_filename(__name__, "thrift")
+else:
+    thrift = ascend_find_exe(exe, args.thrift_bin)
 if thrift is None:
     tb = args.thrift_bin
     sys.stderr.write("error: cannot find the Thrift compiler ({})\n".format(tb))
