@@ -1,5 +1,5 @@
 #!/usr/bin/env python3
-# Copyright (c) Facebook, Inc. and its affiliates.
+# Copyright (c) Meta Platforms, Inc. and affiliates.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -24,8 +24,9 @@ import tempfile
 import traceback
 import unittest
 
+import pkg_resources
+
 FIXTURE_ROOT = "."
-THRIFT_REL = "thrift/compiler/thrift"
 
 
 def ascend_find_exe(path, target):
@@ -42,7 +43,8 @@ def ascend_find_exe(path, target):
 
 
 exe = os.path.join(os.getcwd(), sys.argv[0])
-thrift = ascend_find_exe(exe, THRIFT_REL)
+thrift = pkg_resources.resource_filename(__name__, "thrift")
+assert thrift
 fixtures_root_dir = os.path.join(FIXTURE_ROOT, "thrift/compiler/test/fixtures")
 
 
@@ -173,7 +175,6 @@ class CompilerTest(unittest.TestCase):
                 or ("schema" == lang)
                 or ("mstch_cpp2" == lang)
                 or ("mstch_java" == lang)
-                or ("mstch_java2" == lang)
             ):
                 args.remove("-r")
 
